@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -677,7 +678,7 @@ public:
   // concatenated this means we'll add up with several display lists for normal
   // drawing operations.
   virtual void drawVideoFrame(GnashImage* frame, const SWFMatrix* m,
-          const rect* bounds, bool /*smooth*/)
+          const SWFRect* bounds, bool /*smooth*/)
   {
     GLint index;
 
@@ -708,7 +709,7 @@ public:
     _render_indices.push_back(index);
   }
   
-  virtual void reallyDrawVideoFrame(GnashImage* frame, const SWFMatrix* m, const rect* bounds)
+  virtual void reallyDrawVideoFrame(GnashImage* frame, const SWFMatrix* m, const SWFRect* bounds)
   {
   
     if (frame->type() == GNASH_IMAGE_RGBA)
@@ -746,8 +747,8 @@ public:
 
     size_t height = frame->height();
     size_t width = frame->width();
-    float zx = w_bounds / (float) width;
-    float zy = h_bounds / (float) height;
+    float zx = w_bounds / static_cast<float>(width);
+    float zy = h_bounds / static_cast<float>(height);
     glPixelZoom(zx,  -zy);  // flip & zoom image
     glDrawPixels(width, height, GL_RGB, GL_UNSIGNED_BYTE, ptr);
 
@@ -762,7 +763,7 @@ public:
 
   // FIXME
   geometry::Range2d<int>
-  world_to_pixel(const rect& worldbounds)
+  world_to_pixel(const SWFRect& worldbounds)
   {
     // TODO: verify this is correct
     geometry::Range2d<int> ret(worldbounds.getRange());
@@ -789,8 +790,13 @@ public:
 
     gluOrtho2D(x0, x1, y0, y1);
     
-    _width  = fabsf(x1 - x0);
-    _height = fabsf(y1 - y0);
+    _width  = std::fabs(x1 - x0);
+    _height = std::fabs(y1 - y0);
+
+    glScalef(static_cast<float>(twipsToPixels(_width)) /
+    static_cast<float>(viewport_width),
+    static_cast<float>(twipsToPixels(_height)) / 
+    static_cast<float>(viewport_height), 1.0f);
 
     // Setup the clear color. The actual clearing will happen in end_display.
     if (bg_color.m_a) {
@@ -1013,7 +1019,7 @@ public:
     
     dummy_fs.push_back(coloring);
     
-    std::vector<line_style> dummy_ls;
+    std::vector<LineStyle> dummy_ls;
     
     draw_subshape(path_vec, SWFMatrix(), dummy_cx, dummy_fs, dummy_ls);
   }
@@ -1272,7 +1278,7 @@ public:
   
   
   
-  bool apply_line_style(const line_style& style, const cxform& cx, const SWFMatrix& mat)
+  bool apply_line_style(const LineStyle& style, const cxform& cx, const SWFMatrix& mat)
   {
   //  GNASH_REPORT_FUNCTION;
      
@@ -1304,15 +1310,15 @@ public:
          LOG_ONCE( log_unimpl(_("Unidirectionally scaled strokes in OGL renderer")) );
       }
       
-      float stroke_scale = fabsf(mat.get_x_scale()) + fabsf(mat.get_y_scale());
+      float stroke_scale = std::fabs(mat.get_x_scale()) + std::fabs(mat.get_y_scale());
       stroke_scale /= 2.0f;
-      stroke_scale *= (fabsf(_xscale) + fabsf(_yscale)) / 2.0f;
+      stroke_scale *= (std::fabs(_xscale) + std::fabs(_yscale)) / 2.0f;
       width *= stroke_scale;
       width = twipsToPixels(width);
 
       GLfloat width_info[2];
       
-      glGetFloatv( GL_LINE_WIDTH_RANGE, width_info);          
+      glGetFloatv(GL_LINE_WIDTH_RANGE, width_info);          
       
       if (width > width_info[1]) {
         LOG_ONCE( log_unimpl("Your OpenGL implementation does not support the line width" \
@@ -1374,7 +1380,7 @@ public:
   void
   draw_outlines(const PathVec& path_vec, const PathPointMap& pathpoints, const SWFMatrix& mat,
                 const cxform& cx, const std::vector<fill_style>& fill_styles,
-                const std::vector<line_style>& line_styles)
+                const std::vector<LineStyle>& line_styles)
   {
   
     for (PathVec::const_iterator it = path_vec.begin(), end = path_vec.end();
@@ -1538,7 +1544,7 @@ public:
     const SWFMatrix& mat,
     const cxform& cx,
     const std::vector<fill_style>& fill_styles,
-    const std::vector<line_style>& line_styles)
+    const std::vector<LineStyle>& line_styles)
   {
     PathVec normalized = normalize_paths(path_vec);
     PathPointMap pathpoints = getPathPoints(normalized);
@@ -1589,6 +1595,8 @@ public:
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
       }
       
+      glDisable(GL_TEXTURE_GEN_S);
+      glDisable(GL_TEXTURE_GEN_T);    
       glDisable(GL_TEXTURE_1D);
       glDisable(GL_TEXTURE_2D);      
     }
@@ -1641,7 +1649,7 @@ public:
     std::vector<PathVec::const_iterator> subshapes = find_subshapes(path_vec);
     
     const std::vector<fill_style>& fill_styles = shape.fillStyles();
-    const std::vector<line_style>& line_styles = shape.lineStyles();
+    const std::vector<LineStyle>& line_styles = shape.lineStyles();
     
     for (size_t i = 0; i < subshapes.size()-1; ++i) {
       PathVec subshape_paths;
@@ -1669,7 +1677,7 @@ public:
     
     glyph_fs.push_back(coloring);
     
-    std::vector<line_style> dummy_ls;
+    std::vector<LineStyle> dummy_ls;
     
     oglScopeMatrix scope_mat(mat);
     
