@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -50,7 +51,8 @@ public:
 		_timeout=to;
 	}
 	bool init(int, char ***) { return true; }
-	bool createWindow(const char* /*title*/, int /*width*/, int /*height*/)
+	bool createWindow(const char* /*title*/, int /*width*/, int /*height*/,
+	                  int /*yPosition*/, int /*xPosition*/)
 	{
 		return true;
 	}
@@ -59,7 +61,7 @@ public:
 	bool setupEvents()  { return true; }
 	void renderBuffer()  { }
 
-	void quit() { _quit = true; }
+	void quitUI() { _quit = true; }
 
 private:
 
