@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,6 +20,11 @@
 #ifndef AQUA_GLUE_H
 #define AQUA_GLUE_H
 
+#ifdef HAVE_CONFIG_H
+#include "gnashconfig.h"
+#endif
+
+
 #include "gnash.h"
 #include "snappingrange.h"
 #include <AGL/agl.h>
@@ -32,7 +38,7 @@ class AquaGlue
     virtual ~AquaGlue() { }
     virtual bool init(int argc, char **argv[]) = 0;
     virtual bool prepDrawingArea(int width, int height, AGLDrawable drawable) = 0;
-    virtual render_handler* createRenderHandler() = 0;
+    virtual Renderer* createRenderHandler() = 0;
     virtual void render() = 0;
   protected:
     int _bpp;
