@@ -1,5 +1,6 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -291,26 +292,28 @@ bool FBGui::init(int /*argc*/, char *** /*argv*/)
 }
 
 bool FBGui::initialize_renderer() {
-  int _width    = var_screeninfo.xres;
-  int _height   = var_screeninfo.yres;
-  int _bpp = var_screeninfo.bits_per_pixel;
-  int _size = fix_screeninfo.smem_len;   // TODO: should recalculate!  
-  unsigned char *_mem;
-  Renderer_agg_base *agg_handler;
+  const int width    = var_screeninfo.xres;
+  const int height   = var_screeninfo.yres;
+  const int bpp = var_screeninfo.bits_per_pixel;
+  const int size = fix_screeninfo.smem_len; 
+
+  // TODO: should recalculate!  
+  unsigned char* mem;
+  Renderer_agg_base* agg_handler;
   
-  m_stage_width = _width;
-  m_stage_height = _height;
+  m_stage_width = width;
+  m_stage_height = height;
   
-  _validbounds.setTo(0, 0, _width-1, _height-1);
+  _validbounds.setTo(0, 0, width - 1, height - 1);
     
   
-  #ifdef DOUBLE_BUFFER
+#ifdef DOUBLE_BUFFER
   log_debug(_("Double buffering enabled"));
-  _mem = buffer;
-  #else
+  mem = buffer;
+#else
   log_debug(_("Double buffering disabled"));
-  _mem = fbmem;
-  #endif
+  mem = fbmem;
+#endif
   
   
   agg_handler = NULL;
@@ -329,7 +332,7 @@ bool FBGui::initialize_renderer() {
     var_screeninfo.red.offset, var_screeninfo.red.length,
     var_screeninfo.green.offset, var_screeninfo.green.length,
     var_screeninfo.blue.offset, var_screeninfo.blue.length,
-    _bpp
+    bpp
   );
 
   if (pixelformat) {    
@@ -339,14 +342,15 @@ bool FBGui::initialize_renderer() {
     return false;
   }
 
-  assert(agg_handler!=NULL);
+  assert(agg_handler);
+
   _renderer.reset(agg_handler);
 
-  _runResources.setRenderer(boost::shared_ptr<Renderer>(_renderer));
+  _runResources.setRenderer(_renderer);
   
-  m_rowsize = var_screeninfo.xres_virtual*((_bpp+7)/8);
+  m_rowsize = var_screeninfo.xres_virtual*((bpp+7)/8);
   
-  agg_handler->init_buffer(_mem, _size, _width, _height, m_rowsize);
+  agg_handler->init_buffer(mem, size, width, height, m_rowsize);
   
   disable_terminal();
 
@@ -360,7 +364,8 @@ bool FBGui::run()
   double start_timer;
   
   if (!gettimeofday(&tv, NULL))
-    start_timer = (double)tv.tv_sec + (double)tv.tv_usec / 1000000.0;
+    start_timer = static_cast<double>(tv.tv_sec) +
+    static_cast<double>(tv.tv_usec) / 1000000.0;
   else
     start_timer = 0.0;
     
@@ -428,7 +433,8 @@ void FBGui::renderBuffer()
 #endif
 }
 
-bool FBGui::createWindow(const char* /*title*/, int /*width*/, int /*height*/)
+bool FBGui::createWindow(const char* /*title*/, int /*width*/, int /*height*/,
+                     int /*xPosition*/, int /*yPosition*/)
 {
   // Framebuffer has no windows... :-)
 
@@ -891,7 +897,7 @@ bool FBGui::check_mouse()
     // button
     if (btn != mouse_btn) {
       mouse_btn = btn;
-      
+printf("clicked: %d\n", btn);      
       notify_mouse_clicked(btn, 1);  // mark=??
       //log_debug(_("mouse click! %d"), btn);
     }    
@@ -983,8 +989,10 @@ bool FBGui::check_mouse()
     */
     
     
-    new_x = (int)(((double)new_x - 355) / (1702 - 355) * 1536 + 256);
-    new_y = (int)(((double)new_y - 482) / (1771 - 482) * 1536 + 256);
+    new_x = static_cast<int>(((static_cast<double>(new_x )- 355) / (1702 - 355)
+    * 1536 + 256));
+    new_y = static_cast<int>(((static_cast<double>(new_y) - 482) / (1771 - 482)
+    * 1536 + 256));
     
     
     new_x = new_x * m_stage_width / 2048;
@@ -999,6 +1007,7 @@ bool FBGui::check_mouse()
     
     if (new_btn != mouse_btn) {
       mouse_btn = new_btn;      
+printf("clicked: %d\n", mouse_btn);      
       notify_mouse_clicked(mouse_btn, 1);  // mask=?
       activity = true;
     }
@@ -1164,9 +1173,8 @@ bool FBGui::check_mouse()
   // Assuming we will never read less than one full struct...  
   
   while ((loops++ < 100) && (read(input_fd, &ev, sizeof ev) == (sizeof ev))) {
-  
+
     if (ev.type == EV_SYN) {    // synchronize (apply information)
-    
       if ((new_mouse_x != mouse_x) || (new_mouse_y != mouse_y)) {
       
         mouse_x = new_mouse_x;
@@ -1201,7 +1209,7 @@ bool FBGui::check_mouse()
       }
 
       if (coordinatedebug)
-        printf("% 5d / % 5d / % 5d\n", mouse_x, mouse_y, mouse_btn);
+        printf("DEBUG: % 5d / % 5d / % 5d\n", mouse_x, mouse_y, mouse_btn);
       
     }
   
@@ -1215,6 +1223,7 @@ bool FBGui::check_mouse()
     if (ev.type == EV_ABS) {    // absolute coordinate
       if (ev.code == ABS_X) new_mouse_x = ev.value;
       if (ev.code == ABS_Y) new_mouse_y = ev.value;
+      if (ev.code == ABS_PRESSURE) new_mouse_btn = ev.value >= 128;
     }
     
     if (ev.type == EV_REL) {    // relative movement
@@ -1347,7 +1356,7 @@ gnash::key::code FBGui::scancode_to_gnash_key(int code, bool shift) {
     case KEY_KPMINUS       : return gnash::key::KP_SUBTRACT;
     case KEY_KPPLUS        : return gnash::key::KP_ADD;
     case KEY_KPDOT         : return gnash::key::KP_DECIMAL;
-    case KEY_KPASTERISK    : return gnash::key::KP_MULITPLY;  // typo in GnashKey.h
+    case KEY_KPASTERISK    : return gnash::key::KP_MULTIPLY;
     case KEY_KPENTER       : return gnash::key::KP_ENTER;
     
     case KEY_ESC           : return gnash::key::ESCAPE;
