@@ -1,5 +1,6 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,8 +29,8 @@
 #include "log.h"
 #include "gui.h"
 
-#include "render_handler.h"
-#include "render_handler_agg.h"
+#include "Renderer.h"
+#include "Renderer_agg.h"
 
 using namespace fltk;
 
@@ -42,7 +43,7 @@ class FltkAggGlue : public fltk::Widget
       ~FltkAggGlue();
      // resize(int width, int height);
       void draw();
-      render_handler* createRenderHandler();
+      Renderer* createRenderHandler();
       void initBuffer(int width, int height);
       void resize(int width, int height);
       void render(geometry::Range2d<int>& bounds);
@@ -53,7 +54,7 @@ class FltkAggGlue : public fltk::Widget
       int _height;
       int _stride;
       unsigned char* _offscreenbuf;
-      render_handler* _renderer;
+      Renderer* _renderer;
       //Rectangle _bounds;
       geometry::Range2d<int> _drawbounds;
       geometry::Range2d<int> _validbounds;
