@@ -1,6 +1,7 @@
 // gui.cpp:  Top level GUI for SWF player, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,24 +24,25 @@
 
 #include "MovieClip.h"
 #include "gui.h"
-#include "render.h"  // debug
-#include "render_handler.h"
+#include "Renderer.h"
 #include "sound_handler.h"
 #include "movie_root.h"
 #include "VM.h"
+#include "DisplayObject.h"
+#include "tu_file.h"
+#include "gnash.h"
 
 #ifdef GNASH_FPS_DEBUG
 #include "ClockTime.h"
 #include <boost/format.hpp>
 #endif
 
+#include <vector>
 #include <cstdio>
 #include <cstring>
-#include <algorithm> // std::max, std::min
-
-#ifdef SKIP_RENDERING_IF_LATE
-#include "WallClockTimer.h"
-#endif
+#include <algorithm> 
+#include <boost/algorithm/string/replace.hpp>
+#include <boost/lexical_cast.hpp>
 
 /// Define this to make sure each frame is fully rendered from ground up
 /// even if no motion has been detected in the movie.
@@ -69,17 +71,22 @@
 //
 //#define DEBUG_MOUSE_COORDINATES 1
 
+// Define the following macro if you want to skip rendering
+// when late on FPS time.
+// This is an experimental feature, so it's off by default
+//#define SKIP_RENDERING_IF_LATE 1
+
+
 
 namespace gnash {
 
-Gui::Gui() :
+Gui::Gui(RunResources& r) :
     _loop(true),
     _xid(0),
     _width(1),
     _height(1),
-    _depth(16),
+    _runResources(r),
     _interval(0),
-    _renderer(NULL),
     _redraw_flag(true),
     _fullscreen(false),
     _mouseShown(true),
@@ -94,15 +101,19 @@ Gui::Gui() :
     ,fps_counter_total(0)
     ,fps_timer(0)
     ,fps_timer_interval(0.0)
+    ,frames_dropped(0)
 #endif
-#ifdef SKIP_RENDERING_IF_LATE
-    ,estimatedDisplayTime(0) // milliseconds (will grow later..)
-#endif // SKIP_RENDERING_IF_LATE
     ,_movieDef(0)
     ,_stage(0)
     ,_stopped(false)
     ,_started(false)
     ,_showUpdatedRegions(false)
+
+    // NOTE: it's important that _systemClock is constructed
+    //       before and destroyed after _virtualClock !
+    ,_systemClock()
+    ,_virtualClock(_systemClock)
+
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
     ,_xpointer(0)
     ,_ypointer(0)
@@ -113,15 +124,14 @@ Gui::Gui() :
 
 }
 
-Gui::Gui(unsigned long xid, float scale, bool loop, unsigned int depth)
+Gui::Gui(unsigned long xid, float scale, bool loop, RunResources& r)
 	:
     _loop(loop),
     _xid(xid),
     _width(1),
     _height(1),
-    _depth(depth),
+    _runResources(r),
     _interval(0),
-    _renderer(NULL),
     _redraw_flag(true),
     _fullscreen(false),
     _mouseShown(true),
@@ -136,15 +146,19 @@ Gui::Gui(unsigned long xid, float scale, bool loop, unsigned int depth)
     ,fps_counter_total(0)    
     ,fps_timer(0)
     ,fps_timer_interval(0.0)
+    ,frames_dropped(0)
 #endif        
-#ifdef SKIP_RENDERING_IF_LATE
-    ,estimatedDisplayTime(0) // milliseconds (will grow later..)
-#endif // SKIP_RENDERING_IF_LATE
     ,_movieDef(0)
     ,_stage(0)
     ,_stopped(false)
     ,_started(false)
     ,_showUpdatedRegions(false)
+
+    // NOTE: it's important that _systemClock is constructed
+    //       before and destroyed after _virtualClock !
+    ,_systemClock()
+    ,_virtualClock(_systemClock)
+
 #ifdef ENABLE_KEYBOARD_MOUSE_MOVEMENTS 
     ,_xpointer(0)
     ,_ypointer(0)
@@ -158,10 +172,10 @@ Gui::~Gui()
 {
     if ( _movieDef.get() ) log_debug("~Gui - _movieDef refcount: %d", _movieDef->get_ref_count());
 
-    delete _renderer;
 #ifdef GNASH_FPS_DEBUG
     if ( fps_timer_interval ) {
-        std::cerr << "Total frame advances: " << fps_counter_total << std::endl;
+        std::cerr << "Total frame advances/drops: "
+                  << fps_counter_total << "/" << frames_dropped << std::endl;
     }
 #endif
 }
@@ -172,12 +186,27 @@ Gui::setFullscreen()
     log_unimpl(_("Fullscreen not yet supported in this GUI"));
 }
 
+void
+Gui::resizeWindow(int /*width*/, int /*height*/)
+{
+    log_unimpl(_("Window resize not yet supported in this GUI"));
+}
+
 void
 Gui::unsetFullscreen()
 {
     log_unimpl(_("Fullscreen not yet supported in this GUI"));
 }
 
+void
+Gui::quit() {
+    // Take a screenshot of the last frame if required.
+    if (_screenShotter.get()) {
+        _screenShotter->last();
+    }
+    quitUI();
+}
+
 void
 Gui::hideMenu()
 {
@@ -200,24 +229,24 @@ Gui::showMenu(bool /* show */)
 void
 Gui::allowScale(bool allow)
 {
-	if ( ! _stage )
-	{
+	if (!_stage) {
 		log_error("Gui::allowScale called before a movie_root was available");
 		return;
 	}
 
-	if ( allow ) _stage->setStageScaleMode(movie_root::showAll);
+	if (allow) _stage->setStageScaleMode(movie_root::showAll);
 	else _stage->setStageScaleMode(movie_root::noScale);
 }
 
 void
 Gui::toggleFullscreen()
 {
+    /// Sends request to Gnash core to change display state.
 	if (_fullscreen) {
-		unsetFullscreen();
+		_stage->setStageDisplayState(movie_root::DISPLAYSTATE_NORMAL);
 	}
 	else {
-		setFullscreen();
+		_stage->setStageDisplayState(movie_root::DISPLAYSTATE_FULLSCREEN);
 	} 
 }
 
@@ -377,7 +406,7 @@ Gui::updateStageMatrix()
 	//scaleMode, valign, halign);
 
 	// TODO: have a generic set_matrix ?
-	if ( _renderer ) {
+	if (_renderer.get()) {
 		_renderer->set_scale(_xscale, _yscale);
 		_renderer->set_translation(_xoffset, _yoffset);
 	}
@@ -426,7 +455,7 @@ Gui::toggleSound()
     assert (_stage);
     // @todo since we registered the sound handler, shouldn't we know
     //       already what it is ?!
-    sound::sound_handler* s = _stage->runInfo().soundHandler();
+    sound::sound_handler* s = _stage->runResources().soundHandler();
 
     if (!s) return;
 
@@ -461,7 +490,7 @@ Gui::notify_mouse_moved(int ux, int uy)
 		display(m);
 	}
     
-	character* activeEntity = m->getActiveEntityUnderPointer();
+	DisplayObject* activeEntity = m->getActiveEntityUnderPointer();
 	if ( activeEntity )
 	{
 		if ( activeEntity->isSelectableTextField() )
@@ -524,23 +553,23 @@ Gui::refreshView()
 void
 Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed) 
 {
-	movie_root* m = _stage;
 
 	/* Handle GUI shortcuts */
 	if (pressed)
 	{
-		if (k == gnash::key::ESCAPE)
-		{
-			if (isFullscreen())
-			{
-				unsetFullscreen();
+		if (k == gnash::key::ESCAPE) {
+			if (isFullscreen()) {
+				_stage->setStageDisplayState(movie_root::DISPLAYSTATE_NORMAL);
 			}
 		}
 		
-		if (modifier & gnash::key::GNASH_MOD_CONTROL)
-		{
-			switch(k)
+		if (modifier & gnash::key::GNASH_MOD_CONTROL) {
+			switch (k)
 			{
+                case gnash::key::o:
+                case gnash::key::O:
+                    takeScreenShot();
+                    break;
 				case gnash::key::r:
 				case gnash::key::R:
 					restart();
@@ -563,31 +592,34 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 				case gnash::key::F:
 					toggleFullscreen();
 					break;
+				case gnash::key::h:
+				case gnash::key::H:
+					showUpdatedRegions(!showUpdatedRegions());
+					break;
 				case gnash::key::MINUS:
 				{
 					// Max interval allowed: 1 second (1FPS)
-					unsigned int ni = std::min(_interval+2, 1000u);
+					const size_t ni = std::min<size_t>(_interval + 2, 1000u);
 					setInterval(ni);
 					break;
                 }
 				case gnash::key::PLUS:
 				{
 					// Min interval allowed: 1/100 second (100FPS)
-					unsigned int ni = std::max(_interval-2, 10u);
+					const size_t ni = std::max<size_t>(_interval - 2, 10u);
 					setInterval(ni);
 					break;
                                 }
 				case gnash::key::EQUALS:
 				{
-					if ( _stage )
-					{
-						float fps = _stage->get_movie_definition()->get_frame_rate();
+					if (_stage) {
+						const float fps = _stage->frameRate();
 						// Min interval allowed: 1/100 second (100FPS)
-						unsigned int ni = 1000.0/fps;
+						const size_t ni = 1000.0/fps;
 						setInterval(ni);
 					}
 					break;
-                                }
+                }
 				default:
 					break;
 			}
@@ -596,16 +628,15 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 			if ( _keyboardMouseMovements )
 			{
 				int step = _keyboardMouseMovementsStep; 
-				if (modifier & gnash::key::GNASH_MOD_SHIFT) step*=5; // x5 if SHIFT is pressed
-				switch(k)
+                // x5 if SHIFT is pressed
+				if (modifier & gnash::key::GNASH_MOD_SHIFT) step *= 5; 
+				switch (k)
 				{
 					case gnash::key::UP:
 					{
 						int newx = _xpointer;
 						int newy = _ypointer-step;
-						//log_debug("(theoretically) From %d,%d to %d,%d (step %d)", _xpointer, _ypointer, newx, newy, step);
 						if ( newy < 0 ) newy=0;
-						//log_debug("From %d,%d to %d,%d", _xpointer, _ypointer, newx, newy);
 						notify_mouse_moved(newx, newy);
 						break;
 					}
@@ -613,9 +644,7 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 					{
 						int newx = _xpointer;
 						int newy = _ypointer+step;
-						//log_debug("(theoretically) From %d,%d to %d,%d (step %d)", _xpointer, _ypointer, newx, newy, step);
 						if ( newy >= _height ) newy = _height-1;
-						//log_debug("From %d,%d to %d,%d", _xpointer, _ypointer, newx, newy);
 						notify_mouse_moved(newx, newy);
 						break;
 					}
@@ -623,19 +652,15 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 					{
 						int newx = _xpointer-step;
 						int newy = _ypointer;
-						//log_debug("(theoretically) From %d,%d to %d,%d (step %d)", _xpointer, _ypointer, newx, newy, step);
 						if ( newx < 0 ) newx = 0;
-						//log_debug("From %d,%d to %d,%d", _xpointer, _ypointer, newx, newy);
 						notify_mouse_moved(newx, newy);
 						break;
 					}
 					case gnash::key::RIGHT:
 					{
-						int newy = _ypointer;
-						int newx = _xpointer+step;
-						//log_debug("(theoretically) From %d,%d to %d,%d (step %d)", _xpointer, _ypointer, newx, newy, step);
+						const int newy = _ypointer;
+						int newx = _xpointer + step;
 						if ( newx >= _width ) newx = _width-1;
-						//log_debug("From %d,%d to %d,%d", _xpointer, _ypointer, newx, newy);
 						notify_mouse_moved(newx, newy);
 						break;
 					}
@@ -647,15 +672,14 @@ Gui::notify_key_event(gnash::key::code k, int modifier, bool pressed)
 		}
 	}
 
-    if ( ! _started ) return;
+    if (!_started) return;
 
-    if ( _stopped ) return;
+    if (_stopped) return;
 
-	if ( m->notify_key_event(k, pressed) )
-	{
+	if (_stage->notify_key_event(k, pressed)) {
 		// any action triggered by the
 		// event required screen refresh
-		display(m);
+		display(_stage);
 	}
 
 }
@@ -701,12 +725,12 @@ Gui::display(movie_root* m)
 		changed_ranges.growBy(40.0f / _xscale);
 		
 		// optimize ranges
-		changed_ranges.combine_ranges();
+		changed_ranges.combineRanges();
 		
 	}
-
-	if (redraw_flag)     // TODO: Remove this and want_redraw to avoid confusion!?
-	{
+    
+    // TODO: Remove this and want_redraw to avoid confusion!?
+	if (redraw_flag)  {
 		changed_ranges.setWorld();
 	}
 	
@@ -771,7 +795,7 @@ Gui::display(movie_root* m)
 		// show invalidated region using a red rectangle
 		// (Flash debug style)
 		IF_DEBUG_REGION_UPDATES (
-		if ( ! changed_ranges.isWorld() )
+		if (_renderer.get() && !changed_ranges.isWorld())
 		{
 		
 			for (size_t rno = 0; rno < changed_ranges.size(); rno++) {
@@ -793,7 +817,7 @@ Gui::display(movie_root* m)
 				corners[3].x = xmin;
 				corners[3].y = ymax;
 				SWFMatrix no_transform;
-				gnash::render::draw_poly(corners, 4,
+				_renderer->draw_poly(corners, 4,
 					rgba(0,0,0,0), rgba(255,0,0,255), no_transform, false);
 					
 			}
@@ -820,8 +844,11 @@ Gui::play()
         assert (_stage);
         // @todo since we registered the sound handler, shouldn't we know
         //       already what it is ?!
-        sound::sound_handler* s = _stage->runInfo().soundHandler();
+        sound::sound_handler* s = _stage->runResources().soundHandler();
         if ( s ) s->unpause();
+
+        log_debug("Starting virtual clock");
+        _virtualClock.resume();
     }
 
     playHook ();
@@ -840,29 +867,37 @@ Gui::stop()
 
     // @todo since we registered the sound handler, shouldn't we know
     //       already what it is ?!
-    sound::sound_handler* s = _stage->runInfo().soundHandler();
+    sound::sound_handler* s = _stage->runResources().soundHandler();
     if ( s ) s->pause();
 
+    log_debug("Pausing virtual clock");
+    _virtualClock.pause();
+
     stopHook();
 }
 
 void
 Gui::pause()
 {
-    if ( _stopped )
-    {
+    if (_stopped) {
         play();
+        return;
     }
-    else
-    {
-        // @todo since we registered the sound handler, shouldn't we know
-        //       already what it is ?!
-    	sound::sound_handler* s = _stage->runInfo().soundHandler();
-    	if ( s ) s->pause();
-        _stopped = true;
 
-        stopHook();
-    }
+    // TODO: call stop() instead ?
+    // The only thing I see is that ::stop exits full-screen,
+    // but I'm not sure that's intended behaviour
+
+    // @todo since we registered the sound handler, shouldn't we know
+    //       already what it is ?!
+    sound::sound_handler* s = _stage->runResources().soundHandler();
+    if (s) s->pause();
+    _stopped = true;
+
+    log_debug("Pausing virtual clock");
+    _virtualClock.pause();
+
+    stopHook();
 }
 
 void
@@ -875,120 +910,141 @@ Gui::start()
         return;
     }
 
-    std::auto_ptr<movie_instance> mr ( _movieDef->create_movie_instance() );
-    mr->setVariables(_flashVars);
-
-    _stage->setRootMovie( mr.release() ); // will construct the instance
-    resize_view(_width, _height); // to properly update stageMatrix if scaling is given 
+    // Initializes the stage with a Movie and the passed flash vars.
+    _stage->init(_movieDef.get(), _flashVars);
 
     bool background = true; // ??
     _stage->set_background_alpha(background ? 1.0f : 0.05f);
 
     // @todo since we registered the sound handler, shouldn't we know
     //       already what it is ?!
-    sound::sound_handler* s = _stage->runInfo().soundHandler();
+    sound::sound_handler* s = _stage->runResources().soundHandler();
     if ( s ) s->unpause();
     _started = true;
+    
+    // to properly update stageMatrix if scaling is given  
+    resize_view(_width, _height); 
+
+    log_debug("Starting virtual clock");
+    _virtualClock.resume();
+
 }
 
 bool
 Gui::advanceMovie()
 {
+	if (isStopped()) return true;
 
-	if ( isStopped() ) return true;
-
-    if ( ! _started ) start();
-  
-
-
-#ifdef SKIP_RENDERING_IF_LATE
-	WallClockTimer advanceTimer;
-#endif // SKIP_RENDERING_IF_LATE
+    if (!_started) start();
 
 	gnash::movie_root* m = _stage;
 	
-#ifdef GNASH_FPS_DEBUG
-	fpsCounterTick(); // will be a no-op if fps_timer_interval is zero
-#endif
-
 // Define REVIEW_ALL_FRAMES to have *all* frames
-// consequencially displaied. Useful for debugging.
+// consequentially displayed. Useful for debugging.
 //#define REVIEW_ALL_FRAMES 1
 
 #ifndef REVIEW_ALL_FRAMES
 	// Advance movie by one frame
-	m->advance();
+	bool advanced = m->advance();
 #else
-	size_t cur_frame = m->getRootMovie()->get_current_frame();
-	size_t tot_frames = m->getRootMovie()->get_frame_count();
-	m->advance();
-	m->get_movie_definition()->ensure_frame_loaded(tot_frames);
-	m->goto_frame(cur_frame+1);
-    	m->set_play_state(gnash::MovieClip::PLAY);
+	const size_t cur_frame = m->getRootMovie()->get_current_frame();
+	const size_t tot_frames = m->getRootMovie()->get_frame_count();
+	const bool advanced = m->advance();
+	
+    m->getRootMovie.ensureFrameLoaded(tot_frames);
+	m->goto_frame(cur_frame + 1);
+    m->set_play_state(gnash::MovieClip::PLAYSTATE_PLAY);
 	log_debug(_("Frame %d"), m->get_current_frame());
 #endif
+    
+
+#ifdef GNASH_FPS_DEBUG
+    // will be a no-op if fps_timer_interval is zero
+	if (advanced) fpsCounterTick(); 
+#endif
 
 
+
+    // TODO: ask stage about doDisplay ?
+    // - if it didn't advance might need to check updateAfterEvent
+    bool doDisplay = true;
+
 #ifdef SKIP_RENDERING_IF_LATE
+    // We want to skip rendering IFF it's time to advance again.
+    // We'll ask the stage about it
+    if (_stage->timeToNextFrame() <= 0) {
 
-	boost::uint32_t advanceTime = advanceTimer.elapsed(); // in milliseconds !
-
-	boost::uint32_t timeSlot = _interval; // milliseconds between advance calls 
-
-	if ( advanceTime+gui->estimatedDisplayTime < timeSlot )
-	{
-		advanceTimer.restart();
-		display(m);
-		boost::uint32_t displayTime = advanceTimer.elapsed();
-
-		if ( displayTime > estimatedDisplayTime)
-		{
-			//log_debug("Display took %6.6g seconds over %6.6g available for each frame", displayTime, timeSlot);
-
-			// Don't update estimatedDisplayTime if it's bigger then timeSlot*0.8
-			if (  displayTime < timeSlot*0.8 )
-			{
-				// TODO: check for absurdly high values, like we can't set
-				//       estimatedDisplayTime to a value higher then FPS, or
-				//       we'll simply never display...
-				estimatedDisplayTime = displayTime;
-			}
-		}
-	}
-	else
-	{
-		log_debug("We're unable to keep up with FPS speed: "
-			"advanceTime was %u + estimatedDisplayTime (%u) "
-			"== %u, over a timeSlot of %u",
-			advanceTime, estimatedDisplayTime,
-			advanceTime+estimatedDisplayTime, timeSlot);
-		// TODO: increment a counter, we don't want to skip too many frames
-	}
-#else // ndef SKIP_RENDERING_IF_LATE
-
-	display(m);
-
+        // or should it be if advanced ?
+        if (doDisplay) {
+            // TODO: take note of a frame drop (count them)
+            //log_debug("Frame rendering dropped due to being late");
+#ifdef GNASH_FPS_DEBUG
+            ++frames_dropped;
+#endif
+        }
+        doDisplay = false;
+    }
 #endif // ndef SKIP_RENDERING_IF_LATE
-	
-	if ( ! loops() )
-	{
+
+	if (doDisplay) display(m);
+
+	if (!loops()) {
 		size_t curframe = m->get_current_frame(); // can be 0 on malformed SWF
-		gnash::MovieClip* si = m->getRootMovie();
-		if (curframe + 1 >= si->get_frame_count())
-		{
+		const gnash::MovieClip& si = m->getRootMovie();
+		if (curframe + 1 >= si.get_frame_count()) {
 			quit(); 
 		}
 	}
 
-    /// Quit if we've reached the advance limit.
-    if (_maxAdvances && (_advances++ > _maxAdvances))
-    {
-        quit();
+    if (_screenShotter.get()) {
+        _screenShotter->screenShot(_advances);
+    }
+
+    // Only increment advances and check for exit condition when we've
+    // really changed frame.
+    if (advanced) {
+
+        /// Quit if we've reached the frame advance limit.
+        if (_maxAdvances && (_advances > _maxAdvances)) {
+            quit();
+        }
+        ++_advances;
     }
 
 	return true;
 }
 
+void
+Gui::takeScreenShot()
+{
+    if (!_screenShotter.get()) {
+        // If no ScreenShotter exists, none was requested at startup.
+        // We use a default filename pattern.
+        URL url(_runResources.baseURL());
+        std::string::size_type p = url.path().rfind('/');
+        const std::string& name = (p == std::string::npos) ? url.path() :
+            url.path().substr(p + 1);
+        const std::string& filename = "screenshot-" + name + "-%f";
+        _screenShotter.reset(new ScreenShotter(_renderer, filename));
+    }
+    assert (_screenShotter.get());
+    _screenShotter->now();
+}
+
+void
+Gui::requestScreenShots(const ScreenShotter::FrameList& l, bool last,
+        const std::string& filename)
+{
+    // Nothing to do if there is no renderer or if no frames should be
+    // saved.
+    if (!_renderer.get() || (l.empty() && !last)) return;
+
+    _screenShotter.reset(new ScreenShotter(_renderer, filename));
+    if (last) _screenShotter->lastFrame();
+    _screenShotter->setFrames(l);
+
+}
+
 void
 Gui::setCursor(gnash_cursor_type /*newcursor*/)
 {
@@ -1001,7 +1057,7 @@ Gui::want_redraw()
 }
 
 void
-Gui::setInvalidatedRegion(const rect& /*bounds*/)
+Gui::setInvalidatedRegion(const SWFRect& /*bounds*/)
 {
 }
 
@@ -1011,10 +1067,10 @@ Gui::setInvalidatedRegions(const InvalidatedRanges& ranges)
 	// fallback to single regions
 	geometry::Range2d<float> full = ranges.getFullArea();
 	
-	rect bounds;
+	SWFRect bounds;
 	
 	if (full.isFinite())
-		bounds = rect(full.getMinX(), full.getMinY(), full.getMaxX(), full.getMaxY());
+		bounds = SWFRect(full.getMinX(), full.getMinY(), full.getMaxX(), full.getMaxY());
 	else
 	if (full.isWorld())
 		bounds.set_world();
@@ -1073,13 +1129,12 @@ Gui::getMovieInfo() const
     //
     topIter = tr->insert(topIter, StringPair("Mouse Entities", ""));
 
-    const character* ch;
+    const DisplayObject* ch;
     ch = stage.getActiveEntityUnderPointer();
     if ( ch )
     {
 	    std::stringstream ss;
 	    ss << ch->getTarget() << " (" + typeName(*ch)
-            << " - id:" << ch->get_id()
             << " - depth:" << ch->get_depth()
             << " - useHandCursor:" << ch->allowHandCursor()
             << ")";
@@ -1091,7 +1146,7 @@ Gui::getMovieInfo() const
     {
 	    std::stringstream ss;
 	    ss << ch->getTarget() << " (" + typeName(*ch) 
-               << " - id:" << ch->get_id() << " - depth:" << ch->get_depth()
+               << " - depth:" << ch->get_depth()
                << ")";
 	firstLevelIter = tr->append_child(topIter, StringPair("Topmost entity under mouse pointer", ss.str()));
     }
@@ -1101,7 +1156,6 @@ Gui::getMovieInfo() const
     {
 	    std::stringstream ss;
 	    ss << ch->getTarget() << " (" + typeName(*ch) 
-               << " - id:" << ch->get_id()
                << " - depth:" << ch->get_depth() << ")";
     	firstLevelIter = tr->append_child(topIter, StringPair("Dragging character: ", ss.str()));
     }
@@ -1191,9 +1245,11 @@ Gui::fpsCounterTick()
     //log_debug("Effective frame rate: %0.2f fps", (float)(fps_counter/secs));
     std::cerr << boost::format("Effective frame rate: %0.2f fps "
                                "(min %0.2f, avg %0.2f, max %0.2f, "
-                               "%u frames in %0.1f secs total)") % rate %
+                               "%u frames in %0.1f secs total, "
+                               "dropped %u)") % rate %
                                fps_rate_min % avg % fps_rate_max %
-                               fps_counter_total % secs_total << std::endl;
+                               fps_counter_total % secs_total %
+                               frames_dropped << std::endl;
       
     fps_counter = 0;
     fps_timer = current_timer;
@@ -1230,9 +1286,73 @@ Gui::setStage(movie_root* stage)
 bool
 Gui::yesno(const std::string& question)
 {
-    log_error("This gui didn't override 'yesno', assuming 'yes' answer to question: %s", question);
+    log_error("This gui didn't override 'yesno', assuming 'yes' answer to "
+            "question: %s", question);
     return true;
 }
 
+void
+Gui::setQuality(Quality q)
+{
+	if (!_stage) {
+		log_error("Gui::setQuality called before a movie_root was available");
+		return;
+	}
+    _stage->setQuality(q);
+}
+
+Quality
+Gui::getQuality() const
+{
+	if (!_stage) {
+		log_error("Gui::getQuality called before a movie_root was available");
+        // just a guess..
+		return QUALITY_HIGH;
+	}
+    return _stage->getQuality();
+}
+
+void
+ScreenShotter::saveImage(const std::string& id) const
+{
+    // Replace all "%f" in the filename with the frameAdvance.
+    std::string outfile(_fileName);
+    boost::replace_all(outfile, "%f", id);
+
+    FILE* f = std::fopen(outfile.c_str(), "wb");
+    if (f) {
+        boost::shared_ptr<IOChannel> t(new tu_file(f, true));
+        _renderer->renderToImage(t, GNASH_FILETYPE_PNG);
+    }
+    else {
+        log_error("Failed to open screenshot file \"%s\"!", outfile);
+    }
+}
+
+void
+ScreenShotter::screenShot(size_t frameAdvance)
+{
+    // Save an image if an spontaneous screenshot was requested or the
+    // frame is in the list of requested frames.
+    if (_immediate || std::binary_search(_frames.begin(), _frames.end(),
+                frameAdvance)) {
+        saveImage(boost::lexical_cast<std::string>(frameAdvance));
+        _immediate = false;
+    }
+}
+
+void
+ScreenShotter::last() const
+{
+    if (_last) saveImage("last");
+}
+
+void
+ScreenShotter::setFrames(const FrameList& frames)
+{
+    _frames = frames;
+    std::sort(_frames.begin(), _frames.end());
+}
+
 // end of namespace
 }
