@@ -1,5 +1,6 @@
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,6 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "sdl_glue.h"
+#include "Renderer_ogl.h"
 
 
 
@@ -28,8 +30,9 @@ class SdlOglGlue : public SdlGlue
     SdlOglGlue();
     virtual ~SdlOglGlue();
 
-    bool init(int argc, char ***argv);
-    render_handler* createRenderHandler( int depth);
+    bool init(int argc, char **argv[]);
+    Renderer* createRenderHandler( int depth);
+    void setInvalidatedRegions(const InvalidatedRanges& ranges);
     bool prepDrawingArea(int width, int height, boost::uint32_t sdl_flags);
     void render();
   private:
