@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,6 +27,7 @@
 
 //#include "buffer.h"
 #include "element.h"
+#include "buffer.h"
 #include "dsodefs.h" // DSOEXPORT
 
 /// \namespace amf
@@ -83,7 +85,7 @@ class DSOEXPORT Flv {
         AUDIO_55KHZ = 0x0,
         AUDIO_11KHZ = 0x1,
         AUDIO_22KHZ = 0x2,
-        AUDIO_44KHZ = 0x3,
+        AUDIO_44KHZ = 0x3
     } flv_sound_rate_e;
     /// \enum Flv::flv_sound_format_e.
     ///
@@ -95,7 +97,7 @@ class DSOEXPORT Flv {
         AUDIO_NELLYMOSER_8KHZ = 0x5,
         AUDIO_NELLYMOSER = 0x6,
         // These next are only supported by Gnash
-        AUDIO_VORBIS = 0x7,
+        AUDIO_VORBIS = 0x7
     } flv_sound_format_e;
 
     // Video Tag types
@@ -112,7 +114,7 @@ class DSOEXPORT Flv {
         VIDEO_SCREEN2 = 0x6,
         VIDEO_THEORA = 0x7,
         VIDEO_DIRAC = 0x8,
-        VIDEO_SPEEX = 0x9,
+        VIDEO_SPEEX = 0x9
     } flv_video_codec_e;
     
     /// \enum Flv::flv_video_frame_e.
@@ -172,7 +174,8 @@ class DSOEXPORT Flv {
     /// @param buf a smart pointer to a Buffer containing the data.
     ///
     /// @return a smart pointer to data structure that contains the data.
-    boost::shared_ptr<flv_header_t> decodeHeader(boost::shared_ptr<amf::Buffer> buf);
+    boost::shared_ptr<flv_header_t> decodeHeader(boost::shared_ptr<amf::Buffer> buf) { return decodeHeader(buf->reference()); };
+    boost::shared_ptr<flv_header_t> decodeHeader(boost::uint8_t *data);
 
     /// \brief Decode a MetaData object.
     ///		This is after the header, but before all the other tags usually
@@ -211,7 +214,8 @@ class DSOEXPORT Flv {
     /// @param flags The data to deserialize.
     /// 
     /// @return a smart pointer to an video data structure that contains the data.
-    boost::shared_ptr<flv_tag_t> decodeTagHeader(boost::shared_ptr<amf::Buffer> &buf);
+    boost::shared_ptr<flv_tag_t> decodeTagHeader(boost::shared_ptr<amf::Buffer> &buf) { return decodeTagHeader(buf->reference()); };
+    boost::shared_ptr<flv_tag_t> decodeTagHeader(boost::uint8_t *data);
 
     /// \brief Find the named property for this Object.
     ///
@@ -255,6 +259,11 @@ class DSOEXPORT Flv {
     ///		The array of properties for this Flv file, which is
     ///		populated by the data from the first onMetaTag block.
     std::vector<boost::shared_ptr<amf::Element> > _properties;
+
+    /// \var _metadata
+    ///         The data contained in the first onMetaData tag from
+    ///         the FLV file.
+    boost::shared_ptr<amf::Element> _metadata;
     
 }; // end of class definition
 
