@@ -1,6 +1,7 @@
 // GnashSleep.h -- How do you sleep?
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,10 +20,6 @@
 #ifndef GNASH_SLEEP_H
 #define GNASH_SLEEP_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 // Headers for sleep
 #if defined(_WIN32) || defined(WIN32)
 # include <windows.h>
