@@ -9,7 +9,7 @@
 #ifndef TU_OPENGL_INCLUDES_H
 #define TU_OPENGL_INCLUDES_H
 
-#include "tu_config.h"
+#include "gnashconfig.h"
 
 // WIN32 includes.  We don't want to have to include windows.h because
 // it's such a pig, so #define a couple things that are required to
@@ -67,10 +67,16 @@
 #endif // __MACH__
 
 #if !defined(WIN32) && (!defined(__MACH__) || defined(GUI_GTK))
-# define GL_GLEXT_PROTOTYPES
+#ifndef GL_GLEXT_PROTOTYPES
+# define GL_GLEXT_PROTOTYPES 1
+#endif
+#ifdef __MACH__
+# include <OpenGL/glext.h>
+#else
 # include <GL/gl.h>
 # include <GL/glx.h>
 # include <GL/glu.h>
+#endif
 #ifndef APIENTRY
 # define APIENTRY
 #endif
