@@ -1,6 +1,7 @@
 // Button.cpp:  Mouse-sensitive buttons, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -35,7 +36,6 @@
 #include "fn_call.h" 
 #include "ExecutableCode.h"
 #include "namedStrings.h"
-#include "Object.h" // for getObjectInterface
 #include "StringPredicates.h"
 #include "GnashKey.h" 
 #include "SoundInfoRecord.h" 
@@ -173,6 +173,16 @@ frame loop:
 
 namespace gnash {
 
+namespace {
+    as_value button_blendMode(const fn_call& fn);
+    as_value button_cacheAsBitmap(const fn_call& fn);
+    as_value button_filters(const fn_call& fn);
+    as_value button_scale9Grid(const fn_call& fn);
+    as_value button_setTabIndex(const fn_call& fn);
+    as_value button_getTabIndex(const fn_call& fn);
+    as_value button_getDepth(const fn_call& fn);
+}
+
 namespace {
 
 class ButtonActionExecutor {
@@ -195,19 +205,15 @@ class ButtonActionPusher {
 public:
     ButtonActionPusher(movie_root& mr, DisplayObject* this_ptr)
         :
-        called(false),
         _mr(mr),
         _tp(this_ptr)
     {}
 
-    void operator() (const action_buffer& ab)
+    void operator()(const action_buffer& ab)
     {
-        _mr.pushAction(ab, boost::intrusive_ptr<DisplayObject>(_tp));
-        called = true;
+        _mr.pushAction(ab, _tp);
     }
 
-    bool called;
-
 private:
     movie_root& _mr;
     DisplayObject* _tp;
@@ -215,22 +221,19 @@ private:
 
 }
 
-// Forward declarations
-static as_object* getButtonInterface();
-
 namespace {
     void addInstanceProperty(Button& b, DisplayObject* d) {
         if (!d) return;
         const std::string& name = d->get_name();
         if (name.empty()) return;
-        b.init_member(name, d, 0);
+        getObject(&b)->init_member(name, getObject(d), 0);
     }
 
     void removeInstanceProperty(Button& b, DisplayObject* d) {
         if (!d) return;
         const std::string& name = d->get_name();
         if (name.empty()) return;
-        b.delProperty(getStringTable(b).find(name));
+        getObject(&b)->delProperty(getStringTable(*getObject(&b)).find(name));
     }
 }
 
@@ -255,85 +258,52 @@ static bool isCharacterNull(DisplayObject* ch, bool includeUnloaded)
 static void
 attachButtonInterface(as_object& o)
 {
-
-    as_c_function_ptr gettersetter;
-
-    o.init_property(NSV::PROP_uQUALITY, DisplayObject::quality,
-            DisplayObject::quality);
-    
-    o.init_property(NSV::PROP_uHIGHQUALITY, DisplayObject::highquality,
-            DisplayObject::highquality);
-
-    gettersetter = &DisplayObject::x_getset;
-    o.init_property(NSV::PROP_uX, *gettersetter, *gettersetter);
-
-    gettersetter = &DisplayObject::y_getset;
-    o.init_property(NSV::PROP_uY, *gettersetter, *gettersetter);
-
-    gettersetter = &DisplayObject::xscale_getset;
-    o.init_property(NSV::PROP_uXSCALE, *gettersetter, *gettersetter);
-
-    gettersetter = &DisplayObject::yscale_getset;
-    o.init_property(NSV::PROP_uYSCALE, *gettersetter, *gettersetter);
-
-    gettersetter = &DisplayObject::xmouse_get;
-    o.init_readonly_property(NSV::PROP_uXMOUSE, *gettersetter);
-
-    gettersetter = &DisplayObject::ymouse_get;
-    o.init_readonly_property(NSV::PROP_uYMOUSE, *gettersetter);
-
-    gettersetter = &DisplayObject::alpha_getset;
-    o.init_property(NSV::PROP_uALPHA, *gettersetter, *gettersetter);
-
-    gettersetter = &DisplayObject::visible_getset;
-    o.init_property(NSV::PROP_uVISIBLE, *gettersetter, *gettersetter);
-
-    gettersetter = &DisplayObject::width_getset;
-    o.init_property(NSV::PROP_uWIDTH, *gettersetter, *gettersetter);
-
-    gettersetter = &DisplayObject::height_getset;
-    o.init_property(NSV::PROP_uHEIGHT, *gettersetter, *gettersetter);
-
-    gettersetter = &DisplayObject::rotation_getset;
-    o.init_property(NSV::PROP_uROTATION, *gettersetter, *gettersetter);
-
-    gettersetter = &DisplayObject::parent_getset;
-    o.init_property(NSV::PROP_uPARENT, *gettersetter, *gettersetter);
-    
-    gettersetter = &DisplayObject::target_getset;
-    o.init_property(NSV::PROP_uTARGET, *gettersetter, *gettersetter);
-
-    gettersetter = DisplayObject::name_getset;
-    o.init_property(NSV::PROP_uNAME, gettersetter, gettersetter);
     
     const int unprotected = 0;
     o.init_member(NSV::PROP_ENABLED, true, unprotected);
     o.init_member("useHandCursor", true, unprotected);
+    
+    const int swf8Flags = PropFlags::onlySWF8Up;
+    VM& vm = getVM(o);
+
+    o.init_property("tabIndex", *vm.getNative(105, 1), *vm.getNative(105, 2),
+            swf8Flags);
+    
+    o.init_member("getDepth", vm.getNative(105, 3), unprotected);
+
+    NativeFunction* gs;
+    gs = vm.getNative(105, 4);
+    o.init_property("scale9Grid", *gs, *gs, swf8Flags);
+    gs = vm.getNative(105, 5);
+    o.init_property("filters", *gs, *gs, swf8Flags);
+    gs = vm.getNative(105, 6);
+    o.init_property("cacheAsBitmap", *gs, *gs, swf8Flags);
+    gs = vm.getNative(105, 7);
+    o.init_property("blendMode", *gs, *gs, swf8Flags);
 
 }
 
-Button::Button(const SWF::DefineButtonTag* const def, DisplayObject* parent,
-        int id)
+Button::Button(as_object* object, const SWF::DefineButtonTag* def,
+        DisplayObject* parent)
     :
-    InteractiveObject(parent, id),
+    InteractiveObject(object, parent),
     _lastMouseFlags(FLAG_IDLE),
     _mouseFlags(FLAG_IDLE),
     _mouseState(MOUSESTATE_UP),
     _def(def)
 {
-
-    set_prototype(getButtonInterface());
+    assert(object);
 
     // check up presence Key events
     if (_def->hasKeyPressHandler()) {
-        getRoot(*this).add_key_listener(this);
+        stage().add_key_listener(this);
     }
 
 }
 
 Button::~Button()
 {
-    getRoot(*this).remove_key_listener(this);
+    stage().remove_key_listener(this);
 }
 
 bool
@@ -341,8 +311,8 @@ Button::trackAsMenu()
 {
     // TODO: check whether the AS or the tag value takes precedence.
     as_value track;
-    string_table& st = getStringTable(*this);
-    if (get_member(st.find("trackAsMenu"), &track)) {
+    string_table& st = getStringTable(*getObject(this));
+    if (getObject(this)->get_member(st.find("trackAsMenu"), &track)) {
         return track.to_bool();
     }
     if (_def) return _def->trackAsMenu();
@@ -353,36 +323,29 @@ bool
 Button::isEnabled()
 {
     as_value enabled;
-    if (!get_member(NSV::PROP_ENABLED, &enabled)) return false;
+    if (!getObject(this)->get_member(NSV::PROP_ENABLED, &enabled)) return false;
 
     return enabled.to_bool();
 }
 
 
-bool
-Button::on_event(const event_id& id)
+void
+Button::notifyEvent(const event_id& id)
 {
-    if (unloaded())
-    {
+    if (unloaded()) {
         // We dont' respond to events while unloaded
         // See bug #22982
-#if 0 // debugging..
-        log_debug("Button %s received %s event while unloaded: ignored",
-            getTarget(), id);
-#endif
-        return false; 
+        return; 
     }
 
     // We only respond keypress events
-    if ( id.id() != event_id::KEY_PRESS ) return false;
+    if ( id.id() != event_id::KEY_PRESS ) return;
 
     // We only respond to valid key code (should we assert here?)
-    if ( id.keyCode() == key::INVALID ) return false;
+    if ( id.keyCode() == key::INVALID ) return;
 
-    ButtonActionPusher xec(getRoot(*this), this); 
+    ButtonActionPusher xec(stage(), this); 
     _def->forEachTrigger(id, xec);
-
-    return xec.called;
 }
 
 bool
@@ -524,7 +487,7 @@ Button::mouseEvent(const event_id& event)
         if (!_def->hasSound()) break;
 
         // Check if there is a sound handler
-        sound::sound_handler* s = getRunResources(*this).soundHandler();
+        sound::sound_handler* s = getRunResources(*getObject(this)).soundHandler();
         if (!s) break;
 
         int bi; // button sound array index [0..3]
@@ -554,18 +517,16 @@ Button::mouseEvent(const event_id& event)
         const SWF::DefineButtonSoundTag::ButtonSound& bs = 
             _def->buttonSound(bi);
 
-        // DisplayObject zero is considered as null DisplayObject
+        // character zero is considered as null character
         if (!bs.soundID) break;
 
         // No actual sound ?
         if (!bs.sample) break;
 
-        if (bs.soundInfo.stopPlayback)
-        {
+        if (bs.soundInfo.stopPlayback) {
             s->stop_sound(bs.sample->m_sound_handler_id);
         }
-        else
-        {
+        else {
             const SWF::SoundInfoRecord& sinfo = bs.soundInfo;
 
             const sound::SoundEnvelopes* env = 
@@ -592,25 +553,18 @@ Button::mouseEvent(const event_id& event)
     // the action queue on mouse event.
     //
 
-    movie_root& mr = getRoot(*this);
+    movie_root& mr = stage();
 
     ButtonActionPusher xec(mr, this); 
     _def->forEachTrigger(event, xec);
 
     // check for built-in event handler.
     std::auto_ptr<ExecutableCode> code ( get_event_handler(event) );
-    if ( code.get() )
-    {
-        //log_debug(_("Got statically-defined handler for event: %s"), event);
+    if (code.get()) {
         mr.pushAction(code, movie_root::apDOACTION);
     }
 
-    // Call conventional attached method.
-    boost::intrusive_ptr<as_function> method =
-        getUserDefinedEventHandler(event.functionKey());
-    if (method) {
-        mr.pushAction(method, this, movie_root::apDOACTION);
-    }
+    callMethod(getObject(this), event.functionKey());
 }
 
 
@@ -788,10 +742,10 @@ Button::add_invalidated_bounds(InvalidatedRanges& ranges, bool force)
     );
 }
 
-rect
+SWFRect
 Button::getBounds() const
 {
-    rect allBounds;
+    SWFRect allBounds;
 
     typedef std::vector<const DisplayObject*> Chars;
     Chars actChars;
@@ -800,7 +754,7 @@ Button::getBounds() const
     {
         const DisplayObject* ch = *i;
         // Child bounds need be transformed in our coordinate space
-        rect lclBounds = ch->getBounds();
+        SWFRect lclBounds = ch->getBounds();
         SWFMatrix m = ch->getMatrix();
         allBounds.expand_to_transformed_rect(m, lclBounds);
     }
@@ -822,46 +776,6 @@ Button::pointInShape(boost::int32_t x, boost::int32_t y) const
     return false; 
 }
 
-as_object*
-Button::get_path_element(string_table::key key)
-{
-    as_object* ch = getPathElementSeparator(key);
-    if ( ch ) return ch;
-
-    const std::string& name = getStringTable(*this).value(key);
-    return getChildByName(name); // possibly NULL
-}
-
-DisplayObject *
-Button::getChildByName(const std::string& name)
-{
-    // Get all currently active DisplayObjects, including unloaded
-    DisplayObjects actChars;
-    getActiveCharacters(actChars, true);
-
-    // Lower depth first for duplicated names, so we sort
-    std::sort(actChars.begin(), actChars.end(), charDepthLessThen);
-
-    for (DisplayObjects::iterator i=actChars.begin(), e=actChars.end(); i!=e; ++i)
-    {
-
-        DisplayObject* const child = *i;
-        const std::string& childname = child->get_name();
- 
-        if (getSWFVersion(*this) >= 7 )
-        {
-            if ( childname == name ) return child;
-        }
-        else
-        {
-            StringNoCaseEqual noCaseCompare;
-            if ( noCaseCompare(childname, name) ) return child;
-        }
-    }
-
-    return NULL;
-}
-
 void
 Button::stagePlacementCallback(as_object* initObj)
 {
@@ -870,7 +784,7 @@ Button::stagePlacementCallback(as_object* initObj)
     if (initObj) {
         log_unimpl("Button placed with an initObj. How did this happen? "
                 "We'll copy the properties anyway");
-        copyProperties(*initObj);
+        getObject(this)->copyProperties(*initObj);
     }
 
     saveOriginalTarget(); // for soft refs
@@ -919,31 +833,23 @@ Button::stagePlacementCallback(as_object* initObj)
 
 #ifdef GNASH_USE_GC
 void
-Button::markReachableResources() const
+Button::markOwnResources() const
 {
-    assert(isReachable());
 
     _def->setReachable();
 
     // Mark state DisplayObjects as reachable
-    for (DisplayObjects::const_iterator i=_stateCharacters.begin(), e=_stateCharacters.end();
-            i!=e; ++i)
+    for (DisplayObjects::const_iterator i = _stateCharacters.begin(),
+            e = _stateCharacters.end(); i != e; ++i)
     {
         DisplayObject* ch = *i;
-        if ( ch ) ch->setReachable();
+        if (ch) ch->setReachable();
     }
 
     // Mark hit DisplayObjects as reachable
-    for (DisplayObjects::const_iterator i = _hitCharacters.begin(),
-            e=_hitCharacters.end(); i != e; ++i)
-    {
-        DisplayObject* ch = *i;
-        assert ( ch );
-        ch->setReachable();
-    }
+    std::for_each(_hitCharacters.begin(), _hitCharacters.end(),
+            std::mem_fun(&DisplayObject::setReachable));
 
-    // DisplayObject class members
-    markDisplayObjectReachable();
 }
 #endif // GNASH_USE_GC
 
@@ -959,9 +865,8 @@ Button::unload()
             e = _stateCharacters.end(); i != e; ++i)
     {
         DisplayObject* ch = *i;
-        if ( ! ch ) continue;
-        if ( ch->unloaded() ) continue;
-        if ( ch->unload() ) childsHaveUnload = true;
+        if (!ch || ch->unloaded()) continue;
+        if (ch->unload()) childsHaveUnload = true;
     }
 
     // NOTE: we don't need to ::unload or ::destroy here
@@ -982,13 +887,10 @@ Button::destroy()
 {
 
     for (DisplayObjects::iterator i = _stateCharacters.begin(),
-            e=_stateCharacters.end(); i != e; ++i)
-    {
+            e=_stateCharacters.end(); i != e; ++i) {
         DisplayObject* ch = *i;
-        if ( ! ch ) continue;
-        if ( ch->isDestroyed() ) continue;
+        if (!ch || ch->isDestroyed()) continue;
         ch->destroy();
-        *i = 0;
     }
 
     // NOTE: we don't need to ::unload or ::destroy here
@@ -1002,119 +904,14 @@ Button::destroy()
     DisplayObject::destroy();
 }
 
-bool
-Button::get_member(string_table::key name_key, as_value* val,
-    string_table::key nsname)
-{
-    // FIXME: use addProperty interface for these !!
-    // TODO: or at least have a DisplayObject:: protected method take
-    //       care of these ?
-    //       Duplicates code in DisplayObject::getPathElementSeparator too..
-    //
-    if (name_key == NSV::PROP_uROOT) {
-        // getAsRoot() will take care of _lockroot
-        val->set_as_object(getAsRoot());
-        return true;
-    }
-
-    // NOTE: availability of _global doesn't depend on VM version
-    //             but on actual movie version. Example: if an SWF4 loads
-    //             an SWF6 (to, say, _level2), _global will be unavailable
-    //             to the SWF4 code but available to the SWF6 one.
-    //
-    // see MovieClip.as
-    if (getMovieVersion() > 5 && name_key == NSV::PROP_uGLOBAL ) {
-        // The "_global" ref was added in SWF6
-        val->set_as_object(getGlobal(*this));
-        return true;
-    }
-
-    const std::string& name = getStringTable(*this).value(name_key);
-
-    movie_root& mr = getRoot(*this);
-    unsigned int levelno;
-    if ( mr.isLevelTarget(name, levelno) ) {
-        Movie* mo = mr.getLevel(levelno).get();
-        if ( mo ) {
-            val->set_as_object(mo);
-            return true;
-        }
-        else {
-            return false;
-        }
-    }
-
-    // TOCHECK : Try object members, BEFORE display list items
-    //
-    if (as_object::get_member(name_key, val, nsname))
-    {
-
-    // ... trying to be useful to Flash coders ...
-    // The check should actually be performed before any return
-    // prior to the one due to a match in the DisplayList.
-    // It's off by default anyway, so not a big deal.
-    // See bug #18457
-#define CHECK_FOR_NAME_CLASHES 1
-#ifdef CHECK_FOR_NAME_CLASHES
-        IF_VERBOSE_ASCODING_ERRORS(
-        if ( getChildByName(name) )
-        {
-            log_aserror(_("A button member (%s) clashes with "
-                    "the name of an existing DisplayObject "
-                    "in its display list.    "
-                    "The member will hide the "
-                    "DisplayObject"), name);
-        }
-        );
-#endif
-
-        return true;
-    }
-
-    // Try items on our display list.
-    DisplayObject* ch = getChildByName(name);
-
-    if (ch) {
-        // Found object.
-
-        // If the object is an ActionScript referenciable one we
-        // return it, otherwise we return ourselves
-        if ( ch->isActionScriptReferenceable() ) {
-            val->set_as_object(ch);
-        }
-        else {
-            val->set_as_object(this);
-        }
-
-        return true;
-    }
-
-    return false;
-
-}
-
 int
-Button::getMovieVersion() const
+Button::getDefinitionVersion() const
 {
     return _def->getSWFVersion();
 }
 
-static as_object*
-getButtonInterface()
-{
-  static boost::intrusive_ptr<as_object> proto;
-  if ( proto == NULL )
-  {
-    proto = new as_object(getObjectInterface());
-    VM::get().addStatic(proto.get());
-
-    attachButtonInterface(*proto);
-  }
-  return proto.get();
-}
-
 static as_value
-button_ctor(const fn_call& /* fn */)
+button_ctor(const fn_call& /*fn*/)
 {
     return as_value();
 }
@@ -1123,19 +920,26 @@ void
 button_class_init(as_object& global, const ObjectURI& uri)
 {
     // This is going to be the global Button "class"/"function"
-    Global_as* gl = getGlobal(global);
-    as_object* proto = getButtonInterface();
-    as_object* cl = gl->createClass(&button_ctor, proto);
+    Global_as& gl = getGlobal(global);
+    as_object* proto = gl.createObject();
+    as_object* cl = gl.createClass(&button_ctor, proto);
+    attachButtonInterface(*proto);
 
     // Register _global.MovieClip
-    global.init_member(getName(uri), cl, as_object::DefaultFlags,
-            getNamespace(uri));
+    global.init_member(uri, cl, as_object::DefaultFlags);
 }
 
 void
-registerButtonNative(as_object& /*global*/)
+registerButtonNative(as_object& global)
 {
-    // TODO: button.getDepth
+    VM& vm = getVM(global);
+    vm.registerNative(button_setTabIndex, 105, 1);
+    vm.registerNative(button_getTabIndex, 105, 2);
+    vm.registerNative(button_getDepth, 105, 3);
+    vm.registerNative(button_scale9Grid, 105, 4);
+    vm.registerNative(button_filters, 105, 5);
+    vm.registerNative(button_cacheAsBitmap, 105, 6);
+    vm.registerNative(button_blendMode, 105, 7);
 }
 
 #ifdef USE_SWFTREE
@@ -1175,10 +979,69 @@ Button::mouseStateName(MouseState s)
         case MOUSESTATE_DOWN: return "DOWN";
         case MOUSESTATE_OVER: return "OVER";
         case MOUSESTATE_HIT: return "HIT";
-        default: return "UNKNOWN (error?)";
+        default: std::abort();
     }
 }
 
+namespace {
+
+as_value
+button_blendMode(const fn_call& fn)
+{
+    Button* obj = ensure<IsDisplayObject<Button> >(fn);
+    UNUSED(obj);
+    return as_value();
+}
+
+as_value
+button_cacheAsBitmap(const fn_call& fn)
+{
+    Button* obj = ensure<IsDisplayObject<Button> >(fn);
+    UNUSED(obj);
+    return as_value();
+}
+
+as_value
+button_filters(const fn_call& fn)
+{
+    Button* obj = ensure<IsDisplayObject<Button> >(fn);
+    UNUSED(obj);
+    return as_value();
+}
+
+as_value
+button_scale9Grid(const fn_call& fn)
+{
+    Button* obj = ensure<IsDisplayObject<Button> >(fn);
+    UNUSED(obj);
+    return as_value();
+}
+
+as_value
+button_getTabIndex(const fn_call& fn)
+{
+    Button* obj = ensure<IsDisplayObject<Button> >(fn);
+    UNUSED(obj);
+    return as_value();
+}
+
+as_value
+button_setTabIndex(const fn_call& fn)
+{
+    Button* obj = ensure<IsDisplayObject<Button> >(fn);
+    UNUSED(obj);
+    return as_value();
+}
+
+as_value
+button_getDepth(const fn_call& fn)
+{
+    Button* obj = ensure<IsDisplayObject<Button> >(fn);
+    UNUSED(obj);
+    return as_value();
+}
+
+} // anonymous namespace
 } // end of namespace gnash
 
 
