@@ -1,6 +1,7 @@
 // dlist.h:  Display list definitions, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -46,7 +47,7 @@ namespace gnash {
 namespace gnash {
 
 /// A DisplayItem is simply a DisplayObject object 
-typedef boost::intrusive_ptr<DisplayObject> DisplayItem;
+typedef DisplayObject* DisplayItem;
 
 /// A list of on-stage DisplayObjects, ordered by depth
 //
@@ -373,7 +374,7 @@ public:
 		for (const_iterator it = nonRemoved(),
                 itEnd = _charsByDepth.end(); it != itEnd; ++it) {
 
-			boost::intrusive_ptr<DisplayObject> ch = *it;
+			DisplayObject* ch = *it;
 			int depth = ch->get_depth();
 			if (!depths.insert(depth).second) {
 				log_debug("Depth %d is duplicated in DisplayList %p",
@@ -399,7 +400,7 @@ private:
 	///
 	/// TODO: inspect what should happen if the target depth is already
     /// occupied
-	void reinsertRemovedCharacter(boost::intrusive_ptr<DisplayObject> ch);
+	void reinsertRemovedCharacter(DisplayObject* ch);
 
 	container_type _charsByDepth;
 
@@ -415,7 +416,7 @@ DisplayList::visitForward(V& visitor)
 		it != itEnd; ++it) {
 
 		DisplayItem& di = *it;
-		if (!visitor(di.get())) break;
+		if (!visitor(di)) break;
 	}
 }
 
@@ -427,7 +428,7 @@ DisplayList::visitBackward(V& visitor)
 			itEnd = _charsByDepth.rend(); it != itEnd; ++it) {
 
 		DisplayItem& di = *it;
-		if (!visitor(di.get())) break;
+		if (!visitor(di)) break;
 	}
 }
 
@@ -439,7 +440,7 @@ DisplayList::visitBackward(V& visitor) const
 			itEnd = _charsByDepth.rend(); it != itEnd; ++it) {
 
 		const DisplayItem& di = *it;
-		if (!visitor(di.get())) break;
+		if (!visitor(di)) break;
 	}
 }
 
@@ -450,7 +451,7 @@ DisplayList::visitAll(V& visitor)
 	for (iterator it = _charsByDepth.begin(), itEnd = _charsByDepth.end();
 		it != itEnd; ++it) {
 
-		visitor(it->get());
+		visitor(*it);
 	}
 }
 
@@ -461,7 +462,7 @@ DisplayList::visitAll(V& visitor) const
 	for (const_iterator it = _charsByDepth.begin(),
             itEnd = _charsByDepth.end(); it != itEnd; ++it) {
 
-		visitor(it->get());
+		visitor(*it);
 	}
 }
 
