@@ -1,6 +1,7 @@
 // MovieClip.h:  Stateful live Sprite instance, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -85,6 +86,8 @@ class MovieClip : public DisplayObjectContainer
 
 public:
 
+    typedef std::map<std::string, std::string> MovieVariables;
+
     typedef std::list<const action_buffer*> ActionList;
 
     typedef movie_definition::PlayList PlayList;
@@ -113,15 +116,8 @@ public:
     /// @param parent
     ///     Parent of the created instance in the display list.
     ///     May be 0 for top-level movies (_level#).
-    ///
-    /// @param id
-    ///     Identifier of the DisplayObject definition this is an instance
-    ///     of. This is required by DisplayObject class, but probably
-    ///     to be deprecated if every instance has a reference to its
-    ///     definition, which should know its id...
-    ///
-    MovieClip(const movie_definition* const def, Movie* root,
-            DisplayObject* parent, int id);
+    MovieClip(as_object* object, const movie_definition* def,
+            Movie* root, DisplayObject* parent);
 
     virtual ~MovieClip();
 
@@ -137,7 +133,7 @@ public:
     virtual MovieClip* getAsRoot();
 
     /// Get the composite bounds of all component drawing elements
-    virtual rect getBounds() const;
+    virtual SWFRect getBounds() const;
 
     // See dox in DisplayObject.h
     virtual bool pointInShape(boost::int32_t x, boost::int32_t y) const;
@@ -149,7 +145,6 @@ public:
     ///
     /// all sprites except mouse-insensitive dynamic masks are hitable.
     /// _visible property is ignored for hitable DisplayObjects.
-    ///
     virtual bool pointInHitableShape(boost::int32_t x, boost::int32_t y) const;
 
     /// Return 0-based index to current frame
@@ -187,9 +182,9 @@ public:
         return isDynamic() ? 0 : _def->get_bytes_loaded();
     }
 
-    const rect& get_frame_size() const
+    const SWFRect& get_frame_size() const
     {
-        static const rect r;
+        static const SWFRect r;
         return _def ? _def->get_frame_size() : r;
     }
 
@@ -237,11 +232,6 @@ public:
         return _droptarget;
     }
     
-    virtual bool wantsInstanceName() const
-    {
-        return true; // sprites can be referenced 
-    }
-
     virtual void advance();
 
     /// Set the sprite state at the specified frame number.
@@ -267,19 +257,16 @@ public:
     /// @return
     /// True if the frame_spec could be resolved to a frame number.
     /// False if the frame_spec was invalid.
-    ///
     bool get_frame_number(const as_value& frame_spec, size_t& frameno) const;
 
     /// MovieClip instances need to handle cxform specially 
     //
     /// This is to suppor the Color asobject class
-    ///
     cxform  get_world_cxform() const;
 
     /// Update user-defined color transform
     //
     /// This should only be used by the Color AS class
-    ///
     void set_user_cxform(const cxform& cx)
     {
         set_invalidated();
@@ -289,7 +276,6 @@ public:
     /// Return the user-defined color transform
     //
     /// This should only be used by the Color AS class
-    ///
     cxform get_user_cxform() const
     {
         return _userCxform;
@@ -306,7 +292,6 @@ public:
     /// Swap depth of the given DisplayObjects in the DisplayList
     //
     /// See DisplayList::swapDepths for more info
-    ///
     void swapDepths(DisplayObject* ch1, int newdepth)
     {
         _displayList.swapDepths(ch1, newdepth);
@@ -315,13 +300,10 @@ public:
     /// Return the DisplayObject at given depth in our DisplayList.
     //
     /// @return NULL if the specified depth is available (no chars there)
-    ///
     DisplayObject* getDisplayObjectAtDepth(int depth);
 
-    DisplayObject* add_empty_movieclip(const std::string& name, int depth);
-
-    boost::intrusive_ptr<DisplayObject> add_textfield(const std::string& name,
-            int depth, int x, int y, float width, float height);
+    /// Attach a DisplayObject at the specified depth.
+    DisplayObject* addDisplayListObject(DisplayObject* obj, int depth);
 
     /// Place a DisplayObject or mask to the DisplayList.
     //
@@ -343,7 +325,6 @@ public:
     ///
     /// @return
     ///     A pointer to the DisplayObject being added or NULL
-    ///
     DisplayObject* add_display_object(const SWF::PlaceObject2Tag* tag,
             DisplayList& dlist);
 
@@ -376,7 +357,6 @@ public:
     /// will be set to the old one.
     /// if false, the transformation SWFMatrix of the new DisplayObject will
     /// be untouched.
-    ///
     void replace_display_object(DisplayObject* ch, int depth,
         bool use_old_cxform, bool use_old_matrix);
 
@@ -390,7 +370,6 @@ public:
     /// (2)the id might be used for specifying a DisplayObject
     /// in the depth(think about multiple DisplayObjects within the same
     /// depth, not tested and a rare case)
-    ///
     void remove_display_object(int depth, int /* id */)
     {
         set_invalidated();
@@ -409,7 +388,6 @@ public:
     ///
     /// @return true on success, false on failure
     /// FIXME: currently never returns false !
-    ///
     bool attachCharacter(DisplayObject& newch, int depth, as_object* initObject);
 
     /// Handle placement event
@@ -425,7 +403,6 @@ public:
     //
     /// (1) Construct this instance as an ActionScript object.
     ///     See constructAsScriptObject() method.
-    ///
     virtual void stagePlacementCallback(as_object* initObj = 0);
 
     /// Unload all contents in the displaylist and this instance
@@ -447,7 +424,6 @@ public:
     /// 
     /// Note: this function will release most memory associated with
     /// the sprite as no members or drawable should be needed anymore.
-    ///
     void destroy();
         
     /// Add the given action buffer to the list of action
@@ -473,17 +449,11 @@ public:
     ///
     /// @param cid
     /// The referenced DisplayObject id
-    ///
     void execute_init_action_buffer(const action_buffer& a, int cid);
 
     /// Execute a single action buffer (DOACTION block)
     void execute_action(const action_buffer& ab);
 
-    /// For debugging -- return the id of the DisplayObject
-    /// at the specified depth.
-    /// Return -1 if nobody's home.
-    int get_id_at_depth(int depth);
-
     MovieClip* to_movie () { return this; }
 
     /// The various methods for sending data in requests.
@@ -496,17 +466,8 @@ public:
         METHOD_POST
     };
 
-    /// Load a movie in this sprite, replacing it
-    //
-    /// @param url
-    ///  The url to load the movie from. Can be a bitmap or an SWF.
-    ///
-    /// @param postdata
-    ///  IF not NULL, use as the POST body for HTTP requests
-    ///
-    /// Return: true if it succeeded, false otherwise
-    ///
-    bool loadMovie(const URL& url, const std::string* postdata=NULL);
+    // See dox in DisplayObject.h
+    virtual void getLoadedMovie(Movie* newMovie);
 
     /// \brief
     /// Load url-encoded variables from the given url, optionally
@@ -523,29 +484,37 @@ public:
     ///
     /// @param sendVarsMethod: The VariablesMethod to use. If METHOD_NONE,
     ///                        no data will be sent.
-    ///
     void loadVariables(const std::string& urlstr,
             VariablesMethod sendVarsMethod);
 
+    /// Get TextField variables
     //
-    // ActionScript support
-    //
+    /// TODO: this is unlikely to be the best way of doing it, and it would
+    /// simplify things if this function could be dropped.
+    bool getTextFieldVariables(const ObjectURI& uri, as_value& val);
 
-    // See dox in as_object.h
-    bool get_member(string_table::key name, as_value* val, 
-        string_table::key nsname = 0);
-        
-    // See dox in as_object.h
-    virtual bool set_member(string_table::key name, const as_value& val,
-        string_table::key nsname = 0, bool ifFound=false);
+    // Set TextField variables
+    //
+    /// TODO: this is also unlikely to be the best way to do it.
+    bool setTextFieldVariables(const ObjectURI& uri, const as_value& val);;
+
+    /// Search for a named object on the DisplayList
+    //
+    /// These are properties, but not attached as genuine members to the
+    /// MovieClip object. They take priority over DisplayObject magic
+    /// properties and inherited properties, but not over own properties.
+    //
+    /// @param name     The name of the object. This function handles
+    ///                 case-sensitivity.
+    /// @return         The object if found, otherwise 0.
+    DisplayObject* getDisplayListObject(string_table::key name);
 
     /// Overridden to look in DisplayList for a match
-    as_object* get_path_element(string_table::key key);
+    as_object* pathElement(string_table::key key);
 
     /// Execute the actions for the specified frame. 
     //
     /// The frame_spec could be an integer or a string.
-    ///
     virtual void call_frame_actions(const as_value& frame_spec);
 
     // delegates to movie_root 
@@ -569,13 +538,11 @@ public:
     ///
     /// @param init_object
     ///     If not null, will be used to copy properties over.
-    ///
-    boost::intrusive_ptr<MovieClip> duplicateMovieClip(
-        const std::string& newname,
+    MovieClip* duplicateMovieClip(const std::string& newname,
         int newdepth, as_object* init_object=NULL);
         
     /// Dispatch event handler(s), if any.
-    virtual bool on_event(const event_id& id);
+    virtual void notifyEvent(const event_id& id);
 
     // inherited from DisplayObject class, see dox in DisplayObject.h
     as_environment& get_environment() {
@@ -587,7 +554,6 @@ public:
     //
     /// A TextField variable is a variable that acts
     /// as a setter/getter for a TextField 'text' member.
-    ///
     void set_textfield_variable(const std::string& name, TextField* ch);
 
     void add_invalidated_bounds(InvalidatedRanges& ranges, bool force);
@@ -601,26 +567,13 @@ public:
     /// Placing an object at the depth returned by
     /// this function should result in a DisplayObject
     /// that is displayd above all others
-    ///
     int getNextHighestDepth() const {
         return _displayList.getNextHighestDepth();
     }
 
-    void testInvariant() const {
-        // _currentFrame may be 0, since this is our initial
-        // condition. Still, frame count might be 0 as well, and
-        // loaded frames too !
-        //assert(_currentFrame < _def->get_frame_count());
-#ifndef GNASH_USE_GC 
-        assert(get_ref_count() > 0); // or we're constructed but
-                                     // not stored in a boost::intrusive_ptr
-#endif
-    }
-
     /// Set the currently playing m_sound_stream_id
     // 
     // TODO: rename to setStreamingSoundId
-    //
     void setStreamSoundId(int id);
 
     /// Remove this sprite from the stage.
@@ -644,12 +597,8 @@ public:
     /// A testcases for this behaviour can be found in 
     ///
     /// testsuite/misc-ming.all/displaylist_depths_test.swf
-    ///
     void removeMovieClip();
 
-    /// Create a Bitmap DisplayObject at the specified depth.
-    void attachBitmap(boost::intrusive_ptr<BitmapData_as> bd, int depth);
-
     /// Render this MovieClip to a GnashImage using the passed transform
     //
     /// @return     The GnashImage with the MovieClip drawn onto it.
@@ -657,7 +606,7 @@ public:
             const SWFMatrix& mat = SWFMatrix(), 
             const cxform& cx = cxform(),
             DisplayObject::BlendMode bm = DisplayObject::BLENDMODE_NORMAL,
-            const rect& clipRect = rect(),
+            const SWFRect& clipRect = SWFRect(),
             bool smooth = false);
 
     /// @name Drawing API
@@ -667,9 +616,9 @@ public:
         bool vScale=true, bool hScale=true,
         bool pixelHinting=false,
         bool noClose=false,
-        cap_style_e startCapStyle=CAP_ROUND,
-        cap_style_e endCapStyle=CAP_ROUND,
-        join_style_e joinStyle=JOIN_ROUND,
+        CapStyle startCapStyle=CAP_ROUND,
+        CapStyle endCapStyle=CAP_ROUND,
+        JoinStyle joinStyle=JOIN_ROUND,
         float miterLimitFactor=1.0f)
     {
         _drawable.lineStyle(thickness, color, vScale, hScale,
@@ -713,14 +662,14 @@ public:
     void lineTo(boost::int32_t x, boost::int32_t y)
     {
         set_invalidated();
-        _drawable.lineTo(x, y, getMovieVersion());
+        _drawable.lineTo(x, y, getDefinitionVersion());
     }
 
     void curveTo(boost::int32_t cx, boost::int32_t cy, 
                  boost::int32_t ax, boost::int32_t ay)
     {
         set_invalidated();
-        _drawable.curveTo(cx, cy, ax, ay, getMovieVersion());
+        _drawable.curveTo(cx, cy, ax, ay, getDefinitionVersion());
     }
 
     void clear()
@@ -736,10 +685,8 @@ public:
 
     /// @} Drawing API
 
-    typedef std::map<std::string, std::string> VariableMap;
-
     /// Set all variables in the given map with their corresponding values
-    DSOEXPORT void setVariables(VariableMap& vars);
+    DSOEXPORT void setVariables(const MovieVariables& vars);
 
     /// Enumerate child DisplayObjects
     //
@@ -764,7 +711,6 @@ public:
     /// definition, either MovieClip or any user-speficied one
     /// (see sprite_definition::registerClass). 
     /// It will also invoke the onClipConstruct and onConstruct handlers.
-    ///
     void constructAsScriptObject();
 
     /// Return true if getAsRoot() should return the *relative* root,
@@ -775,17 +721,12 @@ public:
     /// false otherwise. True for relative root.
     void setLockRoot(bool lr) { _lockroot=lr; }
 
-    /// \brief
-    /// Return version of the SWF definition of this instance
-    /// as been parsed from.
-    //
-    int getMovieVersion() const;
+    /// Return the version of the SWF this MovieClip was parsed from.
+    virtual int getDefinitionVersion() const;
 
 protected:
 
-#ifdef GNASH_USE_GC
-    /// Mark sprite-specific reachable resources and invoke
-    /// the parent's class version (markDisplayObjectReachable)
+    /// Mark sprite-specific reachable resources.
     //
     /// sprite-specific reachable resources are:
     ///     - DisplayList items (current, backup and frame0 ones)
@@ -795,8 +736,7 @@ protected:
     /// - Textfields having an associated variable registered in this instance.
     /// - Relative root of this instance (_swf)
     ///
-    virtual void markReachableResources() const;
-#endif // GNASH_USE_GC
+    virtual void markOwnResources() const;
     
     // Used by BitmapMovie.
     void placeDisplayObject(DisplayObject* ch, int depth) {       
@@ -805,7 +745,7 @@ protected:
 
 private:
 
-    typedef std::vector<boost::intrusive_ptr<TextField> > TextFields;
+    typedef std::vector<TextField*> TextFields;
 
     /// A container for textfields, indexed by their variable name
     typedef std::map<std::string, TextFields> TextFieldIndex;
@@ -849,9 +789,6 @@ private:
     /// are disabled, and automatic tab ordering won't include it.
     bool isEnabled() const;
 
-    // See dox in DisplayObject.h
-    bool allowHandCursor() const;
-
     /// Check whether a point hits our drawable shape.
     //
     /// This is possible because the drawable does not have its own
