@@ -1,6 +1,7 @@
 // Video.cpp:  Draw individual video frames, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,12 +25,10 @@
 #include "fn_call.h"
 #include "as_value.h"
 #include "flash/net/NetStream_as.h"
-#include "Range2d.h"
 #include "builtin_function.h" // for getter/setter properties
 #include "NativeFunction.h" 
 #include "movie_root.h"
 #include "VM.h"
-#include "Object.h"
 #include "MediaHandler.h" // for setting up embedded video decoder 
 #include "VideoDecoder.h" // for setting up embedded video decoder
 #include "namedStrings.h"
@@ -44,7 +43,6 @@ namespace {
     as_object* getVideoInterface(as_object& where);
     void attachPrototypeProperties(as_object& o);
     void attachVideoInterface(as_object& o);
-    void attachVideoProperties(as_object& o);
     as_value video_ctor(const fn_call& fn);
     as_value video_attach(const fn_call& fn);
     as_value video_clear(const fn_call& fn);
@@ -54,26 +52,20 @@ namespace {
     as_value video_height(const fn_call& fn);
 }
 
-Video::Video(const SWF::DefineVideoStreamTag* const def,
-		DisplayObject* parent, int id)
+Video::Video(as_object* object,
+        const SWF::DefineVideoStreamTag* def, DisplayObject* parent)
 	:
-	DisplayObject(parent, id),
+	DisplayObject(getRoot(*object), object, parent),
 	m_def(def),
 	_ns(0),
-	_embeddedStream(m_def ? true : false),
+	_embeddedStream(m_def),
 	_lastDecodedVideoFrameNum(-1),
 	_lastDecodedVideoFrame(),
     _smoothing(false)
 {
-
-	set_prototype(getVideoInterface(*this));
-	if (_embeddedStream)
-	{
-		attachVideoProperties(*this);
-		initializeDecoder();
-        
-        attachPrototypeProperties(*get_prototype());
-	}
+    assert(object);
+    assert(def);
+    initializeDecoder();
 }
 
 Video::~Video()
@@ -137,12 +129,10 @@ Video::clear()
 void
 Video::display(Renderer& renderer)
 {
-	// if m_def is NULL we've been constructed by 'new Video', in this
-	// case I think display() would never be invoked on us...
 	assert(m_def);
 
 	SWFMatrix m = getWorldMatrix();
-	const rect& bounds = m_def->bounds();
+	const SWFRect& bounds = m_def->bounds();
 
 	GnashImage* img = getVideoFrame();
 	if (img)
@@ -257,7 +247,7 @@ Video::stagePlacementCallback(as_object* initObj)
     saveOriginalTarget(); // for softref
 
     // Register this video instance as a live DisplayObject
-    getRoot(*this).addLiveChar(this);
+    stage().addLiveChar(this);
 }
 
 
@@ -277,13 +267,9 @@ Video::add_invalidated_bounds(InvalidatedRanges& ranges, bool force)
     
 	ranges.add(m_old_invalidated_ranges);
 	
-	// NOTE: do not use m_def->boundss()
+	assert (m_def);
 
-	// if m_def is NULL we've been constructed by 'new Video', in this
-	// case I think add_invalidated_bouns would never be invoked on us...
-	assert ( m_def );
-
-	rect bounds;	
+	SWFRect bounds;	
 	bounds.expand_to_transformed_rect(getWorldMatrix(), m_def->bounds());
 	
 	ranges.add(bounds.getRange());            
@@ -301,12 +287,13 @@ void
 video_class_init(as_object& global, const ObjectURI& uri)
 {
 	// This is going to be the global Video "class"/"function"
-    Global_as* gl = getGlobal(global);
-    as_object* cl = gl->createClass(&video_ctor, getVideoInterface(global));
+    Global_as& gl = getGlobal(global);
+    as_object* proto = gl.createObject();
+    as_object* cl = gl.createClass(&video_ctor, proto);
+    attachVideoInterface(*proto);
 
 	// Register _global.Video
-	global.init_member(getName(uri), cl, as_object::DefaultFlags,
-            getNamespace(uri));
+	global.init_member(uri, cl, as_object::DefaultFlags);
 }
 
 void
@@ -318,43 +305,35 @@ registerVideoNative(as_object& global)
     vm.registerNative(video_clear, 667, 2);
 }
 
-rect
+SWFRect
 Video::getBounds() const
 {
 	if (_embeddedStream) return m_def->bounds();
 
 	// TODO: return the bounds of the dynamically
 	//       loaded video if not embedded ?
-	return rect();
+	return SWFRect();
 }
 
-#ifdef GNASH_USE_GC
 void
-Video::markReachableResources() const
+Video::markOwnResources() const
 {
 	if (_ns) _ns->setReachable();
-
-	// Invoke DisplayObject's version of reachability mark
-	markDisplayObjectReachable();
 }
-#endif // GNASH_USE_GC
-
-namespace {
 
 as_object*
-getVideoInterface(as_object& where)
+createVideoObject(Global_as& gl)
 {
-	static boost::intrusive_ptr<as_object> proto;
-	if ( proto == NULL )
-	{
-		proto = new as_object(getObjectInterface());
-		getVM(where).addStatic(proto.get());
-
-		attachVideoInterface(*proto);
-	}
-	return proto.get();
+    // TODO: how to use this for AS3 as well?
+    // Turn into constructBuiltin()
+    as_object* obj = getObjectWithPrototype(gl, NSV::CLASS_VIDEO);
+    as_object* proto = obj->get_prototype();
+    if (proto) attachPrototypeProperties(*proto);
+    return obj;
 }
 
+namespace {
+
 void
 attachVideoInterface(as_object& o)
 {
@@ -380,56 +359,10 @@ attachPrototypeProperties(as_object& proto)
     proto.init_property("width", &video_width, &video_width, flags);
 }
 
-void
-attachVideoProperties(as_object& o)
-{
-
-	as_c_function_ptr gettersetter;
-
-	gettersetter = &DisplayObject::x_getset;
-	o.init_property(NSV::PROP_uX, *gettersetter, *gettersetter);
-
-	gettersetter = &DisplayObject::y_getset;
-	o.init_property(NSV::PROP_uY, *gettersetter, *gettersetter);
-
-	gettersetter = &DisplayObject::xscale_getset;
-	o.init_property(NSV::PROP_uXSCALE, *gettersetter, *gettersetter);
-
-	gettersetter = &DisplayObject::yscale_getset;
-	o.init_property(NSV::PROP_uYSCALE, *gettersetter, *gettersetter);
-
-	gettersetter = &DisplayObject::xmouse_get;
-	o.init_readonly_property(NSV::PROP_uXMOUSE, *gettersetter);
-
-	gettersetter = &DisplayObject::ymouse_get;
-	o.init_readonly_property(NSV::PROP_uYMOUSE, *gettersetter);
-
-	gettersetter = &DisplayObject::alpha_getset;
-	o.init_property(NSV::PROP_uALPHA, *gettersetter, *gettersetter);
-
-	gettersetter = &DisplayObject::visible_getset;
-	o.init_property(NSV::PROP_uVISIBLE, *gettersetter, *gettersetter);
-
-	gettersetter = &DisplayObject::width_getset;
-	o.init_property(NSV::PROP_uWIDTH, *gettersetter, *gettersetter);
-
-	gettersetter = &DisplayObject::height_getset;
-	o.init_property(NSV::PROP_uHEIGHT, *gettersetter, *gettersetter);
-
-	gettersetter = &DisplayObject::rotation_getset;
-	o.init_property(NSV::PROP_uROTATION, *gettersetter, *gettersetter);
-
-	gettersetter = &DisplayObject::parent_getset;
-	o.init_property(NSV::PROP_uPARENT, *gettersetter, *gettersetter);
-
-	gettersetter = &DisplayObject::target_getset;
-	o.init_property(NSV::PROP_uTARGET, *gettersetter, *gettersetter);
-}
-
 as_value
 video_attach(const fn_call& fn)
 {
-	boost::intrusive_ptr<Video> video = ensureType<Video>(fn.this_ptr);
+	Video* video = ensure<IsDisplayObject<Video> >(fn);
 
 	if (fn.nargs < 1)
 	{
@@ -439,7 +372,7 @@ video_attach(const fn_call& fn)
 		return as_value();
 	}
 
-    as_object* obj = fn.arg(0).to_object(*getGlobal(fn)).get();
+    as_object* obj = fn.arg(0).to_object(getGlobal(fn));
 	NetStream_as* ns;
 
     if (isNativeType(obj, ns)) {
@@ -457,7 +390,7 @@ video_attach(const fn_call& fn)
 as_value
 video_deblocking(const fn_call& fn)
 {
-	boost::intrusive_ptr<Video> video = ensureType<Video>(fn.this_ptr);
+	Video* video = ensure<IsDisplayObject<Video> >(fn);
     UNUSED(video);
 
     log_unimpl("Video.deblocking");
@@ -467,7 +400,7 @@ video_deblocking(const fn_call& fn)
 as_value
 video_smoothing(const fn_call& fn)
 {
-	boost::intrusive_ptr<Video> video = ensureType<Video>(fn.this_ptr);
+	Video* video = ensure<IsDisplayObject<Video> >(fn);
 
     if (!fn.nargs) return as_value(video->smoothing());
 
@@ -481,21 +414,21 @@ video_smoothing(const fn_call& fn)
 as_value
 video_width(const fn_call& fn)
 {
-	boost::intrusive_ptr<Video> video = ensureType<Video>(fn.this_ptr);
+	Video* video = ensure<IsDisplayObject<Video> >(fn);
     return as_value(video->width());
 }
 
 as_value
 video_height(const fn_call& fn)
 {
-	boost::intrusive_ptr<Video> video = ensureType<Video>(fn.this_ptr);
+	Video* video = ensure<IsDisplayObject<Video> >(fn);
     return as_value(video->height());
 }
 
 as_value
 video_clear(const fn_call& fn)
 {
-	boost::intrusive_ptr<Video> video = ensureType<Video>(fn.this_ptr);
+	Video* video = ensure<IsDisplayObject<Video> >(fn);
 
     video->clear();
     return as_value();
@@ -504,13 +437,7 @@ video_clear(const fn_call& fn)
 as_value
 video_ctor(const fn_call& /* fn */)
 {
-	log_debug("new Video() TESTING !");
-
-	// I'm not sure We can rely on the def and parent values being accepted 
-    // as NULL. Not till we add some testing...
-	boost::intrusive_ptr<DisplayObject> obj = new Video(NULL, NULL, -1);
-	obj->setDynamic();
-	return as_value(obj.get()); // will keep alive
+	return as_value();
 }
 
 } // anonymous namespace
