@@ -1,6 +1,7 @@
 // AsBroadcaster.cpp - AsBroadcaster AS interface
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +18,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "Array_as.h" // for _listeners construction
 #include "log.h"
 #include "AsBroadcaster.h"
@@ -28,7 +25,6 @@
 #include "builtin_function.h"
 #include "NativeFunction.h" 
 #include "VM.h" // for getPlayerVersion() 
-#include "Object.h" // for getObjectInterface
 #include "Global_as.h"
 #include "namedStrings.h"
 
@@ -84,18 +80,18 @@ public:
     }
 
     /// Call a method on the given value
-    void visit(as_value& v)
+    void operator()(const as_value& v)
     {
-        boost::intrusive_ptr<as_object> o = v.to_object(*getGlobal(_fn));
+        boost::intrusive_ptr<as_object> o = v.to_object(getGlobal(_fn));
         if ( ! o ) return;
 
         as_value method;
         o->get_member(_eventKey, &method);
-        _fn.super = o->get_super(_eventName.c_str());
+        _fn.super = o->get_super(_eventKey);
 
         if (method.is_function()) {
             _fn.this_ptr = o.get();
-            method.to_as_function()->call(_fn);
+            method.to_function()->call(_fn);
         }
 
         ++_dispatched;
@@ -116,16 +112,18 @@ public:
 void 
 AsBroadcaster::initialize(as_object& o)
 {
-    Global_as* gl = getGlobal(o);
+    Global_as& gl = getGlobal(o);
 
     // Find _global.AsBroadcaster.
     as_object* asb =
-        gl->getMember(NSV::CLASS_AS_BROADCASTER).to_object(*gl).get();
+        gl.getMember(NSV::CLASS_AS_BROADCASTER).to_object(gl);
 
     // If it's not an object, these are left undefined, but they are
     // always attached to the initialized object.
     as_value al, rl;
 
+    const int flags = as_object::DefaultFlags;
+
     if (asb) {
         al = asb->getMember(NSV::PROP_ADD_LISTENER);
         rl = asb->getMember(NSV::PROP_REMOVE_LISTENER);
@@ -136,10 +134,18 @@ AsBroadcaster::initialize(as_object& o)
     
     // The function returned by ASnative(101, 12) is attached, even though
     // this may not exist (e.g. if _global.ASnative is altered)
-    const as_value& asn = gl->callMethod(NSV::PROP_AS_NATIVE, 101, 12);
+    const as_value& asn = callMethod(&gl, NSV::PROP_AS_NATIVE, 101, 12);
     o.set_member(NSV::PROP_BROADCAST_MESSAGE, asn);
 
-    o.set_member(NSV::PROP_uLISTENERS, new Array_as());
+    // This corresponds to  "_listeners = [];", which is different from
+    // _listeners = new Array();
+    o.set_member(NSV::PROP_uLISTENERS, gl.createArray());
+ 
+    // This function should call ASSetPropFlags on these four properties.
+    o.set_member_flags(NSV::PROP_BROADCAST_MESSAGE, flags);
+    o.set_member_flags(NSV::PROP_ADD_LISTENER, flags);
+    o.set_member_flags(NSV::PROP_REMOVE_LISTENER, flags);
+    o.set_member_flags(NSV::PROP_uLISTENERS, flags);
 
 }
 
@@ -150,14 +156,14 @@ attachAsBroadcasterStaticInterface(as_object& o)
                       PropFlags::dontDelete |
                       PropFlags::onlySWF6Up;
 
-    Global_as* gl = getGlobal(o);
+    Global_as& gl = getGlobal(o);
 
     o.init_member("initialize",
-            gl->createFunction(asbroadcaster_initialize), flags);
+            gl.createFunction(asbroadcaster_initialize), flags);
     o.init_member(NSV::PROP_ADD_LISTENER,
-            gl->createFunction(asbroadcaster_addListener), flags);
+            gl.createFunction(asbroadcaster_addListener), flags);
     o.init_member(NSV::PROP_REMOVE_LISTENER,
-            gl->createFunction(asbroadcaster_removeListener), flags);
+            gl.createFunction(asbroadcaster_removeListener), flags);
 
     VM& vm = getVM(o);
     o.init_member(NSV::PROP_BROADCAST_MESSAGE, vm.getNative(101, 12),
@@ -209,7 +215,7 @@ asbroadcaster_initialize(const fn_call& fn)
         return as_value();
     }
 
-    boost::intrusive_ptr<as_object> tgt = tgtval.to_object(*getGlobal(fn));
+    boost::intrusive_ptr<as_object> tgt = tgtval.to_object(getGlobal(fn));
     if ( ! tgt )
     {
         IF_VERBOSE_ASCODING_ERRORS(
@@ -226,12 +232,13 @@ asbroadcaster_initialize(const fn_call& fn)
 as_value
 asbroadcaster_addListener(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = fn.this_ptr;
+
+    as_object* obj = ensure<ValidThis>(fn);
 
     as_value newListener; assert(newListener.is_undefined());
     if ( fn.nargs ) newListener = fn.arg(0);
 
-    obj->callMethod(NSV::PROP_REMOVE_LISTENER, newListener);
+    callMethod(obj, NSV::PROP_REMOVE_LISTENER, newListener);
 
     as_value listenersValue;
 
@@ -256,30 +263,16 @@ asbroadcaster_addListener(const fn_call& fn)
                 "an object: %s"), (void*)fn.this_ptr, fn.dump_args(),
                 listenersValue);
         );
-        return as_value(false); // TODO: check this
+        // TODO: check this
+        return as_value(false); 
     }
 
-    boost::intrusive_ptr<as_object> listenersObj =
-        listenersValue.to_object(*getGlobal(fn));
-    assert(listenersObj);
+    as_object* listeners = listenersValue.to_object(getGlobal(fn));
 
-    boost::intrusive_ptr<Array_as> listeners = boost::dynamic_pointer_cast<Array_as>(listenersObj);
-    if ( ! listeners )
-    {
-        IF_VERBOSE_ASCODING_ERRORS(
-        log_aserror(_("%p.addListener(%s): this object's _listener isn't "
-                "an array: %s -- will call 'push' on it anyway"),
-                (void*)fn.this_ptr,
-                fn.dump_args(), listenersValue);
-        );
+    // We checked is_object() above.
+    assert(listeners); 
 
-        listenersObj->callMethod(NSV::PROP_PUSH, newListener);
-
-    }
-    else
-    {
-        listeners->push(newListener);
-    }
+    callMethod(listeners, NSV::PROP_PUSH, newListener);
 
     return as_value(true);
 
@@ -317,53 +310,33 @@ asbroadcaster_removeListener(const fn_call& fn)
         return as_value(false); // TODO: check this
     }
 
-    boost::intrusive_ptr<as_object> listenersObj =
-        listenersValue.to_object(*getGlobal(fn));
-    assert(listenersObj);
+    as_object* listeners = listenersValue.to_object(getGlobal(fn));
+    assert(listeners);
 
-    as_value listenerToRemove; assert(listenerToRemove.is_undefined());
-    if ( fn.nargs ) listenerToRemove = fn.arg(0);
+    as_value listenerToRemove; 
+    if (fn.nargs) listenerToRemove = fn.arg(0);
 
-    boost::intrusive_ptr<Array_as> listeners = 
-        boost::dynamic_pointer_cast<Array_as>(listenersObj);
-    if ( ! listeners )
-    {
-        IF_VERBOSE_ASCODING_ERRORS(
-        log_aserror(_("%p.addListener(%s): this object's _listener isn't an "
-                "array: %s"), (void*)fn.this_ptr, fn.dump_args(),
-                listenersValue);
-        );
+    // Remove the first listener matching the new value
+    // See http://www.senocular.com/flash/tutorials/
+    // listenersasbroadcaster/?page=2
+    
+    // This is an ActionScript-like implementation, which is why it looks
+    // like poor C++.
+    const int length = toInt(listeners->getMember(NSV::PROP_LENGTH));
+    int i = 0;
+    string_table& st = getStringTable(fn);
 
-        // TODO: implement brute force scan of pseudo-array
-        unsigned int length = 
-            listenersObj->getMember(NSV::PROP_LENGTH).to_int();
-
-        string_table& st = getStringTable(fn);
-
-        for (unsigned int i=0; i<length; ++i)
-        {
-            as_value iVal(i);
-            std::string n = iVal.to_string();
-            as_value v = listenersObj->getMember(st.find(n));
-            if ( v.equals(listenerToRemove) )
-            {
-                listenersObj->callMethod(NSV::PROP_SPLICE, iVal, as_value(1));
-                return as_value(true); 
-            }
+    while (i < length) {
+        std::ostringstream s;
+        s << i;
+        as_value el = listeners->getMember(st.find(s.str()));
+        if (el.equals(listenerToRemove)) {
+            callMethod(listeners, NSV::PROP_SPLICE, s.str(), 1);
+            return as_value(true);
         }
-
-        return as_value(false); // TODO: check this
-    }
-    else
-    {
-        // Remove the first listener matching the new value
-        // See http://www.senocular.com/flash/tutorials/
-        // listenersasbroadcaster/?page=2
-        // TODO: make this call as a normal (don't want to
-        // rely on _listeners type at all)
-        bool removed = listeners->removeFirst(listenerToRemove);
-        return as_value(removed);
+        ++i;
     }
+    return as_value(false);
 
 }
 
@@ -371,7 +344,7 @@ asbroadcaster_removeListener(const fn_call& fn)
 as_value
 asbroadcaster_broadcastMessage(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = fn.this_ptr;
+    as_object* obj = ensure<ValidThis>(fn);
 
     as_value listenersValue;
 
@@ -379,12 +352,10 @@ asbroadcaster_broadcastMessage(const fn_call& fn)
     //       inheritance chain in case its own property _listeners 
     //       has been deleted while another one is found in any base
     //       class.
-    if ( ! obj->get_member(NSV::PROP_uLISTENERS, &listenersValue) )
-    {
+    if (!obj->get_member(NSV::PROP_uLISTENERS, &listenersValue)) {
         IF_VERBOSE_ASCODING_ERRORS(
             log_aserror(_("%p.addListener(%s): this object has no "
-                    "_listeners member"), (void*)fn.this_ptr,
-                    fn.dump_args());
+                    "_listeners member"), obj, fn.dump_args());
         );
         return as_value(); // TODO: check this
     }
@@ -400,22 +371,9 @@ asbroadcaster_broadcastMessage(const fn_call& fn)
         return as_value(); // TODO: check this
     }
 
-    boost::intrusive_ptr<Array_as> listeners =
-        boost::dynamic_pointer_cast<Array_as>(
-                listenersValue.to_object(*getGlobal(fn)));
+    as_object* listeners = listenersValue.to_object(getGlobal(fn));
 
-    if ( ! listeners )
-    {
-        IF_VERBOSE_ASCODING_ERRORS(
-        log_aserror(_("%p.addListener(%s): this object's _listener "
-                "isn't an array: %s"), (void*)fn.this_ptr,
-                fn.dump_args(), listenersValue);
-        );
-        return as_value(); // TODO: check this
-    }
-
-    if (!fn.nargs)
-    {
+    if (!fn.nargs) {
         IF_VERBOSE_ASCODING_ERRORS(
         log_aserror("%p.broadcastMessage() needs an argument", 
             (void*)fn.this_ptr);
@@ -424,11 +382,11 @@ asbroadcaster_broadcastMessage(const fn_call& fn)
     }
 
     BroadcasterVisitor visitor(fn); 
-    listeners->visitAll(visitor);
+    foreachArray(*listeners, visitor);
 
-    unsigned int dispatched = visitor.eventsDispatched();
+    const size_t dispatched = visitor.eventsDispatched();
 
-    if ( dispatched ) return as_value(true);
+    if (dispatched) return as_value(true);
 
     return as_value(); 
 
