@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,23 +17,19 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifndef __GNASH_ASOBJ_CUSTOMACTIONS_H__
-#define __GNASH_ASOBJ_CUSTOMACTIONS_H__
-
-#include <memory> // for auto_ptr
+#ifndef GNASH_ASOBJ_CUSTOMACTIONS_H
+#define GNASH_ASOBJ_CUSTOMACTIONS_H
 
 namespace gnash {
 
 class as_object;
+class ObjectURI;
 
 /// Initialize the global CustomActions class
-void customactions_class_init(as_object& global);
+void customactions_class_init(as_object& global, const ObjectURI& uri);
 
-/// Return a CustomActions instance (in case the core lib needs it)
-//std::auto_ptr<as_object> init_customactions_instance();
   
 } // end of gnash namespace
 
-// __GNASH_ASOBJ_CUSTOMACTIONS_H__
 #endif
 
