@@ -1,6 +1,7 @@
 // Date_as.cpp:  ActionScript class for date and time, for Gnash.
 // 
-//  Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//  Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -24,7 +25,7 @@
 // Flash player handles a huge range of dates, including
 // thousands of years BC. The timestamp value is correspondingly
 // large: it is a double, which has a minimum size of 8 bytes
-// in the C++ standard. Methods provided by ctime and sys/time.h
+// in the C++ standard. Methods provided by <ctime>
 // generally rely on time_t whose size varies according to platform.
 // It is not big enough to deal with all valid SWF timestamps,
 // so this class uses its own methods to convert to and from 
@@ -63,17 +64,14 @@
 //    not to use it for time and date functions (though for portable
 //    timing it might well be useful).
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "log.h"
-#include "utility.h"
+#include "GnashNumeric.h"
 #include "Date_as.h"
 #include "fn_call.h"
+#include "Global_as.h"
 #include "GnashException.h"
 #include "builtin_function.h"
-#include "Object.h" // for getObjectInterface
+#include "NativeFunction.h" 
 #include "ClockTime.h"
 #include "VM.h"
 
@@ -109,7 +107,6 @@ namespace {
 
     // Forward declarations
     
-    as_object* getDateInterface();
     void attachDateInterface(as_object& o);
     void attachDateStaticInterface(as_object& o);
 
@@ -144,7 +141,6 @@ namespace {
     template<bool utc> as_value date_setSeconds(const fn_call& fn);
     as_value date_setYear(const fn_call& fn);
     as_value date_tostring(const fn_call& fn);
-    as_value date_valueof(const fn_call& fn);
     as_value date_UTC(const fn_call& fn);
 
     void fillGnashTime(double time, GnashTime& gt);
@@ -162,7 +158,6 @@ namespace {
 
 Date_as::Date_as(double value)
     :
-    as_object(getDateInterface()),
     _timeValue(value)
 {
 }
@@ -207,27 +202,31 @@ Date_as::toString() const
 }
 
 void
-Date_as::init(as_object& global)
+date_class_init(as_object& global, const ObjectURI& uri)
 {
-    // This is going to be the global Date "class"/"function"
-    static boost::intrusive_ptr<builtin_function> cl;
 
-    if ( cl == NULL ) {
-        cl = new builtin_function(&date_new, getDateInterface());
-        
-        // replicate static interface to class (Date.UTC)
-        attachDateStaticInterface(*cl);
-    }
+    Global_as& gl = getGlobal(global);
+    as_object* proto = gl.createObject();
+    as_object* cl = gl.createClass(&date_new, proto);
+    attachDateInterface(*proto);
+    
+    const int flags = PropFlags::readOnly;
+    cl->set_member_flags(NSV::PROP_uuPROTOuu, flags);
+    cl->set_member_flags(NSV::PROP_CONSTRUCTOR, flags);
+    cl->set_member_flags(NSV::PROP_PROTOTYPE, flags);
+
+    // Attach static interface to class (Date.UTC)
+    attachDateStaticInterface(*cl);
 
     // Register _global.Date
-    global.init_member("Date", cl.get());
+    global.init_member(uri, cl, as_object::DefaultFlags);
 
 }
 
 void
-Date_as::registerNative(as_object& global)
+registerDateNative(as_object& global)
 {
-    VM& vm = global.getVM();
+    VM& vm = getVM(global);
 
     vm.registerNative(date_getFullYear, 103, 0); 
     vm.registerNative(date_getYear, 103, 1);
@@ -336,7 +335,7 @@ void truncateDouble(T& target, double value)
 void
 attachDateInterface(as_object& o)
 {
-    VM& vm = o.getVM();
+    VM& vm = getVM(o);
 
     o.init_member("getFullYear", vm.getNative(103, 0));
     o.init_member("getYear", vm.getNative(103, 1));
@@ -375,31 +374,20 @@ attachDateInterface(as_object& o)
     o.init_member("setUTCMinutes", vm.getNative(103, 141));
     o.init_member("setUTCSeconds", vm.getNative(103, 142));
     o.init_member("setUTCMilliseconds", vm.getNative(103, 143));
-    o.init_member("valueOf", new builtin_function(date_valueof));
+
+    string_table& st = getStringTable(o);
+    o.init_member("valueOf", o.getMember(st.find("getTime")));
 
 }   
 
 void
 attachDateStaticInterface(as_object& o)
 {
-    VM& vm = o.getVM();
-    o.init_member("UTC", vm.getNative(103, 257));
+    VM& vm = getVM(o);
+    const int flags = as_object::DefaultFlags | PropFlags::readOnly;
+    o.init_member("UTC", vm.getNative(103, 257), flags);
 }
 
-as_object*
-getDateInterface()
-{
-    static boost::intrusive_ptr<as_object> o;
-    if ( !o )
-    {
-        o = new as_object(getObjectInterface());
-        VM::get().addStatic(o.get());
-        attachDateInterface(*o);
-    }
-    return o.get();
-}
-
-
 /// \brief Date constructor
 //
 /// The constructor has three forms: 0 args, 1 arg and 2-7 args.
@@ -418,7 +406,17 @@ as_value
 date_new(const fn_call& fn)
 {
 
-	boost::intrusive_ptr<Date_as> date;
+    as_object* obj = fn.this_ptr;
+
+    // The Date ctor called as a conversion function constructs a new
+    // date.
+    if (!fn.isInstantiation()) {
+        Global_as& gl = getGlobal(fn);
+        as_function* ctor = gl.getMember(NSV::CLASS_DATE).to_function();
+        if (!ctor) return as_value();
+        fn_call::Args args;
+        return constructInstance(*ctor, fn.env(), args);
+    }
 
     // Reject all date specifications containing Infinities and NaNs.
     // The commercial player does different things according to which
@@ -426,17 +424,17 @@ date_new(const fn_call& fn)
     // for now, we just use rogue_date_args' algorithm
     double foo;
     if (( foo = rogue_date_args(fn, 7)) != 0.0) {
-        date = new Date_as(foo);
-        return as_value(date.get());
+        obj->setRelay(new Date_as(foo));
+        return as_value();
     }
 
-    if (fn.nargs < 1 || fn.arg(0).is_undefined() || !(fn.isInstantiation()) ) {
+    if (fn.nargs < 1 || fn.arg(0).is_undefined()) {
         // Time now
-        date = new Date_as;
+        obj->setRelay(new Date_as);
     }
     else if (fn.nargs == 1) {
         // Set the value in milliseconds since 1970 UTC
-        date = new Date_as(fn.arg(0).to_number());
+        obj->setRelay(new Date_as(fn.arg(0).to_number()));
     }
     else {
         // Create a time from the supplied (at least 2) arguments.
@@ -447,9 +445,9 @@ date_new(const fn_call& fn)
         gt.minute = 0;
         gt.hour = 0;
         gt.monthday = 1;
-        gt.month = fn.arg(1).to_int();
+        gt.month = toInt(fn.arg(1));
     
-        int year = fn.arg(0).to_int();
+        int year = toInt(fn.arg(0));
         
         // GnashTime.year is the value since 1900 (like struct tm)
         // negative value is a year before 1900. A year between 0
@@ -468,15 +466,15 @@ date_new(const fn_call& fn)
                 )
             case 7:
                 // fractions of milliseconds are ignored
-                gt.millisecond = fn.arg(6).to_int();
+                gt.millisecond = toInt(fn.arg(6));
             case 6:
-                gt.second = fn.arg(5).to_int();
+                gt.second = toInt(fn.arg(5));
             case 5:
-                gt.minute = fn.arg(4).to_int();
+                gt.minute = toInt(fn.arg(4));
             case 4:
-                gt.hour = fn.arg(3).to_int();
+                gt.hour = toInt(fn.arg(3));
             case 3:
-                gt.monthday = fn.arg(2).to_int();
+                gt.monthday = toInt(fn.arg(2));
             case 2:
                 break;
                 // Done already
@@ -487,11 +485,11 @@ date_new(const fn_call& fn)
         // due to shortcomings in the timezoneoffset calculation, but should
         // be internally consistent.
         double localTime = makeTimeValue(gt);
-        date = new Date_as(
-                localTime - clocktime::getTimeZoneOffset(localTime) * 60000);
+        obj->setRelay(new Date_as(
+                localTime - clocktime::getTimeZoneOffset(localTime) * 60000));
     }
     
-    return as_value(date.get());
+    return as_value();
 }
 
 //
@@ -535,7 +533,7 @@ inline as_value timeElement(T dateFunc, boost::int32_t GnashTime::* element,
 as_value
 date_getYear(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(localTime, &GnashTime::year, date->getTimeValue());
 }
 
@@ -544,7 +542,7 @@ date_getYear(const fn_call& fn)
 as_value
 date_getFullYear(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(
             localTime, &GnashTime::year, date->getTimeValue(), 1900);
 }
@@ -554,7 +552,7 @@ date_getFullYear(const fn_call& fn)
 as_value
 date_getMonth(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(localTime, &GnashTime::month, date->getTimeValue());
 }
 
@@ -563,7 +561,7 @@ date_getMonth(const fn_call& fn)
 as_value
 date_getDate(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(localTime, &GnashTime::monthday, date->getTimeValue());
 }
 
@@ -573,7 +571,7 @@ date_getDate(const fn_call& fn)
 as_value
 date_getDay(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(localTime, &GnashTime::weekday, date->getTimeValue());
 }
 
@@ -583,7 +581,7 @@ date_getDay(const fn_call& fn)
 as_value
 date_getHours(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(localTime, &GnashTime::hour, date->getTimeValue());
 }
 
@@ -594,7 +592,7 @@ date_getHours(const fn_call& fn)
 as_value
 date_getMinutes(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(localTime, &GnashTime::minute, date->getTimeValue());
 }
 
@@ -604,7 +602,7 @@ date_getMinutes(const fn_call& fn)
 as_value
 date_getSeconds(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(localTime, &GnashTime::second, date->getTimeValue());
 }
 
@@ -616,7 +614,7 @@ date_getSeconds(const fn_call& fn)
 as_value
 date_getMilliseconds(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(
             localTime, &GnashTime::millisecond, date->getTimeValue());
 }
@@ -627,7 +625,7 @@ date_getMilliseconds(const fn_call& fn)
 as_value
 date_getUTCFullYear(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(universalTime, &GnashTime::year,
            date->getTimeValue(), 1900);
 }
@@ -635,21 +633,21 @@ date_getUTCFullYear(const fn_call& fn)
 as_value
 date_getUTCYear(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(universalTime, &GnashTime::year, date->getTimeValue());
 }
 
 as_value
 date_getUTCMonth(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(universalTime, &GnashTime::month, date->getTimeValue());
 }
 
 as_value
 date_getutcdate(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(
             universalTime, &GnashTime::monthday, date->getTimeValue());
 }
@@ -658,7 +656,7 @@ date_getutcdate(const fn_call& fn)
 as_value
 date_getUTCDay(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(
                 universalTime, &GnashTime::weekday, date->getTimeValue());
 }
@@ -666,7 +664,7 @@ date_getUTCDay(const fn_call& fn)
 as_value
 date_getUTCHours(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(
                 universalTime, &GnashTime::hour, date->getTimeValue());
 }
@@ -674,7 +672,7 @@ date_getUTCHours(const fn_call& fn)
 as_value
 date_getUTCMinutes(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return timeElement(universalTime, &GnashTime::minute, date->getTimeValue());
 }
 
@@ -697,7 +695,7 @@ localTimeZoneOffset(double time)
 as_value
 date_getTimezoneOffset(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return as_value(-localTimeZoneOffset(date->getTimeValue()));
 }
 
@@ -719,7 +717,7 @@ date_getTimezoneOffset(const fn_call& fn)
 as_value
 date_setTime(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
 
     if (fn.nargs < 1 || fn.arg(0).is_undefined()) {
         IF_VERBOSE_ASCODING_ERRORS(
@@ -732,7 +730,7 @@ date_setTime(const fn_call& fn)
         const double magicMaxValue = 8.64e+15;
         double d = fn.arg(0).to_number();
 
-        if (!utility::isFinite(d) || std::abs(d) > magicMaxValue) {
+        if (!isFinite(d) || std::abs(d) > magicMaxValue) {
             date->setTimeValue(NaN);
         }
         else {
@@ -826,12 +824,11 @@ dateToGnashTime(Date_as& date, GnashTime& gt, bool utc)
 //
 // Heaven knows what happens if it is 1.30 localtime and you change the date
 // to the day the clocks go forward.
-
 template<bool utc>
 as_value
 date_setfullyear(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
 
     if (fn.nargs < 1) {
         IF_VERBOSE_ASCODING_ERRORS(
@@ -844,22 +841,10 @@ date_setfullyear(const fn_call& fn)
     }
     else {
         GnashTime gt;
-
         dateToGnashTime(*date, gt, utc);
-        gt.year = fn.arg(0).to_int() - 1900;
-        switch (fn.nargs)
-        {
-            default:
-                IF_VERBOSE_ASCODING_ERRORS(
-                    log_aserror(_("Date.set%sFullYear was called with "
-                        "more than three arguments"), utc ? "UTC" : "");
-                );
-    
-            case 3:
-                gt.monthday = fn.arg(2).to_int();
-            case 2:
-                gt.month = fn.arg(1).to_int();
-        }
+        gt.year = toInt(fn.arg(0)) - 1900;
+        if (fn.nargs >= 2) gt.month = toInt(fn.arg(1));
+        if (fn.nargs >= 3) gt.monthday = toInt(fn.arg(2));
         gnashTimeToDate(gt, *date, utc);
   }
   return as_value(date->getTimeValue());
@@ -882,7 +867,7 @@ date_setfullyear(const fn_call& fn)
 as_value
 date_setYear(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
 
     // assert(fn.nargs == 1);
     if (fn.nargs < 1) {
@@ -906,8 +891,8 @@ date_setYear(const fn_call& fn)
 
         truncateDouble(gt.year, year);
 
-        if (fn.nargs >= 2) gt.month = fn.arg(1).to_int();
-        if (fn.nargs >= 3) gt.monthday = fn.arg(2).to_int();
+        if (fn.nargs >= 2) gt.month = toInt(fn.arg(1));
+        if (fn.nargs >= 3) gt.monthday = toInt(fn.arg(2));
         if (fn.nargs > 3) {
             IF_VERBOSE_ASCODING_ERRORS(
                 log_aserror(_("Date.setYear was called with more than three "
@@ -936,7 +921,7 @@ template<bool utc>
 as_value
 date_setmonth(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
 
     // assert(fn.nargs >= 1 && fn.nargs <= 2);
     if (fn.nargs < 1) {
@@ -992,7 +977,7 @@ template<bool utc>
 as_value
 date_setDate(const fn_call& fn)
 {
-  boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+  Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
 
   if (fn.nargs < 1) {
       IF_VERBOSE_ASCODING_ERRORS(
@@ -1005,7 +990,7 @@ date_setDate(const fn_call& fn)
     GnashTime gt;
 
     dateToGnashTime(*date, gt, utc);
-    gt.monthday = fn.arg(0).to_int();
+    gt.monthday = toInt(fn.arg(0));
     gnashTimeToDate(gt, *date, utc);
   }
   if (fn.nargs > 1) {
@@ -1032,7 +1017,7 @@ template<bool utc>
 as_value
 date_setHours(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
 
     // assert(fn.nargs >= 1 && fn.nargs <= 4);
     if (fn.nargs < 1) {
@@ -1050,10 +1035,10 @@ date_setHours(const fn_call& fn)
         GnashTime gt;
 
         dateToGnashTime(*date, gt, utc);
-        gt.hour = fn.arg(0).to_int();
-        if (fn.nargs >= 2) gt.minute = fn.arg(1).to_int();
-        if (fn.nargs >= 3) gt.second = fn.arg(2).to_int();
-        if (fn.nargs >= 4) gt.millisecond = fn.arg(3).to_int();
+        gt.hour = toInt(fn.arg(0));
+        if (fn.nargs >= 2) gt.minute = toInt(fn.arg(1));
+        if (fn.nargs >= 3) gt.second = toInt(fn.arg(2));
+        if (fn.nargs >= 4) gt.millisecond = toInt(fn.arg(3));
         if (fn.nargs > 4) {
             IF_VERBOSE_ASCODING_ERRORS(
                 log_aserror(_("Date.set%sHours was called with more than "
@@ -1077,7 +1062,7 @@ template<bool utc>
 as_value
 date_setMinutes(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
 
     //assert(fn.nargs >= 1 && fn.nargs <= 3);
     if (fn.nargs < 1) {
@@ -1095,9 +1080,9 @@ date_setMinutes(const fn_call& fn)
         GnashTime gt;
 
         dateToGnashTime(*date, gt, utc);
-        gt.minute = fn.arg(0).to_int();
-        if (fn.nargs >= 2) gt.second = fn.arg(1).to_int();
-        if (fn.nargs >= 3) gt.millisecond = fn.arg(2).to_int();
+        gt.minute = toInt(fn.arg(0));
+        if (fn.nargs >= 2) gt.second = toInt(fn.arg(1));
+        if (fn.nargs >= 3) gt.millisecond = toInt(fn.arg(2));
         if (fn.nargs > 3) {
             IF_VERBOSE_ASCODING_ERRORS(
                 log_aserror(_("Date.set%sMinutes was called with more than "
@@ -1118,7 +1103,7 @@ template<bool utc>
 as_value
 date_setSeconds(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
 
     // assert(fn.nargs >= 1 && fn.nargs <= 2);
     if (fn.nargs < 1) {
@@ -1139,8 +1124,8 @@ date_setSeconds(const fn_call& fn)
         GnashTime gt;
 
         dateToGnashTime(*date, gt, utc);
-        gt.second = fn.arg(0).to_int();
-        if (fn.nargs >= 2) gt.millisecond = fn.arg(1).to_int();
+        gt.second = toInt(fn.arg(0));
+        if (fn.nargs >= 2) gt.millisecond = toInt(fn.arg(1));
         if (fn.nargs > 2) {
             IF_VERBOSE_ASCODING_ERRORS(
                 log_aserror(_("Date.set%sMinutes was called with more than "
@@ -1157,7 +1142,7 @@ template<bool utc>
 as_value
 date_setMilliseconds(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
 
     if (fn.nargs < 1) {
         IF_VERBOSE_ASCODING_ERRORS(
@@ -1199,7 +1184,7 @@ date_setMilliseconds(const fn_call& fn)
 as_value
 date_tostring(const fn_call& fn) 
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return as_value(date->toString());
 }
 
@@ -1268,17 +1253,17 @@ date_UTC(const fn_call& fn) {
             )
         case 7:
             // millisecs is double, but fractions of millisecs are ignored.
-            gt.millisecond = fn.arg(6).to_int();
+            gt.millisecond = toInt(fn.arg(6));
         case 6:
-            gt.second = fn.arg(5).to_int();
+            gt.second = toInt(fn.arg(5));
         case 5:
-            gt.minute = fn.arg(4).to_int();
+            gt.minute = toInt(fn.arg(4));
         case 4:
-            gt.hour = fn.arg(3).to_int();
+            gt.hour = toInt(fn.arg(3));
         case 3:
-            gt.monthday = fn.arg(2).to_int();
+            gt.monthday = toInt(fn.arg(2));
         case 2:   // these last two are always performed
-            gt.month = fn.arg(1).to_int();
+            gt.month = toInt(fn.arg(1));
             {
                 boost::int32_t year = 0;
                 truncateDouble(year, fn.arg(0).to_number());
@@ -1337,20 +1322,12 @@ rogue_date_args(const fn_call& fn, unsigned maxargs)
     return(0.0);
 }
 
-/// \brief Date.valueOf() returns the number of milliseconds since midnight
+/// \brief Date.getTime() returns the number of milliseconds since midnight
 /// January 1, 1970 00:00 UTC, for a Date. The return value can be a fractional
 /// number of milliseconds.
-as_value
-date_valueof(const fn_call& fn)
-{
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
-    return as_value(date->getTimeValue());
-}
-
-
 as_value date_getTime(const fn_call& fn)
 {
-    boost::intrusive_ptr<Date_as> date = ensureType<Date_as>(fn.this_ptr);
+    Date_as* date = ensure<ThisIsNative<Date_as> >(fn);
     return as_value(date->getTimeValue());
 }
 
