@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,7 +26,9 @@ namespace gnash {
 class as_object;
 
 /// Initialize the global MovieClipLoader class
-void moviecliploader_class_init(as_object& global);
+void moviecliploader_class_init(as_object& global, const ObjectURI& uri);
+
+void registerMovieClipLoaderNative(as_object& global);
 
 } // end of gnash namespace
 
