@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,12 +27,7 @@ class as_object;
 class ObjectURI;
 
 /// Initialize the global Object class
-void object_class_init(as_object& global, const ObjectURI& uri);
-
-/// Return an Object instance
-as_object* init_object_instance();
-
-as_object* getObjectInterface();
+void initObjectClass(as_object* proto, as_object& global, const ObjectURI& uri);
 
 void registerObjectNative(as_object& global);
 
