@@ -1,6 +1,7 @@
 // string.cpp:  ActionScript "String" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,24 +17,22 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "smart_ptr.h"
 #include "fn_call.h"
+#include "Global_as.h"
 #include "as_object.h"
 #include "builtin_function.h" // need builtin_function
+#include "NativeFunction.h" 
 #include "log.h"
-#include "Array_as.h"
 #include "as_value.h"
 #include "GnashException.h"
+#include "movie_definition.h" 
 #include "VM.h" 
-#include "Object.h" // for getObjectInterface
 #include "namedStrings.h"
 #include "utf8.h"
 #include "String_as.h"
+#include "GnashNumeric.h"
+#include "Global_as.h"
 
 #include <boost/algorithm/string/case_conv.hpp>
 #include <algorithm>
@@ -58,109 +57,71 @@ namespace {
     as_value string_toUpperCase(const fn_call& fn);
     as_value string_toLowerCase(const fn_call& fn);
     as_value string_toString(const fn_call& fn);
+    as_value string_valueOf(const fn_call& fn);
     as_value string_oldToLower(const fn_call& fn);
     as_value string_oldToUpper(const fn_call& fn);
     as_value string_ctor(const fn_call& fn);
 
     size_t validIndex(const std::wstring& subject, int index);
     void attachStringInterface(as_object& o);
-    as_object* getStringInterface();
-    boost::intrusive_ptr<builtin_function> getStringConstructor();
 
     inline bool checkArgs(const fn_call& fn, size_t min, size_t max,
             const std::string& function);
+
+    inline int getStringVersioned(const fn_call& fn, const as_value& arg,
+            std::string& str);
+
 }
 
-class String_as : public as_object
+String_as::String_as(const std::string& s)
+    :
+    _string(s)
 {
+}
 
-public:
-
-    explicit String_as(const std::string& s)
-            :
-            as_object(getStringInterface()),
-            _string(s)
-    {
-        std::wstring wstr = utf8::decodeCanonicalString(
-                _string, _vm.getSWFVersion());
-        init_member(NSV::PROP_LENGTH, wstr.size(), 
-                as_prop_flags::dontDelete | as_prop_flags::dontEnum); 
-    }
-
-
-    bool useCustomToString() const { return false; }
-
-    std::string get_text_value() const {
-        return _string;
-    }
-
-    as_value get_primitive_value() const {
-        return as_value(_string);
-    }
-
-    const std::string& str() {
-        return _string;
-    }
-
-private:
-    std::string _string;
-};
-
-boost::intrusive_ptr<as_object>
-init_string_instance(const std::string& val)
+void
+registerStringNative(as_object& global)
 {
-	// TODO: get VM from the environment ?
-	VM& vm = VM::get();
-
-	// TODO: get the environment passed in !!
-	as_environment env(vm);
-
-	int swfVersion = vm.getSWFVersion();
-
-	boost::intrusive_ptr<as_function> cl;
-
-	if ( swfVersion < 6 ) {
-		cl = getStringConstructor();
-	}
-	else {
-		as_object* global = vm.getGlobal();
-		as_value clval;
-		if ( ! global->get_member(NSV::CLASS_STRING, &clval) ) {
-			log_debug("UNTESTED: String instantiation requested but "
-                    "_global doesn't contain a 'String' symbol. Returning "
-                    "the NULL object.");
-			return cl;
-		}
-		else if ( ! clval.is_function() ) {
-			log_debug("UNTESTED: String instantiation requested but "
-                    "_global.String is not a function (%s). Returning "
-                    "the NULL object.", clval);
-			return cl;
-		}
-		else {
-			cl = clval.to_as_function();
-			assert(cl);
-		}
-	}
-
-	std::auto_ptr< std::vector<as_value> > args ( new std::vector<as_value> );
-	args->push_back(val);
-	boost::intrusive_ptr<as_object> ret = cl->constructInstance(env, args);
-
-	return ret;
+    VM& vm = getVM(global);
+    vm.registerNative(string_ctor, 251, 0);
+    vm.registerNative(string_valueOf, 251, 1);
+    vm.registerNative(string_toString, 251, 2);
+    vm.registerNative(string_oldToUpper, 102, 0);
+    vm.registerNative(string_toUpperCase, 251, 3);
+    vm.registerNative(string_oldToLower, 102, 1);
+    vm.registerNative(string_toLowerCase, 251, 4);
+    vm.registerNative(string_charAt, 251, 5);
+    vm.registerNative(string_charCodeAt, 251, 6);
+    vm.registerNative(string_concat, 251, 7);
+    vm.registerNative(string_indexOf, 251, 8);
+    vm.registerNative(string_lastIndexOf, 251, 9);
+    vm.registerNative(string_slice, 251, 10);
+    vm.registerNative(string_substring, 251, 11);
+    vm.registerNative(string_split, 251, 12);
+    vm.registerNative(string_substr, 251, 13);
+    vm.registerNative(string_fromCharCode, 251, 14);
 }
 
 // extern (used by Global.cpp)
-void string_class_init(as_object& global)
+void
+string_class_init(as_object& where, const ObjectURI& uri)
 {
     // This is going to be the global String "class"/"function"
-    boost::intrusive_ptr<builtin_function> cl = getStringConstructor();
+    
+    VM& vm = getVM(where);
+    Global_as& gl = getGlobal(where);
 
-    // Register _global.String (should be only visible from SWF5 up)
-    // TODO: register as ASnative(251, 0)
-    // TODO: register as ASnative(3, 0) for SWF5 ?
-    int flags = as_prop_flags::dontEnum; 
-    global.init_member("String", cl.get(), flags);
+    as_object* proto = gl.createObject();
+    as_object* cl = vm.getNative(251, 0);
+    cl->init_member(NSV::PROP_PROTOTYPE, proto);
+    proto->init_member(NSV::PROP_CONSTRUCTOR, cl);
+
+    attachStringInterface(*proto);
+
+    cl->init_member("fromCharCode", vm.getNative(251, 14)); 
+
+    const int flags = PropFlags::dontEnum; 
+    where.init_member(uri, cl, flags);
 }
 
 
@@ -170,95 +131,34 @@ namespace {
 void
 attachStringInterface(as_object& o)
 {
-	VM& vm = o.getVM();
+    VM& vm = getVM(o);
 
-	// ASnative(251, 1) - [String.prototype] valueOf
-	vm.registerNative(as_object::tostring_method, 251, 1);
-	o.init_member("valueOf", vm.getNative(251, 1));
-
-	// ASnative(251, 2) - [String.prototype] toString
-	vm.registerNative(string_toString, 251, 2);
-	o.init_member("toString", vm.getNative(251, 2));
-
-	// ASnative(251, 3) - [String.prototype] toUpperCase
-	// ASnative(102, 0) - SWF5 to upper.
-	vm.registerNative(string_oldToUpper, 102, 0);
-	vm.registerNative(string_toUpperCase, 251, 3);
-	o.init_member("toUpperCase", vm.getNative(251, 3));
-
-	// ASnative(251, 4) - [String.prototype] toLowerCase
-	// ASnative(102, 1) - SWF5 to lower.
-	vm.registerNative(string_oldToLower, 102, 1);
-	vm.registerNative(string_toLowerCase, 251, 4);
-	o.init_member("toLowerCase", vm.getNative(251, 4));
-
-	// ASnative(251, 5) - [String.prototype] charAt
-	vm.registerNative(string_charAt, 251, 5);
-	o.init_member("charAt", vm.getNative(251, 5));
-
-	// ASnative(251, 6) - [String.prototype] charCodeAt
-	vm.registerNative(string_charCodeAt, 251, 6);
-	o.init_member("charCodeAt", vm.getNative(251, 6));
-
-	// ASnative(251, 7) - [String.prototype] concat
-	vm.registerNative(string_concat, 251, 7);
-	o.init_member("concat", vm.getNative(251, 7));
-
-	// ASnative(251, 8) - [String.prototype] indexOf
-	vm.registerNative(string_indexOf, 251, 8);
-	o.init_member("indexOf", vm.getNative(251, 8));
-
-	// ASnative(251, 9) - [String.prototype] lastIndexOf
-	vm.registerNative(string_lastIndexOf, 251, 9);
-	o.init_member("lastIndexOf", vm.getNative(251, 9));
-
-	// ASnative(251, 10) - [String.prototype] slice
-	vm.registerNative(string_slice, 251, 10);
-	o.init_member("slice", vm.getNative(251, 10));
-
-	// ASnative(251, 11) - [String.prototype] substring
-	vm.registerNative(string_substring, 251, 11);
-	o.init_member("substring", vm.getNative(251, 11));
-
-	// ASnative(251, 12) - [String.prototype] split
-	vm.registerNative(string_split, 251, 12);
-	o.init_member("split", vm.getNative(251, 12));
-
-	// ASnative(251, 13) - [String.prototype] substr
-	vm.registerNative(string_substr, 251, 13);
-	o.init_member("substr", vm.getNative(251, 13));
-}
-
-as_object*
-getStringInterface()
-{
-    static boost::intrusive_ptr<as_object> o;
-
-    if ( o == NULL )
-    {
-        o = new as_object(getObjectInterface());
-	    VM::get().addStatic(o.get());
-
-        attachStringInterface(*o);
-    }
-
-    return o.get();
+    o.init_member("valueOf", vm.getNative(251, 1));
+    o.init_member("toString", vm.getNative(251, 2));
+    o.init_member("toUpperCase", vm.getNative(251, 3));
+    o.init_member("toLowerCase", vm.getNative(251, 4));
+    o.init_member("charAt", vm.getNative(251, 5));
+    o.init_member("charCodeAt", vm.getNative(251, 6));
+    o.init_member("concat", vm.getNative(251, 7));
+    o.init_member("indexOf", vm.getNative(251, 8));
+    o.init_member("lastIndexOf", vm.getNative(251, 9));
+    o.init_member("slice", vm.getNative(251, 10));
+    o.init_member("substring", vm.getNative(251, 11));
+    o.init_member("split", vm.getNative(251, 12));
+    o.init_member("substr", vm.getNative(251, 13));
 }
 
 // all the arguments will be converted to string and concatenated.
 as_value
 string_concat(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
-
-    const int swfVersion = fn.env().get_version();
-
     as_value val(fn.this_ptr);
-    
-    std::string str = val.to_string();
 
-    for (unsigned int i = 0; i < fn.nargs; i++) {
-        str += fn.arg(i).to_string_versioned(swfVersion);
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
+
+    for (size_t i = 0; i < fn.nargs; i++) {
+        str += fn.arg(i).to_string(version);
     }
 
     return as_value(str);
@@ -269,36 +169,22 @@ string_concat(const fn_call& fn)
 as_value
 string_slice(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    /// version to use is the one of the SWF containing caller code.
-    /// If callerDef is null, this calls is spontaneous (system-event?)
-    /// in which case we should research on which version should drive
-    /// behaviour.
-    /// NOTE: it is unlikely that a system event triggers string_split so
-    ///       in most cases a null callerDef means the caller forgot to 
-    ///       set the field (ie: a programmatic error)
-    if ( ! fn.callerDef )
-    {
-        log_error("No fn_call::callerDef in string_slice call");
-    }
-    const int version = fn.callerDef ? fn.callerDef->get_version() :
-        obj->getVM().getSWFVersion();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
     if (!checkArgs(fn, 1, 2, "String.slice()")) return as_value();
 
-    size_t start = validIndex(wstr, fn.arg(0).to_int());
+    size_t start = validIndex(wstr, toInt(fn.arg(0)));
 
     size_t end = wstr.length();
 
     if (fn.nargs >= 2)
     {
-    	end = validIndex(wstr, fn.arg(1).to_int());
+        end = validIndex(wstr, toInt(fn.arg(1)));
 
     } 
 
@@ -309,7 +195,7 @@ string_slice(const fn_call& fn)
 
     size_t retlen = end - start;
 
-    log_debug("start: %d, end: %d, retlen: %d", start, end, retlen);
+    //log_debug("start: %d, end: %d, retlen: %d", start, end, retlen);
 
     return as_value(utf8::encodeCanonicalString(
                 wstr.substr(start, retlen), version));
@@ -320,7 +206,7 @@ string_slice(const fn_call& fn)
 // element containing the entire string is returned:
 // 1. No arguments are passed.
 // 2. The delimiter is empty.
-// 3. The delimiter has more than one character or is undefined and limit is not 0.
+// 3. The delimiter has more than one DisplayObject or is undefined and limit is not 0.
 // 4. The delimiter is not present in the string and the limit is not 0.
 //
 // Accordingly, an empty array is returned only when the limit is less
@@ -338,34 +224,21 @@ string_slice(const fn_call& fn)
 as_value
 string_split(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    /// version to use is the one of the SWF containing caller code.
-    /// If callerDef is null, this calls is spontaneous (system-event?)
-    /// in which case we should research on which version should drive
-    /// behaviour.
-    /// NOTE: it is unlikely that a system event triggers string_split so
-    ///       in most cases a null callerDef means the caller forgot to 
-    ///       set the field (ie: a programmatic error)
-    if (!fn.callerDef ) {
-        log_error("No fn_call::callerDef in string_split call");
-    }
-
-    const int version = fn.callerDef ? fn.callerDef->get_version() :
-        obj->getVM().getSWFVersion();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
     
     std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
-    boost::intrusive_ptr<Array_as> array(new Array_as());
+    Global_as& gl = getGlobal(fn);
+    as_object* array = gl.createArray();
 
     if (fn.nargs == 0)
     {
         // Condition 1:
-        array->push(str);
-        return as_value(array.get());
+        callMethod(array, NSV::PROP_PUSH, str);
+        return as_value(array);
     }
 
     const std::wstring& delim = utf8::decodeCanonicalString(
@@ -376,8 +249,8 @@ string_split(const fn_call& fn)
         (version >= 6 && fn.arg(0).is_undefined()))
     {
         // Condition 2:
-        array->push(str);
-        return as_value(array.get());
+        callMethod(array, NSV::PROP_PUSH, str);
+        return as_value(array);
     }
 
     size_t max = wstr.size() + 1;
@@ -387,21 +260,21 @@ string_split(const fn_call& fn)
         // SWF5
         if (fn.nargs > 1 && !fn.arg(1).is_undefined())
         {
-            int limit = fn.arg(1).to_int();
+            int limit = toInt(fn.arg(1));
             if (limit < 1)
             {
                 // Return empty array.
-                return as_value(array.get());
+                return as_value(array);
             }
-            max = utility::clamp<size_t>(limit, 0, max);
+            max = clamp<size_t>(limit, 0, max);
         }
 
         if (delimiterSize > 1 || fn.arg(0).is_undefined() || wstr.empty())
         {
             // Condition 3 (plus a shortcut if the string itself
             // is empty).
-	        array->push(str);
-	        return as_value(array.get());            
+            callMethod(array, NSV::PROP_PUSH, str);
+            return as_value(array);            
         }
     }
     else
@@ -412,29 +285,31 @@ string_split(const fn_call& fn)
             // If the string itself is empty, SWF6 returns a 0-sized
             // array only if the delimiter is also empty. Otherwise
             // it returns an array with 1 empty element.
-            if (delimiterSize) array->push(str);
-            return as_value(array.get());
+            if (delimiterSize) callMethod(array, NSV::PROP_PUSH, str);
+            return as_value(array);
         }
 
         // If we reach this point, the string is not empty and
         // the delimiter is defined.
         if (fn.nargs > 1 && !fn.arg(1).is_undefined())
         {
-	        int limit = fn.arg(1).to_int();
-	        if (limit < 1) {
-	            // Return empty array if 
-	            return as_value(array.get());
-	        }
-            max = utility::clamp<size_t>(limit, 0, max);
+            int limit = toInt(fn.arg(1));
+            if (limit < 1) {
+                // Return empty array if 
+                return as_value(array);
+            }
+            max = clamp<size_t>(limit, 0, max);
         }
 
         // If the delimiter is empty, put each character in an
         // array element.
-        if ( delim.empty() ) {
-            for (size_t i = 0, e = wstr.size(); i < e; ++i) {
-                array->push(utf8::encodeCanonicalString(wstr.substr(i, 1), version));
+        if (delim.empty()) {
+            for (size_t i = 0, e = std::min<size_t>(wstr.size(), max);
+                    i < e; ++i) {
+                callMethod(array, NSV::PROP_PUSH,
+                       utf8::encodeCanonicalString(wstr.substr(i, 1), version));
             }
-            return as_value(array.get());
+            return as_value(array);
         }
 
     }
@@ -445,16 +320,16 @@ string_split(const fn_call& fn)
     while (num < max) {
         pos = wstr.find(delim, pos);
 
-        array->push(utf8::encodeCanonicalString(
-               		wstr.substr(prevpos, pos - prevpos),
-               		version));
+        callMethod(array, NSV::PROP_PUSH, utf8::encodeCanonicalString(
+                       wstr.substr(prevpos, pos - prevpos), version));
+
         if (pos == std::wstring::npos) break;
         num++;
         prevpos = pos + delimiterSize;
         pos++;
     }
 
-    return as_value(array.get());
+    return as_value(array);
 }
 
 /// String.lastIndexOf[string[, pos]]
@@ -464,19 +339,19 @@ string_split(const fn_call& fn)
 as_value
 string_lastIndexOf(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     if (!checkArgs(fn, 1, 2, "String.lastIndexOf()")) return as_value(-1);
 
-    const std::string& toFind = fn.arg(0).to_string();
+    const std::string& toFind = fn.arg(0).to_string(version);
 
     int start = str.size();
 
     if (fn.nargs >= 2) {
-        start = fn.arg(1).to_int();
+        start = toInt(fn.arg(1));
     }
     
     if (start < 0) {
@@ -500,33 +375,31 @@ string_lastIndexOf(const fn_call& fn)
 as_value
 string_substr(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    int version = obj->getVM().getSWFVersion();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
     if (!checkArgs(fn, 1, 2, "String.substr()")) return as_value(str);
     
-    int start = validIndex(wstr, fn.arg(0).to_int());
+    int start = validIndex(wstr, toInt(fn.arg(0)));
 
     int num = wstr.length();
 
     if (fn.nargs >= 2 && !fn.arg(1).is_undefined())
     {
-        num = fn.arg(1).to_int();
-	    if ( num < 0 )
-	    {
-		    if ( -num <= start ) num = 0;
-		    else
-		    {
-			    num = wstr.length() + num;
-			    if ( num < 0 ) return as_value("");
-		    }
-	    }
+        num = toInt(fn.arg(1));
+        if ( num < 0 )
+        {
+            if ( -num <= start ) num = 0;
+            else
+            {
+                num = wstr.length() + num;
+                if ( num < 0 ) return as_value("");
+            }
+        }
     }
 
     return as_value(utf8::encodeCanonicalString(wstr.substr(start, num), version));
@@ -540,21 +413,21 @@ string_substr(const fn_call& fn)
 as_value
 string_substring(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    int version = obj->getVM().getSWFVersion();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     const std::wstring& wstr = utf8::decodeCanonicalString(str, version);
 
     if (!checkArgs(fn, 1, 2, "String.substring()")) return as_value(str);
 
-    int start = fn.arg(0).to_int();
+    const as_value& s = fn.arg(0);
+
+    int start = toInt(s);
     int end = wstr.size();
 
-    if (start < 0) {
+    if (s.is_undefined() || start < 0) {
         start = 0;
     }
 
@@ -562,8 +435,8 @@ string_substring(const fn_call& fn)
         return as_value("");
     }
 
-    if (fn.nargs >= 2) {
-        int num = fn.arg(1).to_int();
+    if (fn.nargs >= 2 && !fn.arg(1).is_undefined()) {
+        int num = toInt(fn.arg(1));
 
         if (num < 0) {
             num = 0;
@@ -592,39 +465,39 @@ string_substring(const fn_call& fn)
 as_value
 string_indexOf(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
  
     /// Do not return before this, because the toString method should always
     /// be called. (TODO: test).   
-    const std::string& str = val.to_string();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     if (!checkArgs(fn, 1, 2, "String.indexOf")) return as_value(-1);
 
-    int version = obj->getVM().getSWFVersion();
-
     const std::wstring& wstr = utf8::decodeCanonicalString(str, version);
 
     const as_value& tfarg = fn.arg(0); // to find arg
-    const std::wstring& toFind = utf8::decodeCanonicalString(tfarg.to_string(), version);
+    const std::wstring& toFind =
+        utf8::decodeCanonicalString(tfarg.to_string(version),
+                version);
 
     size_t start = 0;
 
     if (fn.nargs >= 2)
     {
         const as_value& saval = fn.arg(1); // start arg val
-        int start_arg = saval.to_int();
+        int start_arg = toInt(saval);
         if ( start_arg > 0 ) start = (size_t) start_arg;
-	else
-	{
-		IF_VERBOSE_ASCODING_ERRORS(
-		if ( start_arg < 0 )
-		{
-			log_aserror("String.indexOf(%s, %s): second argument casts to invalid offset (%d)",
-				tfarg, saval, start_arg);
-		}
-		);
-	}
+    else
+    {
+        IF_VERBOSE_ASCODING_ERRORS(
+        if ( start_arg < 0 )
+        {
+            log_aserror("String.indexOf(%s, %s): second argument casts to invalid offset (%d)",
+                tfarg, saval, start_arg);
+        }
+        );
+    }
     }
 
     size_t pos = wstr.find(toFind, start);
@@ -642,17 +515,16 @@ string_indexOf(const fn_call& fn)
 as_value
 string_fromCharCode(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
 
-    const int version = obj->getVM().getSWFVersion();
+    const int version = getSWFVersion(fn);
 
     if (version == 5)
     {
         std::string str;
         for (unsigned int i = 0; i < fn.nargs; i++)
         {
-            // Maximum 65535, as with all character codes.
-            boost::uint16_t c = static_cast<boost::uint16_t>(fn.arg(i).to_int());
+            // Maximum 65535, as with all DisplayObject codes.
+            boost::uint16_t c = static_cast<boost::uint16_t>(toInt(fn.arg(i)));
             
             // If more than 255, push 'overflow' byte.
             if (c > 255)
@@ -672,7 +544,7 @@ string_fromCharCode(const fn_call& fn)
 
     for (unsigned int i = 0; i < fn.nargs; i++)
     {
-        boost::uint16_t c = static_cast<boost::uint16_t>(fn.arg(i).to_int());
+        boost::uint16_t c = static_cast<boost::uint16_t>(toInt(fn.arg(i)));
         if (c == 0) break;
         wstr.push_back(c);
     }
@@ -684,12 +556,10 @@ string_fromCharCode(const fn_call& fn)
 as_value
 string_charCodeAt(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    int version = obj->getVM().getSWFVersion();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     const std::wstring& wstr = utf8::decodeCanonicalString(str, version);
 
@@ -698,8 +568,8 @@ string_charCodeAt(const fn_call& fn)
             log_aserror(_("string.charCodeAt needs one argument"));
         )
         as_value rv;
-        rv.set_nan();
-        return rv;	// Same as for out-of-range arg
+        setNaN(rv);
+        return rv;    // Same as for out-of-range arg
     }
 
     IF_VERBOSE_ASCODING_ERRORS(
@@ -712,7 +582,7 @@ string_charCodeAt(const fn_call& fn)
 
     if (index >= wstr.length()) {
         as_value rv;
-        rv.set_nan();
+        setNaN(rv);
         return rv;
     }
 
@@ -722,17 +592,15 @@ string_charCodeAt(const fn_call& fn)
 as_value
 string_charAt(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    const std::string& str = val.to_string();
-
-    const int version = obj->getVM().getSWFVersion();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
     if (!checkArgs(fn, 1, 1, "String.charAt()")) return as_value("");
 
     // to_int() makes this safe from overflows.
-    const size_t index = static_cast<size_t>(fn.arg(0).to_int());
+    const size_t index = static_cast<size_t>(toInt(fn.arg(0)));
 
     size_t currentIndex = 0;
 
@@ -758,13 +626,12 @@ string_charAt(const fn_call& fn)
 as_value
 string_toUpperCase(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
 
-    VM& vm = obj->getVM();
-    const int version = vm.getSWFVersion();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
-    std::wstring wstr = utf8::decodeCanonicalString(val.to_string(), version);
+    std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
     // If this is the C locale, the conversion will be wrong.
     // Most other locales are correct. FIXME: get this to
@@ -783,7 +650,7 @@ string_toUpperCase(const fn_call& fn)
     {
         LOG_ONCE(
             log_error(_("Your locale probably can't convert non-ascii "
-            "characters to upper case. Using a UTF8 locale may fix this."));
+            "DisplayObjects to upper case. Using a UTF8 locale may fix this."));
         );
     }
 
@@ -796,13 +663,12 @@ string_toUpperCase(const fn_call& fn)
 as_value
 string_toLowerCase(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
     
-    VM& vm = obj->getVM();
-    const int version = vm.getSWFVersion();
+    std::string str;
+    const int version = getStringVersioned(fn, val, str);
 
-    std::wstring wstr = utf8::decodeCanonicalString(val.to_string(), version);
+    std::wstring wstr = utf8::decodeCanonicalString(str, version);
 
     // If this is the C locale, the conversion will be wrong.
     // Most other locales are correct. FIXME: get this to
@@ -821,7 +687,7 @@ string_toLowerCase(const fn_call& fn)
     {
         LOG_ONCE( 
             log_error(_("Your locale probably can't convert non-ascii "
-                "characters to lower case. Using a UTF8 locale may fix this"));
+                "DisplayObjects to lower case. Using a UTF8 locale may fix this"));
         );
     }
 
@@ -833,10 +699,9 @@ string_toLowerCase(const fn_call& fn)
 as_value
 string_oldToLower(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
 
-    // This should use the C locale; extended characters are
+    // This should use the C locale; extended DisplayObjects are
     // left alone. FIXME: SWF5 should garble the output.
     std::string str = boost::to_lower_copy(val.to_string());
     return as_value(str);
@@ -846,67 +711,81 @@ string_oldToLower(const fn_call& fn)
 as_value
 string_oldToUpper(const fn_call& fn)
 {
-    boost::intrusive_ptr<as_object> obj = ensureType<as_object>(fn.this_ptr);
     as_value val(fn.this_ptr);
 
-    // This should use the C locale; extended characters are
+    // This should use the C locale; extended DisplayObjects are
     // left alone. FIXME: SWF5 should garble the output.
     std::string str = boost::to_upper_copy(val.to_string());
     return as_value(str);
 }
 
+/// This returns as_value.toString() value of an object. For Strings this is
+/// a string, for objects "[type Object]" or "[type Function]", for Booleans
+/// "true" or "false", etc.
+as_value
+string_valueOf(const fn_call& fn)
+{
+    const int version = getSWFVersion(fn);
+    return as_value(fn.this_ptr).to_string(version);
+}
 
 as_value
 string_toString(const fn_call& fn)
 {
-    boost::intrusive_ptr<String_as> obj = ensureType<String_as>(fn.this_ptr);
-    return as_value(obj->str());
+    String_as* str = ensure<ThisIsNative<String_as> >(fn);
+    return as_value(str->value());
 }
 
 
 as_value
 string_ctor(const fn_call& fn)
 {
-	std::string str;
-	
-	if (fn.nargs )
-	{
-		str = fn.arg(0).to_string();
-	}
+    const int version = getSWFVersion(fn);
 
-	if ( ! fn.isInstantiation() )
-	{
-		return as_value(str);
-	}
-	
-	boost::intrusive_ptr<String_as> obj = new String_as(str);
-
-	return as_value(obj.get());
-}
-
-boost::intrusive_ptr<builtin_function>
-getStringConstructor()
-{
-    // This is going to be the global String "class"/"function"
-
-    static boost::intrusive_ptr<builtin_function> cl;
-
-    if ( cl == NULL )
-    {
-	    VM& vm = VM::get();
-
-        cl=new builtin_function(&string_ctor, getStringInterface());
-	    vm.addStatic(cl.get());
-
-	    // ASnative(251, 14) - [String] fromCharCode 
-	    vm.registerNative(string_fromCharCode, 251, 14);
-	    cl->init_member("fromCharCode", vm.getNative(251, 14)); 
+    std::string str;
 
+    if (fn.nargs) {
+        str = fn.arg(0).to_string(version);
     }
 
-    return cl;
-}
+    if (!fn.isInstantiation())
+    {
+        return as_value(str);
+    }
+    
+    as_object* obj = fn.this_ptr;
 
+    obj->setRelay(new String_as(str));
+    std::wstring wstr = utf8::decodeCanonicalString(str, getSWFVersion(fn));
+    obj->init_member(NSV::PROP_LENGTH, wstr.size(), as_object::DefaultFlags);
+
+    return as_value();
+}
+    
+inline int
+getStringVersioned(const fn_call& fn, const as_value& val, std::string& str)
+{
+
+    /// version to use is the one of the SWF containing caller code.
+    /// If callerDef is null, this calls is spontaneous (system-event?)
+    /// in which case we should research on which version should drive
+    /// behaviour.
+    /// NOTE: it is unlikely that a system event triggers string_split so
+    ///       in most cases a null callerDef means the caller forgot to 
+    ///       set the field (ie: a programmatic error)
+    if (!fn.callerDef) {
+        log_error("No fn_call::callerDef in string function call");
+    }
+
+    const int version = fn.callerDef ? fn.callerDef->get_version() :
+        getSWFVersion(fn);
+    
+    str = val.to_string(version);
+
+    return version;
+
+
+}
 
 /// Check the number of arguments, returning false if there
 /// aren't enough, or true if there are either enough or too many.
@@ -945,7 +824,7 @@ validIndex(const std::wstring& subject, int index)
         index = subject.size() + index;
     }
 
-    index = utility::clamp<int>(index, 0, subject.size());
+    index = clamp<int>(index, 0, subject.size());
 
     return index;
 }
