@@ -1,6 +1,7 @@
 // BitmapData_as.cpp:  ActionScript "BitmapData" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +18,6 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "BitmapData_as.h"
 #include "MovieClip.h"
 #include "GnashImage.h"
@@ -33,7 +30,6 @@
 #include "smart_ptr.h" // for boost intrusive_ptr
 #include "builtin_function.h" // need builtin_function
 #include "GnashException.h" // for ActionException
-#include "Object.h" // for AS inheritance
 #include "VM.h" // for addStatics
 
 #include <vector>
@@ -44,48 +40,47 @@ namespace gnash {
 
 namespace {
 
-    as_value BitmapData_applyFilter(const fn_call& fn);
-    as_value BitmapData_clone(const fn_call& fn);
-    as_value BitmapData_colorTransform(const fn_call& fn);
-    as_value BitmapData_copyChannel(const fn_call& fn);
-    as_value BitmapData_copyPixels(const fn_call& fn);
-    as_value BitmapData_dispose(const fn_call& fn);
-    as_value BitmapData_draw(const fn_call& fn);
-    as_value BitmapData_fillRect(const fn_call& fn);
-    as_value BitmapData_floodFill(const fn_call& fn);
-    as_value BitmapData_generateFilterRect(const fn_call& fn);
-    as_value BitmapData_getColorBoundsRect(const fn_call& fn);
-    as_value BitmapData_getPixel(const fn_call& fn);
-    as_value BitmapData_getPixel32(const fn_call& fn);
-    as_value BitmapData_hitTest(const fn_call& fn);
-    as_value BitmapData_merge(const fn_call& fn);
-    as_value BitmapData_noise(const fn_call& fn);
-    as_value BitmapData_paletteMap(const fn_call& fn);
-    as_value BitmapData_perlinNoise(const fn_call& fn);
-    as_value BitmapData_pixelDissolve(const fn_call& fn);
-    as_value BitmapData_scroll(const fn_call& fn);
-    as_value BitmapData_setPixel(const fn_call& fn);
-    as_value BitmapData_setPixel32(const fn_call& fn);
-    as_value BitmapData_threshold(const fn_call& fn);
-    as_value BitmapData_height(const fn_call& fn);
-    as_value BitmapData_rectangle(const fn_call& fn);
-    as_value BitmapData_transparent(const fn_call& fn);
-    as_value BitmapData_width(const fn_call& fn);
-    as_value BitmapData_loadBitmap(const fn_call& fn);
-    as_value BitmapData_ctor(const fn_call& fn);
+    as_value bitmapdata_applyFilter(const fn_call& fn);
+    as_value bitmapdata_clone(const fn_call& fn);
+    as_value bitmapdata_colorTransform(const fn_call& fn);
+    as_value bitmapdata_copyChannel(const fn_call& fn);
+    as_value bitmapdata_copyPixels(const fn_call& fn);
+    as_value bitmapdata_dispose(const fn_call& fn);
+    as_value bitmapdata_draw(const fn_call& fn);
+    as_value bitmapdata_fillRect(const fn_call& fn);
+    as_value bitmapdata_floodFill(const fn_call& fn);
+    as_value bitmapdata_generateFilterRect(const fn_call& fn);
+    as_value bitmapdata_getColorBoundsRect(const fn_call& fn);
+    as_value bitmapdata_getPixel(const fn_call& fn);
+    as_value bitmapdata_getPixel32(const fn_call& fn);
+    as_value bitmapdata_hitTest(const fn_call& fn);
+    as_value bitmapdata_merge(const fn_call& fn);
+    as_value bitmapdata_noise(const fn_call& fn);
+    as_value bitmapdata_paletteMap(const fn_call& fn);
+    as_value bitmapdata_perlinNoise(const fn_call& fn);
+    as_value bitmapdata_pixelDissolve(const fn_call& fn);
+    as_value bitmapdata_scroll(const fn_call& fn);
+    as_value bitmapdata_setPixel(const fn_call& fn);
+    as_value bitmapdata_setPixel32(const fn_call& fn);
+    as_value bitmapdata_threshold(const fn_call& fn);
+    as_value bitmapdata_height(const fn_call& fn);
+    as_value bitmapdata_rectangle(const fn_call& fn);
+    as_value bitmapdata_transparent(const fn_call& fn);
+    as_value bitmapdata_width(const fn_call& fn);
+    as_value bitmapdata_loadBitmap(const fn_call& fn);
+    as_value bitmapdata_ctor(const fn_call& fn);
 
     void attachBitmapDataInterface(as_object& o);
     void attachBitmapDataStaticProperties(as_object& o);
-    as_object* getBitmapDataInterface();
     as_value get_flash_display_bitmap_data_constructor(const fn_call& fn);
 
 }
 
 
-BitmapData_as::BitmapData_as(size_t width, size_t height,
-	              bool transparent, boost::uint32_t fillColor)
-		:
-    as_object(getBitmapDataInterface()),
+BitmapData_as::BitmapData_as(as_object* owner, size_t width, size_t height,
+              bool transparent, boost::uint32_t fillColor)
+    :
+    _owner(owner),
     _width(width),
     _height(height),
     _transparent(transparent),
@@ -94,11 +89,11 @@ BitmapData_as::BitmapData_as(size_t width, size_t height,
 }
 
 void
-BitmapData_as::markReachableResources() const
+BitmapData_as::setReachable() 
 {
     std::for_each(_attachedBitmaps.begin(), _attachedBitmaps.end(),
             std::mem_fun(&DisplayObject::setReachable));
-    markAsObjectReachable();
+    _owner->setReachable();
 }
 
 
@@ -129,6 +124,7 @@ BitmapData_as::updateAttachedBitmaps()
 boost::int32_t
 BitmapData_as::getPixel(int x, int y, bool transparency) const
 {
+    assert(!_bitmapData.empty());
 
     // A value of 0, 0 is inside the bitmap.
     if (x < 0 || y < 0) return 0;
@@ -158,11 +154,8 @@ void
 BitmapData_as::fillRect(int x, int y, int w, int h, boost::uint32_t color)
 {
 
-    GNASH_REPORT_FUNCTION;
-
-    // The bitmap has been "disposed".
-    if (_bitmapData.empty()) return;
-    assert (_bitmapData.size() == _width * _height);
+    // This also catches disposed BitmapDatas.
+    assert(_bitmapData.size() == _width * _height);
 
     if (w < 0 || h < 0) return;
 
@@ -173,14 +166,12 @@ BitmapData_as::fillRect(int x, int y, int w, int h, boost::uint32_t color)
 
     // If x or y is less than 0, make a rectangle of the
     // intersection with the bitmap.    
-    if (x < 0)
-    {
+    if (x < 0) {
         w += x;
         x = 0;
     }
 
-    if (y < 0)
-    {
+    if (y < 0) {
         h += y;
         y = 0;
     }
@@ -202,15 +193,11 @@ BitmapData_as::fillRect(int x, int y, int w, int h, boost::uint32_t color)
     // Make colour non-transparent if the image doesn't support it.
     if (!_transparent) color |= 0xff000000;
     
-    while (it != e)
-    {
-
+    while (it != e) {
         // Fill from x for the width of the rectangle.
         std::fill_n(it + x, w, color);
-
         // Move to the next line
         std::advance(it, _width);
-
     }
 
     updateAttachedBitmaps();
@@ -230,76 +217,72 @@ bitmapdata_class_init(as_object& where, const ObjectURI& uri)
 {
     // TODO: this may not be correct, but it should be enumerable.
     const int flags = 0;
-	where.init_destructive_property(getName(uri),
-                get_flash_display_bitmap_data_constructor, flags,
-                getNamespace(uri));
+	where.init_destructive_property(uri,
+            get_flash_display_bitmap_data_constructor, flags);
 }
 
 
 namespace {
 
 as_value
-BitmapData_applyFilter(const fn_call& fn)
+bitmapdata_applyFilter(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_clone(const fn_call& fn)
+bitmapdata_clone(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_colorTransform(const fn_call& fn)
+bitmapdata_colorTransform(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_copyChannel(const fn_call& fn)
+bitmapdata_copyChannel(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_copyPixels(const fn_call& fn)
+bitmapdata_copyPixels(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = 
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_dispose(const fn_call& fn)
+bitmapdata_dispose(const fn_call& fn)
 {
     // Should free the memory storing the bitmap.
     // All properties afterwards are -1 (even the rectangle)
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
     ptr->dispose();
 	return as_value();
 }
 
 as_value
-BitmapData_draw(const fn_call& fn)
+bitmapdata_draw(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 
     std::ostringstream os;
     fn.dump_args(os);
@@ -310,7 +293,7 @@ BitmapData_draw(const fn_call& fn)
         return as_value();
     }
 
-    MovieClip* mc = fn.arg(0).to_sprite();
+    MovieClip* mc = fn.arg(0).toMovieClip();
 
     if (!mc) {
         // log error
@@ -339,17 +322,22 @@ BitmapData_draw(const fn_call& fn)
 }
 
 as_value
-BitmapData_fillRect(const fn_call& fn)
+bitmapdata_fillRect(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 
     if (fn.nargs < 2) return as_value();
     
+    if (disposed(*ptr)) {
+        IF_VERBOSE_ASCODING_ERRORS(
+            log_aserror("fillRect called on disposed BitmapData!");
+        );
+        return as_value();
+    }
+    
     const as_value& arg = fn.arg(0);
     
-    if ( ! arg.is_object() )
-    {
+    if (!arg.is_object()) {
         /// Isn't an object...
         IF_VERBOSE_ASCODING_ERRORS(
             std::ostringstream ss;
@@ -361,7 +349,7 @@ BitmapData_fillRect(const fn_call& fn)
     }
 
     // This can be any object with the right properties.   
-    as_object* obj = arg.to_object(*getGlobal(fn)).get();
+    as_object* obj = arg.to_object(getGlobal(fn));
     assert(obj);
     
     as_value x, y, w, h;
@@ -371,9 +359,9 @@ BitmapData_fillRect(const fn_call& fn)
     obj->get_member(NSV::PROP_WIDTH, &w);
     obj->get_member(NSV::PROP_HEIGHT, &h);    
 
-    boost::uint32_t color = fn.arg(1).to_int();
+    const boost::uint32_t color = toInt(fn.arg(1));
        
-    ptr->fillRect(x.to_int(), y.to_int(), w.to_int(), h.to_int(), color);
+    ptr->fillRect(toInt(x), toInt(y), toInt(w), toInt(h), color);
     
 	return as_value();
 }
@@ -381,153 +369,167 @@ BitmapData_fillRect(const fn_call& fn)
 
 // Fills the bitmap with a colour starting at point x, y.
 as_value
-BitmapData_floodFill(const fn_call& fn)
+bitmapdata_floodFill(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_generateFilterRect(const fn_call& fn)
+bitmapdata_generateFilterRect(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_getColorBoundsRect(const fn_call& fn)
+bitmapdata_getColorBoundsRect(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_getPixel(const fn_call& fn)
+bitmapdata_getPixel(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 
-    if (fn.nargs < 2)
-    {
+    if (fn.nargs < 2) {
+        return as_value();
+    }
+
+    if (disposed(*ptr)) {
+        IF_VERBOSE_ASCODING_ERRORS(
+            log_aserror("getPixel called on disposed BitmapData!");
+        );
         return as_value();
     }
     
     // TODO: what happens when the pixel is outside the image?
-    
-    int x = fn.arg(0).to_int();
-    int y = fn.arg(1).to_int();
+    const int x = toInt(fn.arg(0));
+    const int y = toInt(fn.arg(1));
     
     return ptr->getPixel(x, y, false);
 }
 
 as_value
-BitmapData_getPixel32(const fn_call& fn)
+bitmapdata_getPixel32(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 
-    if (fn.nargs < 2)
-    {
+    if (fn.nargs < 2) {
+        return as_value();
+    }
+
+    if (disposed(*ptr)) {
+        IF_VERBOSE_ASCODING_ERRORS(
+            log_aserror("getPixel32 called on disposed BitmapData!");
+        );
         return as_value();
     }
     
     // TODO: what happens when the pixel is outside the image?
-    
-    int x = fn.arg(0).to_int();
-    int y = fn.arg(1).to_int();
+    const int x = toInt(fn.arg(0));
+    const int y = toInt(fn.arg(1));
     
     return ptr->getPixel(x, y, true);
 }
 
 
 as_value
-BitmapData_hitTest(const fn_call& fn)
+bitmapdata_hitTest(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_merge(const fn_call& fn)
+bitmapdata_merge(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_noise(const fn_call& fn)
+bitmapdata_noise(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_paletteMap(const fn_call& fn)
+bitmapdata_paletteMap(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_perlinNoise(const fn_call& fn)
+bitmapdata_perlinNoise(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_pixelDissolve(const fn_call& fn)
+bitmapdata_pixelDissolve(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_scroll(const fn_call& fn)
+bitmapdata_scroll(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_setPixel(const fn_call& fn)
+bitmapdata_setPixel(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 
     if (fn.nargs < 3) {
         return as_value();
     }
 
-    double x = fn.arg(0).to_number();
-    double y = fn.arg(1).to_number();
-    if (x < 0 || y < 0) return as_value();
+    if (disposed(*ptr)) {
+        IF_VERBOSE_ASCODING_ERRORS(
+            log_aserror("setPixel called on disposed BitmapData!");
+        );
+        return as_value();
+    }
+
+    const double x = fn.arg(0).to_number();
+    const double y = fn.arg(1).to_number();
+    if (isNaN(x) || isNaN(y) || x < 0 || y < 0) return as_value();
     if (x >= ptr->getWidth() || y >= ptr->getHeight()) {
         return as_value();
     }
 
     // Ignore any transparency here.
-    boost::uint32_t color = fn.arg(2).to_int() & 0xffffff;
+    const boost::uint32_t color = toInt(fn.arg(2)) & 0xffffff;
 
     ptr->setPixel(x, y, color);
 
@@ -535,24 +537,30 @@ BitmapData_setPixel(const fn_call& fn)
 }
 
 as_value
-BitmapData_setPixel32(const fn_call& fn)
+bitmapdata_setPixel32(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 
     if (fn.nargs < 3) {
         return as_value();
     }
 
-    double x = fn.arg(0).to_number();
-    double y = fn.arg(1).to_number();
-    if (x < 0 || y < 0) return as_value();
+    if (disposed(*ptr)) {
+        IF_VERBOSE_ASCODING_ERRORS(
+            log_aserror("setPixel32 called on disposed BitmapData!");
+        );
+        return as_value();
+    }
+
+    const double x = fn.arg(0).to_number();
+    const double y = fn.arg(1).to_number();
+    if (isNaN(x) || isNaN(y) || x < 0 || y < 0) return as_value();
     if (x >= ptr->getWidth() || y >= ptr->getHeight()) {
         return as_value();
     }
 
     // TODO: multiply.
-    boost::uint32_t color = fn.arg(2).to_int();
+    const boost::uint32_t color = toInt(fn.arg(2));
 
     ptr->setPixel32(x, y, color);
 
@@ -560,43 +568,40 @@ BitmapData_setPixel32(const fn_call& fn)
 }
 
 as_value
-BitmapData_threshold(const fn_call& fn)
+bitmapdata_threshold(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
 }
 
 as_value
-BitmapData_height(const fn_call& fn)
+bitmapdata_height(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 
     // Read-only
     if (fn.nargs) return as_value();
     
     // Returns the immutable height of the bitmap or -1 if dispose() has
     // been called.
-    if (ptr->getBitmapData().empty()) return -1;
+    if (disposed(*ptr)) return -1;
 	return as_value(ptr->getHeight());
 }
 
 as_value
-BitmapData_rectangle(const fn_call& fn)
+bitmapdata_rectangle(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr = 
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 
     // Returns the immutable rectangle of the bitmap or -1 if dispose()
     // has been called.
-    if (ptr->getBitmapData().empty()) return -1;
+    if (disposed(*ptr)) return -1;
 
     // If it's not found construction will fail.
     as_value rectangle(fn.env().find_object("flash.geom.Rectangle"));
-    boost::intrusive_ptr<as_function> rectCtor = rectangle.to_as_function();
+    boost::intrusive_ptr<as_function> rectCtor = rectangle.to_function();
 
     if (!rectCtor) {
         log_error("Failed to construct flash.geom.Rectangle!");
@@ -607,46 +612,43 @@ BitmapData_rectangle(const fn_call& fn)
     args += 0.0, 0.0, ptr->getWidth(), ptr->getHeight();
 
     boost::intrusive_ptr<as_object> newRect =
-            rectCtor->constructInstance(fn.env(), args);
+            constructInstance(*rectCtor, fn.env(), args);
 
     return as_value(newRect.get());
 }
 
 as_value
-BitmapData_transparent(const fn_call& fn)
+bitmapdata_transparent(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 
     // Read-only
     if (fn.nargs) return as_value();
     
     // Returns whether bitmap is transparent or -1 if dispose() has been called.
-    if (ptr->getBitmapData().empty()) return -1;
+    if (disposed(*ptr)) return -1;
 	return as_value(ptr->isTransparent());
 }
 
 as_value
-BitmapData_width(const fn_call& fn)
+bitmapdata_width(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 
     // Read-only
     if (fn.nargs) return as_value();
     
     // Returns the immutable width of the bitmap or -1 if dispose() has
     // been called.
-    if (ptr->getBitmapData().empty()) return -1;
+    if (disposed(*ptr)) return -1;
 	return as_value(ptr->getWidth());
 }
 
 
 as_value
-BitmapData_loadBitmap(const fn_call& fn)
+bitmapdata_loadBitmap(const fn_call& fn)
 {
-	boost::intrusive_ptr<BitmapData_as> ptr =
-        ensureType<BitmapData_as>(fn.this_ptr);
+	BitmapData_as* ptr = ensure<ThisIsNative<BitmapData_as> >(fn);
 	UNUSED(ptr);
 	LOG_ONCE( log_unimpl (__FUNCTION__) );
 	return as_value();
@@ -657,17 +659,18 @@ as_value
 get_flash_display_bitmap_data_constructor(const fn_call& fn)
 {
     log_debug("Loading flash.display.BitmapData class");
-    Global_as* gl = getGlobal(fn);
-    as_object* proto = getBitmapDataInterface();
-    as_object* cl = gl->createClass(&BitmapData_ctor, proto);
+    Global_as& gl = getGlobal(fn);
+    as_object* proto = gl.createObject();
+    attachBitmapDataInterface(*proto);
+    as_object* cl = gl.createClass(&bitmapdata_ctor, proto);
     attachBitmapDataStaticProperties(*cl);
     return cl;
 }
 
 as_value
-BitmapData_ctor(const fn_call& fn)
+bitmapdata_ctor(const fn_call& fn)
 {
-
+    as_object* ptr = ensure<ValidThis>(fn);
 	if (fn.nargs < 2) {
         IF_VERBOSE_ASCODING_ERRORS(
              log_aserror("BitmapData constructor requires at least two "
@@ -685,13 +688,13 @@ BitmapData_ctor(const fn_call& fn)
         default:
             // log AS coding error
         case 4:
-            fillColor = fn.arg(3).to_int();
+            fillColor = toInt(fn.arg(3));
         case 3:
             transparent = fn.arg(2).to_bool();
         case 2:
             // Is to_int correct?
-            height = fn.arg(1).to_int();
-            width = fn.arg(0).to_int();
+            height = toInt(fn.arg(1));
+            width = toInt(fn.arg(0));
             break;
     }
     
@@ -704,71 +707,56 @@ BitmapData_ctor(const fn_call& fn)
         throw ActionTypeError();
     }
 
-    boost::intrusive_ptr<BitmapData_as> obj =
-                new BitmapData_as(width, height, transparent, fillColor);
+    ptr->setRelay(
+            new BitmapData_as(ptr, width, height, transparent, fillColor));
 
-	return as_value(obj.get()); // will keep alive
+	return as_value(); 
 }
 
 void
 attachBitmapDataInterface(as_object& o)
 {
-    Global_as* gl = getGlobal(o);
-    o.init_member("applyFilter", gl->createFunction(BitmapData_applyFilter));
-    o.init_member("clone", gl->createFunction(BitmapData_clone));
-    o.init_member("colorTransform", gl->createFunction(
-                BitmapData_colorTransform));
-    o.init_member("copyChannel", gl->createFunction(BitmapData_copyChannel));
-    o.init_member("copyPixels", gl->createFunction(BitmapData_copyPixels));
-    o.init_member("dispose", gl->createFunction(BitmapData_dispose));
-    o.init_member("draw", gl->createFunction(BitmapData_draw));
-    o.init_member("fillRect", gl->createFunction(BitmapData_fillRect));
-    o.init_member("floodFill", gl->createFunction(BitmapData_floodFill));
-    o.init_member("generateFilterRect", gl->createFunction(
-                BitmapData_generateFilterRect));
-    o.init_member("getColorBoundsRect", gl->createFunction(
-                BitmapData_getColorBoundsRect));
-    o.init_member("getPixel", gl->createFunction(BitmapData_getPixel));
-    o.init_member("getPixel32", gl->createFunction(BitmapData_getPixel32));
-    o.init_member("hitTest", gl->createFunction(BitmapData_hitTest));
-    o.init_member("merge", gl->createFunction(BitmapData_merge));
-    o.init_member("noise", gl->createFunction(BitmapData_noise));
-    o.init_member("paletteMap", gl->createFunction(BitmapData_paletteMap));
-    o.init_member("perlinNoise", gl->createFunction(BitmapData_perlinNoise));
-    o.init_member("pixelDissolve", gl->createFunction(
-                BitmapData_pixelDissolve));
-    o.init_member("scroll", gl->createFunction(BitmapData_scroll));
-    o.init_member("setPixel", gl->createFunction(BitmapData_setPixel));
-    o.init_member("setPixel32", gl->createFunction(BitmapData_setPixel32));
-    o.init_member("threshold", gl->createFunction(BitmapData_threshold));
-    o.init_property("height", BitmapData_height, BitmapData_height);
-    o.init_property("rectangle", BitmapData_rectangle, BitmapData_rectangle);
-    o.init_property("transparent", BitmapData_transparent,
-            BitmapData_transparent);
-    o.init_property("width", BitmapData_width, BitmapData_width);
+    Global_as& gl = getGlobal(o);
+    o.init_member("applyFilter", gl.createFunction(bitmapdata_applyFilter));
+    o.init_member("clone", gl.createFunction(bitmapdata_clone));
+    o.init_member("colorTransform", gl.createFunction(
+                bitmapdata_colorTransform));
+    o.init_member("copyChannel", gl.createFunction(bitmapdata_copyChannel));
+    o.init_member("copyPixels", gl.createFunction(bitmapdata_copyPixels));
+    o.init_member("dispose", gl.createFunction(bitmapdata_dispose));
+    o.init_member("draw", gl.createFunction(bitmapdata_draw));
+    o.init_member("fillRect", gl.createFunction(bitmapdata_fillRect));
+    o.init_member("floodFill", gl.createFunction(bitmapdata_floodFill));
+    o.init_member("generateFilterRect", gl.createFunction(
+                bitmapdata_generateFilterRect));
+    o.init_member("getColorBoundsRect", gl.createFunction(
+                bitmapdata_getColorBoundsRect));
+    o.init_member("getPixel", gl.createFunction(bitmapdata_getPixel));
+    o.init_member("getPixel32", gl.createFunction(bitmapdata_getPixel32));
+    o.init_member("hitTest", gl.createFunction(bitmapdata_hitTest));
+    o.init_member("merge", gl.createFunction(bitmapdata_merge));
+    o.init_member("noise", gl.createFunction(bitmapdata_noise));
+    o.init_member("paletteMap", gl.createFunction(bitmapdata_paletteMap));
+    o.init_member("perlinNoise", gl.createFunction(bitmapdata_perlinNoise));
+    o.init_member("pixelDissolve", gl.createFunction(
+                bitmapdata_pixelDissolve));
+    o.init_member("scroll", gl.createFunction(bitmapdata_scroll));
+    o.init_member("setPixel", gl.createFunction(bitmapdata_setPixel));
+    o.init_member("setPixel32", gl.createFunction(bitmapdata_setPixel32));
+    o.init_member("threshold", gl.createFunction(bitmapdata_threshold));
+    o.init_property("height", bitmapdata_height, bitmapdata_height);
+    o.init_property("rectangle", bitmapdata_rectangle, bitmapdata_rectangle);
+    o.init_property("transparent", bitmapdata_transparent,
+            bitmapdata_transparent);
+    o.init_property("width", bitmapdata_width, bitmapdata_width);
 
 }
 
 void
 attachBitmapDataStaticProperties(as_object& o)
 {
-    Global_as* gl = getGlobal(o);
-    o.init_member("loadBitmap", gl->createFunction(BitmapData_loadBitmap));
-}
-
-as_object*
-getBitmapDataInterface()
-{
-	static boost::intrusive_ptr<as_object> o;
-
-	if (!o) {
-		o = new as_object(getObjectInterface());
-		VM::get().addStatic(o.get());
-
-		attachBitmapDataInterface(*o);
-	}
-
-	return o.get();
+    Global_as& gl = getGlobal(o);
+    o.init_member("loadBitmap", gl.createFunction(bitmapdata_loadBitmap));
 }
 
 } // anonymous namespace
