@@ -1,6 +1,6 @@
 // MovieClip_as.h:  ActionScript 3 "MovieClip" class, for Gnash.
 //
-//   Copyright (C) 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,10 +20,6 @@
 #ifndef GNASH_ASOBJ3_MOVIECLIP_H
 #define GNASH_ASOBJ3_MOVIECLIP_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 
 namespace gnash {
 
@@ -35,18 +31,9 @@ class DisplayObject;
 /// Initialize the global MovieClip class
 void movieclip_class_init(as_object& where, const ObjectURI& uri);
 
-/// Get an as_object with the AS3 MovieClip interface.
-as_object* getMovieClipAS3Interface();
-
 /// Register ASNative MovieClip methods (AS2 only).
 void registerMovieClipNative(as_object& where);
 
-/// Used by MovieClip's ctor to attach properties to MovieClip instances.
-void attachMovieClipAS2Properties(DisplayObject& d);
-
-/// Get an as_object with the AS2 MovieClip interface.
-as_object* getMovieClipAS2Interface();
-
 } // gnash namespace
 
 #endif
