@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,9 +19,8 @@
 #ifndef GNASH_ASOBJ_BITMAPFILTER_H
 #define GNASH_ASOBJ_BITMAPFILTER_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
+
+#include "Global_as.h"
 
 namespace gnash {
 
@@ -33,8 +32,13 @@ void bitmapfilter_class_init(as_object& where, const ObjectURI& uri);
 
 void registerBitmapFilterNative(as_object& global);
 
-/// Get the interface, for inheritance.
-as_object *getBitmapFilterInterface();
+
+/// Convenience function only for BitmapFilter subclasses.
+//
+/// This implements the AS code necessary for defining subclasses of
+/// BitmapFilter in AS2.
+void registerBitmapClass(as_object& where, Global_as::ASFunction ctor,
+        Global_as::Properties p, const ObjectURI& uri);
 
 } // end of gnash namespace
 
