@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,17 +20,13 @@
 #include "ConvolutionFilter.h"
 #include "VM.h"
 #include "Global_as.h"
-#include "builtin_function.h"
-#include "Object.h"
 #include "BitmapFilter_as.h"
+#include "builtin_function.h"
 
 namespace gnash {
 
 namespace {
-    
-    as_value convolutionfilter_ctor(const fn_call& fn);
-    as_value convolutionfilter_clone(const fn_call& fn);
-    as_value getConvolutionFilterConstructor(const fn_call& fn);
+    as_value convolutionfilter_new(const fn_call& fn);
     as_value convolutionfilter_matrixX(const fn_call& fn);
     as_value convolutionfilter_matrixY(const fn_call& fn);
     as_value convolutionfilter_matrix(const fn_call& fn);
@@ -43,38 +39,21 @@ namespace {
 
     void attachConvolutionFilterInterface(as_object& o);
 }
- 
-// TODO: Use composition, not inheritance.
-class ConvolutionFilter_as : public as_object, public ConvolutionFilter
+
+/// TODO: should this inherit from BitmapFilter_as (relay)? This might
+/// make cloning easier, but needs some testing first.
+class ConvolutionFilter_as : public Relay, public ConvolutionFilter
 {
 public:
-    ConvolutionFilter_as(as_object *obj)
-        :
-        as_object(obj)
-    {}
-
+    ConvolutionFilter_as() {}
 };
 
-
+/// The prototype of flash.filters.ConvolutionFilter is a new BitmapFilter.
 void
 convolutionfilter_class_init(as_object& where, const ObjectURI& uri)
 {
-    // TODO: this may not be correct, but it should be enumerable.
-    const int flags = 0;
-    where.init_destructive_property(getName(uri),
-            getConvolutionFilterConstructor, flags, getNamespace(uri));
-}
-
-as_object*
-getConvolutionFilterInterface()
-{
-    static as_object* o;
-    if (!o) {
-        o = new as_object(getBitmapFilterInterface());
-        VM::get().addStatic(o);
-        attachConvolutionFilterInterface(*o);
-    }
-    return o;
+    registerBitmapClass(where, convolutionfilter_new,
+            attachConvolutionFilterInterface, uri);
 }
 
 namespace {
@@ -103,28 +82,10 @@ attachConvolutionFilterInterface(as_object& o)
             convolutionfilter_color, flags);
 }
 
-as_value
-getConvolutionFilterConstructor(const fn_call& fn)
-{
-    Global_as* gl = getGlobal(fn);
-    return gl->createClass(&convolutionfilter_ctor,
-            getConvolutionFilterInterface());
-}
-
-as_value
-convolutionfilter_ctor(const fn_call& /*fn*/)
-{
-    boost::intrusive_ptr<as_object> obj =
-        new ConvolutionFilter_as(getConvolutionFilterInterface());
-    return as_value(obj);
-}
-
 as_value
 convolutionfilter_matrixX(const fn_call& fn)
 {
-    // TODO: check whether this is necessary (probably is).
-    boost::intrusive_ptr<ConvolutionFilter_as> ptr =
-        ensureType<ConvolutionFilter_as>(fn.this_ptr);
+    ConvolutionFilter_as* ptr = ensure<ThisIsNative<ConvolutionFilter_as> >(fn);
     UNUSED(ptr);
     log_unimpl(__PRETTY_FUNCTION__);
     return as_value();
@@ -133,8 +94,7 @@ convolutionfilter_matrixX(const fn_call& fn)
 as_value
 convolutionfilter_matrixY(const fn_call& fn)
 {
-    boost::intrusive_ptr<ConvolutionFilter_as> ptr =
-        ensureType<ConvolutionFilter_as>(fn.this_ptr);
+    ConvolutionFilter_as* ptr = ensure<ThisIsNative<ConvolutionFilter_as> >(fn);
     UNUSED(ptr);
     log_unimpl(__PRETTY_FUNCTION__);
     return as_value();
@@ -143,9 +103,7 @@ convolutionfilter_matrixY(const fn_call& fn)
 as_value
 convolutionfilter_divisor(const fn_call& fn)
 {
-    // TODO: check whether this is necessary (probably is).
-    boost::intrusive_ptr<ConvolutionFilter_as> ptr =
-        ensureType<ConvolutionFilter_as>(fn.this_ptr);
+    ConvolutionFilter_as* ptr = ensure<ThisIsNative<ConvolutionFilter_as> >(fn);
     UNUSED(ptr);
     log_unimpl(__PRETTY_FUNCTION__);
     return as_value();
@@ -154,9 +112,7 @@ convolutionfilter_divisor(const fn_call& fn)
 as_value
 convolutionfilter_bias(const fn_call& fn)
 {
-    // TODO: check whether this is necessary (probably is).
-    boost::intrusive_ptr<ConvolutionFilter_as> ptr =
-        ensureType<ConvolutionFilter_as>(fn.this_ptr);
+    ConvolutionFilter_as* ptr = ensure<ThisIsNative<ConvolutionFilter_as> >(fn);
     UNUSED(ptr);
     log_unimpl(__PRETTY_FUNCTION__);
     return as_value();
@@ -165,9 +121,7 @@ convolutionfilter_bias(const fn_call& fn)
 as_value
 convolutionfilter_preserveAlpha(const fn_call& fn)
 {
-    // TODO: check whether this is necessary (probably is).
-    boost::intrusive_ptr<ConvolutionFilter_as> ptr =
-        ensureType<ConvolutionFilter_as>(fn.this_ptr);
+    ConvolutionFilter_as* ptr = ensure<ThisIsNative<ConvolutionFilter_as> >(fn);
     UNUSED(ptr);
     log_unimpl(__PRETTY_FUNCTION__);
     return as_value();
@@ -176,9 +130,7 @@ convolutionfilter_preserveAlpha(const fn_call& fn)
 as_value
 convolutionfilter_clamp(const fn_call& fn)
 {
-    // TODO: check whether this is necessary (probably is).
-    boost::intrusive_ptr<ConvolutionFilter_as> ptr =
-        ensureType<ConvolutionFilter_as>(fn.this_ptr);
+    ConvolutionFilter_as* ptr = ensure<ThisIsNative<ConvolutionFilter_as> >(fn);
     UNUSED(ptr);
     log_unimpl(__PRETTY_FUNCTION__);
     return as_value();
@@ -187,9 +139,7 @@ convolutionfilter_clamp(const fn_call& fn)
 as_value
 convolutionfilter_color(const fn_call& fn)
 {
-    // TODO: check whether this is necessary (probably is).
-    boost::intrusive_ptr<ConvolutionFilter_as> ptr =
-        ensureType<ConvolutionFilter_as>(fn.this_ptr);
+    ConvolutionFilter_as* ptr = ensure<ThisIsNative<ConvolutionFilter_as> >(fn);
     UNUSED(ptr);
     log_unimpl(__PRETTY_FUNCTION__);
     return as_value();
@@ -198,9 +148,7 @@ convolutionfilter_color(const fn_call& fn)
 as_value
 convolutionfilter_alpha(const fn_call& fn)
 {
-    // TODO: check whether this is necessary (probably is).
-    boost::intrusive_ptr<ConvolutionFilter_as> ptr =
-        ensureType<ConvolutionFilter_as>(fn.this_ptr);
+    ConvolutionFilter_as* ptr = ensure<ThisIsNative<ConvolutionFilter_as> >(fn);
     UNUSED(ptr);
     log_unimpl(__PRETTY_FUNCTION__);
     return as_value();
@@ -209,13 +157,20 @@ convolutionfilter_alpha(const fn_call& fn)
 as_value
 convolutionfilter_matrix(const fn_call& fn)
 {
-    // TODO: check whether this is necessary (probably is).
-    boost::intrusive_ptr<ConvolutionFilter_as> ptr =
-        ensureType<ConvolutionFilter_as>(fn.this_ptr);
+    ConvolutionFilter_as* ptr = ensure<ThisIsNative<ConvolutionFilter_as> >(fn);
     UNUSED(ptr);
     log_unimpl(__PRETTY_FUNCTION__);
     return as_value();
 }
 
+as_value
+convolutionfilter_new(const fn_call& fn)
+{
+    boost::intrusive_ptr<as_object> obj = ensure<ThisIs<as_object> >(fn);
+    obj->setRelay(new ConvolutionFilter_as);
+    return as_value();
+}
+
+
 } // anonymous namespace
 } // namespace gnash
