@@ -1,6 +1,7 @@
 // DisplacementMapFilter_as.h:  ActionScript "DisplacementMapFilter" class, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,18 +21,16 @@
 #ifndef __GNASH_ASOBJ_DISPLACEMENTMAPFILTER_H__
 #define __GNASH_ASOBJ_DISPLACEMENTMAPFILTER_H__
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
 
 //#include <memory> // for auto_ptr
 
 namespace gnash {
 
 class as_object;
+class ObjectURI;
 
 /// Initialize the global DisplacementMapFilter class
-void DisplacementMapFilter_class_init(as_object& global);
+void displacementmapfilter_class_init(as_object& where, const ObjectURI& uri);
 
 /// Return a DisplacementMapFilter instance (in case the core lib needs it)
 //std::auto_ptr<as_object> init_DisplacementMapFilter_instance();
