@@ -1,6 +1,6 @@
 // System_as.h:  ActionScript 3 "System" class, for Gnash.
 //
-//   Copyright (C) 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,10 +20,6 @@
 #ifndef GNASH_ASOBJ3_SYSTEM_H
 #define GNASH_ASOBJ3_SYSTEM_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include <vector>
 
 
