@@ -10,11 +10,9 @@
 #endif
 
 #include "Font.h"
-#include "impl.h"
 #include "log.h"
-#include "render.h"
-#include "shape_character_def.h"
-#include "styles.h"
+#include "DefineShapeTag.h"
+#include "LineStyle.h"
 #include "movie_definition.h"
 
 // Define to the name of a default font.
