@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -73,13 +74,15 @@
 #include "dsodefs.h" // DSOEXPORT
 #include "MouseButtonState.h" // for composition
 #include "drag_state.h" // for composition
-#include "asobj/Key_as.h"
 #include "smart_ptr.h" // for memory management
 #include "URL.h" // for loadMovie
 #include "GnashKey.h" // key::code
-#include "movie_instance.h"
-#include "RunInfo.h" // for initialization
+#include "Movie.h"
+#include "RunResources.h" // for initialization
 #include "gnash.h" // Quality
+#include "MovieClip.h"
+#include "SimpleBuffer.h" // for LoadCallback
+#include "MovieLoader.h"
 
 #ifdef USE_SWFTREE
 # include "tree.hh"
@@ -95,19 +98,21 @@
 #endif
 
 #include <vector>
+#include <deque>
 #include <list>
 #include <set>
 #include <bitset>
 #include <boost/noncopyable.hpp>
+#include <boost/thread/thread.hpp>
 
 // Forward declarations
 namespace gnash {
     class ExecutableCode; // for ActionQueue
-    class Stage_as;
     class URL;
     class Timer;
     class MovieClip;
     class VirtualClock;
+    class IOChannel;
 }
 
 namespace gnash
@@ -115,7 +120,7 @@ namespace gnash
 
 struct DepthComparator
 {
-    typedef boost::intrusive_ptr<MovieClip> LevelMovie;
+    typedef MovieClip* LevelMovie;
 
     bool operator() (const LevelMovie& d1, const LevelMovie& d2)
     {
@@ -123,115 +128,121 @@ struct DepthComparator
     }
 };
 
-/// The movie stage (absolute top level node in the characters hierarchy)
+/// This class represents the 'Stage' and top-level movie.
 //
-/// This is a wrapper around the set of loaded levels being played.
-///
-/// There is a *single* instance of this class for each run;
-/// loading external movies will *not* create a new instance of it.
-///
+/// It is a wrapper around the set of loaded levels being played. Each 
+/// 'run' of a SWF movie, including all further movies loaded during the
+/// run, has exactly one movie_root, which is kept for the entire run.
+/// Loading a new top-level movie does not create a new movie_root.
+//
+/// The 'Stage' part of movie_root is accessible through the ActionScript
+/// Stage object, implemented in Stage_as.cpp.
+//
+/// The movie_root class is responsible for accepting and passing on
+/// user events (mouse or key events), for maintaining the heart-beat
+/// mechanism, and for advancing all MovieClips on request from the
+/// hosting application.
+//
+/// The _root object is provided by getAsRoot().
 class DSOEXPORT movie_root : boost::noncopyable
 {
 
 public:
 
+    class LoadCallback {
+    public:
+        LoadCallback(boost::shared_ptr<IOChannel> s, as_object* o)
+            :
+            _stream(s),
+            _obj(o)
+        {}
+        bool processLoad();
+        void setReachable() const;
+    private:
+        boost::shared_ptr<IOChannel> _stream;
+        SimpleBuffer _buf;
+        as_object* _obj;
+    };
+        
+    typedef std::list<LoadCallback> LoadCallbacks;
+
+    typedef std::bitset<key::KEYCOUNT> Keys;
+
     /// Default constructor
     //
     /// Make sure to call setRootMovie() 
     /// before using any of this class methods !
     ///
     movie_root(const movie_definition& def, VirtualClock& clock,
-            const RunInfo& runInfo);
+            const RunResources& runResources);
 
     ~movie_root();
 
-    /// Set the root movie, replacing the current one if any.
+    /// Initialize movie_root with a parsed movie definition
     //
-    /// This is needed for the cases in which the top-level movie
-    /// is replaced by another movie by effect of a loadMovie call
-    /// or similar.
-    ///
-    /// TODO: inspect what happens about VM version
-    ///   (should the *new* movie drive VM operations?
-    ///    -- hope not ! )
-    ///
-    /// Make sure to call this method before using the movie_root,
-    /// as most operations are delegated to the associated/wrapped
-    /// movie_instance.
-    ///
-    /// Note that the display viewport will be updated to match
-    /// the size of given movie.
-    ///
-    /// A call to this method is equivalent to a call to setLevel(0, movie).
-    ///
-    /// @param movie
-    /// The movie_instance to wrap.
-    /// Will be stored in an intrusive_ptr.
-    /// Must have a depth of 0.
-    ///
-    void setRootMovie(movie_instance* movie);
+    /// The definition may be a SWF or Bitmap movie definition.
+    // 
+    /// The created Movie is returned; it is non-const so may be stored,
+    /// queried, and changed by the caller for debugging or manipulation.
+    /// Direct use of the pointer may result in unexpected behaviour during
+    /// SWF playback, so for normal playback this pointer should not be
+    /// used.
+    Movie* init(movie_definition* def,
+            const MovieClip::MovieVariables& variables);
 
     /// Return the movie at the given level (0 if unloaded level).
     //
     /// POST CONDITIONS:
-    /// - The returned character has a depth equal to 'num'
+    /// - The returned DisplayObject has a depth equal to 'num'
     ///
-    boost::intrusive_ptr<movie_instance> getLevel(unsigned int num) const;
+    MovieClip* getLevel(unsigned int num) const;
 
-    /// Load movie at the specified URL in the given level 
+    /// Put the given movie at the given level 
     //
-    /// Note that the display viewport will be updated to reflect
-    /// the new layout.
+    /// @param movie
+    /// The Movie to store at the given level.
+    /// Its depth will be set to <num>+DisplayObject::staticDepthOffset and
+    /// its name to _level<num>
+    void setLevel(unsigned int num, Movie* movie);
+
+    /// Replace an existing level with a new movie
+    //
+    /// Depth will be assigned to external_movie by this function.
+    /// If the give level number doesn't exist an error is logged
+    /// and nothing else happens.
     ///
-    /// @param num
-    /// The level number to load into.
+    /// This method is intended for use by xxx.loadMovie(yyy)
+    /// when 'xxx' is a top-level movie.
     ///
-    /// @param url
-    /// The url to load the movie from.
-    /// Can contain a query string, which would be parsed.
-    ///
-    /// @return false on error, true on success
-    ///
-    bool loadLevel(unsigned int num, const URL& url);
+    void replaceLevel(unsigned int num, Movie* external_movie);
 
     /// Swap depth of a level (or two)
     //
     /// Character's depths are updated.
     ///
     /// @param sp
-    ///        The level to change depth/level of. A pointer to it is expected
-    ///        to be found in the _level# container, or an error will be printed
-    ///        and the call would result in a no-op.
+    ///    The level to change depth/level of. A pointer to it is expected
+    ///    to be found in the _level# container, or an error will be printed
+    ///    and the call would result in a no-op.
     ///
     /// @param depth
-    ///        New depth to assign to the character. If another level exists at 
-    ///        the target depth the latter is moved in place of the former, with
-    ///        its depth also updated.
+    ///    New depth to assign to the DisplayObject. If another level
+    ///    exists at the target depth the latter is moved in place of
+    ///    the former, with its depth also updated.
     ///
-    void swapLevels(boost::intrusive_ptr<MovieClip> sp, int depth);
+    void swapLevels(MovieClip* sp, int depth);
 
     /// Drop level at given depth.
     //
     /// @param depth
     ///   Depth of the level to drop. Note that this is 
-    ///   -character::staticDepthOffset for the root movie. Must be >=0 and
+    ///   -DisplayObject::staticDepthOffset for the root movie. Must be >=0 and
     ///   <= 1048575 or an assertion will fail. Note that if the depth
     ///   evaluates to the original root movie nothing happens (not allowed
     ///   to remove that). It is not tested if it's allowed to remove _level0
     ///   after loading into it.
     void dropLevel(int depth);
 
-    /// @@ should this delegate to _level0?  probably !
-    void set_member(const std::string& /*name*/, const as_value& /*val*/)
-    {
-    }
-
-    /// @@ should this delegate to _level0?  probably !
-    bool get_member(const std::string& /*name*/, as_value* /*val*/)
-    {
-        return false;
-    }
-
     /// Change display viewport coordinates
     //
     /// This currently also change the display scale
@@ -305,9 +316,26 @@ public:
     void set_drag_state(const drag_state& st);
 
     /// @return the originating root movie (not necessarely _level0)
-    movie_instance* getRootMovie() const
+    const Movie& getRootMovie() const
     {
-        return _rootMovie.get();
+        return *_rootMovie;
+    }
+
+    /// Creating new MovieClips needs this for now.
+    //
+    /// TODO: create MovieClips without this and drop. It's deliberately
+    /// different from getRootMovie() so it doesn't end up getting used
+    /// in the same way.
+    Movie* topLevelMovie() const
+    {
+        return _rootMovie;
+    }
+
+    /// Return the current nominal frame rate for the Stage.
+    //
+    /// This is dependent on the Movie set as root movie.
+    float frameRate() const {
+        return _rootMovie->frameRate();
     }
 
     void stop_drag()
@@ -315,15 +343,6 @@ public:
         m_drag_state.reset();
     }
 
-    /// Return definition of originating root movie 
-    //
-    /// TODO: rename to getOriginatingDefinition ?
-    ///
-    movie_definition* get_movie_definition() const
-    {
-        return getRootMovie()->get_movie_definition();
-    }
-
     /// Add an interval timer
     //
     /// @param timer
@@ -336,8 +355,29 @@ public:
     ///         for subsequent call to clear_interval_timer.
     ///         It will NEVER be zero.
     ///
-    unsigned int add_interval_timer(std::auto_ptr<Timer> timer,
-            bool internal = false);
+    unsigned int add_interval_timer(std::auto_ptr<Timer> timer);
+
+    /// Register an object for loading data to.
+    //
+    /// When complete, the object's onData function is called.
+    /// The callback is removed when the load is complete, including failed
+    /// loads.
+    //
+    /// There is no restriction on the type of as_object that can registered.
+    //
+    /// @param obj      The object to update when data is received.
+    /// @param str      The stream to load from.
+    //
+    /// TODO: this function could be improved, e.g. by handling the
+    /// URL checking and stream construction as well.
+    //
+    /// It may be possible for this function to handle all connections if
+    /// it also takes a callback function to call on each advance.
+    void addLoadableObject(as_object* obj, std::auto_ptr<IOChannel> str);
+
+    void addAdvanceCallback(ActiveRelay* obj);
+
+    void removeAdvanceCallback(ActiveRelay* obj);
 
     /// Remove timer identified by given integer
     //
@@ -352,35 +392,43 @@ public:
     ///
     size_t get_current_frame() const
     {
-        return getRootMovie()->get_current_frame();
+        return _rootMovie->get_current_frame();
     }
 
     void set_background_color(const rgba& color);
 
     void set_background_alpha(float alpha);
 
-    float get_background_alpha() const
-    {
-        return m_background_color.m_a / 255.0f;
-    }
-
     /// Return the VM used by this movie_root
     VM& getVM() { return _vm; }
  
     /// Main and only callback from hosting application.
     /// Expected to be called at 10ms resolution.
-    void advance();
+    //
+    /// @return true if the heart-beat resulted in actual
+    ///         SWF playhead advancement (frame advancement)
+    ///
+    bool advance();
+
+    /// \brief
+    /// Return the number of milliseconds available before
+    /// it's time to advance the timeline again.
+    //
+    /// Return value can be negative if we're late...
+    ///
+    int timeToNextFrame() const;
 
     /// Entry point for movie advancement
     //
     /// This function does:
-    ///     - Execute all timers
-    ///     - Reset the next Random number
-    ///     - Advance all advanceable characters in reverse-placement order
-    ///     - Cleanup key listeners
-    ///     - Process all queued actions
-    ///     - Remove unloaded characters from the advanceable characters list.
-    ///     - Run the GC collector
+    ///   - Execute all timers
+    ///   - Reset the next Random number
+    ///   - Advance all advanceable DisplayObjects in reverse-placement order
+    ///   - Cleanup key listeners
+    ///   - Process all queued actions
+    ///   - Remove unloaded DisplayObjects from the advanceable
+    ///     DisplayObjects list.
+    ///   - Run the GC collector
     void advanceMovie();
 
     /// 0-based!! delegates to originating root movie
@@ -388,7 +436,7 @@ public:
     /// TODO: drop this method. currently used by gprocessor.
     void goto_frame(size_t target_frame_number)
     {
-        getRootMovie()->goto_frame(target_frame_number);
+        _rootMovie->goto_frame(target_frame_number);
     }
 
     void display();
@@ -396,58 +444,61 @@ public:
     /// Delegate to originating root movie
     //
     /// TODO: drop ?
-    void set_play_state(MovieClip::play_state s)
+    void set_play_state(MovieClip::PlayState s)
     {
-        getRootMovie()->set_play_state(s);
+        _rootMovie->setPlayState(s);
     }
 
-    /// Notify still loaded character listeners for key events
+    /// Get a unique number for unnamed instances.
+    size_t nextUnnamedInstance();
+
+    /// Notify still loaded DisplayObject listeners for key events
     DSOEXPORT void notify_key_listeners(key::code k, bool down);
 
-    /// Push a new character listener for key events
-    void add_key_listener(character* listener)
+    /// Push a new DisplayObject listener for key events
+    void add_key_listener(DisplayObject* listener)
     {
-        add_listener(m_key_listeners, listener);
+        add_listener(_keyListeners, listener);
     }
 
-    /// Remove a character listener for key events
-    void remove_key_listener(character* listener)
+    /// Remove a DisplayObject listener for key events
+    void remove_key_listener(DisplayObject* listener)
     {
-        remove_listener(m_key_listeners, listener);
+        remove_listener(_keyListeners, listener);
     }
 
-    /// Notify still loaded character listeners for mouse events
+    /// Notify still loaded DisplayObject listeners for mouse events
     DSOEXPORT void notify_mouse_listeners(const event_id& event);
 
-    /// Push a new character listener for mouse events
-    void add_mouse_listener(character* listener)
+    /// Push a new DisplayObject listener for mouse events
+    void add_mouse_listener(DisplayObject* listener)
     {
-        add_listener(m_mouse_listeners, listener);
+        add_listener(_mouseListeners, listener);
     }
 
-    /// Remove a character listener for mouse events
-    void remove_mouse_listener(character* listener)
+    /// Remove a DisplayObject listener for mouse events
+    void remove_mouse_listener(DisplayObject* listener)
     {
-        remove_listener(m_mouse_listeners, listener);
+        remove_listener(_mouseListeners, listener);
     }
 
-    /// Get the character having focus
+    /// Get the DisplayObject having focus
     //
-    /// The character having focus will receive mouse button
+    /// The DisplayObject having focus will receive mouse button
     /// and key presses/releases.
     ///
-    /// @return the character having focus or NULL of none.
+    /// @return the DisplayObject having focus or NULL of none.
     ///
-    boost::intrusive_ptr<character> getFocus();
+    DisplayObject* getFocus();
 
-    /// Set the character having focus
+    /// Set the DisplayObject having focus
     //
     /// @param to
-    /// The character to receive focus. NULL to kill focus.
+    /// The DisplayObject to receive focus. NULL to kill focus.
     /// @return true if the focus operation succeeded, false if the passed
-    /// character cannot receive focus. setFocus(0) is a valid operation, so
+    /// DisplayObject cannot receive focus. setFocus(0) is a valid operation, so
     /// returns true (always succeeds).
-    bool setFocus(boost::intrusive_ptr<character> to);
+    bool setFocus(DisplayObject* to);
     
     DSOEXPORT void add_invalidated_bounds(InvalidatedRanges& ranges,
             bool force);
@@ -460,17 +511,17 @@ public:
     /// currently implmented).
     ///
     /// @return the topmost active entity under pointer or NULL if none.
-    character* getActiveEntityUnderPointer() const;
+    DisplayObject* getActiveEntityUnderPointer() const;
 
     /// Return the topmost non-dragging entity under the pointer
     //
     /// This method triggers a displaylist scan
     ///
     /// @return the topmost non-dragging entity under pointer or NULL if none
-    const character* getEntityUnderPointer() const;
+    const DisplayObject* getEntityUnderPointer() const;
 
-    /// Return the character currently being dragged, if any
-    character* getDraggingCharacter() const;
+    /// Return the DisplayObject currently being dragged, if any
+    DisplayObject* getDraggingCharacter() const;
 
     /// Return true if the mouse pointer is over an active entity
     bool isMouseOverActiveEntity() const;
@@ -479,8 +530,8 @@ public:
 
     /// The possible values of Stage.displayState
     enum DisplayState {
-        normal,
-        fullScreen
+        DISPLAYSTATE_NORMAL,
+        DISPLAYSTATE_FULLSCREEN
     };
 
     /// The possibile values of Stage.scaleMode
@@ -495,7 +546,7 @@ public:
     enum StageHorizontalAlign {
         STAGE_H_ALIGN_C,
         STAGE_H_ALIGN_L,
-        STAGE_H_ALIGN_R,
+        STAGE_H_ALIGN_R
     };
 
     /// The possible vertical position of the Stage
@@ -529,6 +580,14 @@ public:
     /// bottom/centre) as a std::pair
     StageAlign getStageAlignment() const;
 
+	/// Returns the current value of _showMenu which instructs the gui about
+	/// how much to display in the context menu
+	bool getShowMenuState() const;
+
+	/// Sets the value of _showMenu and calls the fscommand handler for the
+	/// current gui
+	void setShowMenuState( bool state );
+
     /// Sets the Stage object's align mode.
     void setStageScaleMode(ScaleMode sm);
     
@@ -568,12 +627,12 @@ public:
     void pushAction(std::auto_ptr<ExecutableCode> code, int lvl=apDOACTION);
 
     /// Push an executable code to the ActionQueue
-    void pushAction(const action_buffer& buf,
-            boost::intrusive_ptr<character> target, int lvl=apDOACTION);
+    void pushAction(const action_buffer& buf, DisplayObject* target,
+            int lvl=apDOACTION);
 
     /// Push a function code to the ActionQueue
-    void pushAction(boost::intrusive_ptr<as_function> func,
-            boost::intrusive_ptr<character> target, int lvl=apDOACTION);
+    void pushAction(as_function* func, DisplayObject* target,
+            int lvl=apDOACTION);
 
 #ifdef GNASH_USE_GC
     /// Mark all reachable resources (for GC)
@@ -585,24 +644,22 @@ public:
     /// - Mouse entities (m_mouse_button_state)
     /// - Timer targets (_intervalTimers)
     /// - Resources reachable by ActionQueue code (_actionQueue)
-    /// - Key listeners (m_key_listeners)
-    /// - Mouse listeners (m_mouse_listeners)
-    /// - global Key object (_keyobject)
-    /// - global Mouse object (_mouseobject)
-    /// - Any character being dragged 
+    /// - Key listeners (_keyListeners)
+    /// - Mouse listeners (_mouseListeners)
+    /// - Any DisplayObject being dragged 
     ///
     void markReachableResources() const;
 #endif // GNASH_USE_GC
 
     /// \brief
-    /// Register a newly born advanceable character to the
-    /// list of characters to be advanced on next ::advance call.
+    /// Register a newly born advanceable DisplayObject to the
+    /// list of DisplayObjects to be advanced on next ::advance call.
     //
-    /// The character will only be advanced if not unloaded when
+    /// The DisplayObject will only be advanced if not unloaded when
     /// its turn comes. Characters are advanced in reverse-placement
     /// order (first registered is advanced last)
     ///
-    void addLiveChar(boost::intrusive_ptr<character> ch)
+    void addLiveChar(DisplayObject* ch)
     {
         // Don't register the object in the list twice 
 #if GNASH_PARANOIA_LEVEL > 1
@@ -648,7 +705,7 @@ public:
     ///
     void flushHigherPriorityActionQueues();
 
-    character* findCharacterByTarget(const std::string& tgtstr) const;
+    DisplayObject* findCharacterByTarget(const std::string& tgtstr) const;
 
     /// Queue a request for loading a movie
     //
@@ -663,8 +720,16 @@ public:
     ///                 key/value pairs
     /// @param method   The VariablesMethod to use for sending the data. If
     ///                 MovieClip::METHOD_NONE, no data will be sent.
+    /// @param handler  An object which will be signalled of load
+    ///                 events (onLoadStart, onLoadComplete, onLoadInit,
+    ///                 onLoadError). Can be null if caller doesn't care.
+    ///                 
     void loadMovie(const std::string& url, const std::string& target,
-            const std::string& data, MovieClip::VariablesMethod method);
+            const std::string& data, MovieClip::VariablesMethod method,
+            as_object* handler=0)
+    {
+        _movieLoader.loadMovie(url, target, data, method, handler);
+    }
 
     /// Send a request to the hosting application (e.g. browser).
     //
@@ -683,17 +748,13 @@ public:
             const std::string& data, MovieClip::VariablesMethod method);
 
 
-    /// Return true if the given string can be interpreted as a _level name
-    //
-    /// @param name
-    ///   The target string.
-    ///   Will be considered case-insensitive if VM version is < 7.
-    ///
-    /// @param levelno
-    ///   Output parameter, will be set to the level number, if true is
-    ///   returned
-    bool isLevelTarget(const std::string& name, unsigned int& levelno);
+    key::code lastKeyEvent() const {
+        return _lastKeyEvent;
+    }
 
+    const Keys& unreleasedKeys() const {
+        return _unreleasedKeys;
+    }
 
     /// Set a filedescriptor to use for host application requests
     /// (for browser communication mostly)
@@ -738,7 +799,8 @@ public:
             const std::string& arg) const;
 
     /// Abstract base class for hosting app handler
-    class AbstractIfaceCallback {
+    class AbstractIfaceCallback
+    {
     public:
 
         /// Get Gui-related information for the core.
@@ -747,7 +809,7 @@ public:
         /// Mouse.hide, System.capabilities etc. The return can be
         /// various types, so it is passed as a string.
         virtual std::string call(const std::string& cmd,
-                const std::string& arg) = 0;
+                const std::string& arg = std::string()) = 0;
 
         /// Ask the hosting application for a yes / no answer to
         /// a question.
@@ -778,6 +840,12 @@ public:
     DSOEXPORT std::string callInterface(const std::string& cmd,
             const std::string& arg = std::string()) const;
 
+    /// Send an error message to the hosting application.
+    //
+    /// @param msg  A message to send describing the error.
+    //
+    /// The hosting app decides what to do with the message, or whether it
+    /// wants to do anything at all. It may show a popup box.
     DSOEXPORT void errorInterface(const std::string& msg) const;
 
     /// Called from the ScriptLimits tag parser to set the
@@ -823,27 +891,36 @@ public:
 	///
 	const std::string& getOriginalURL() const { return _originalURL; }
 
-    const RunInfo& runInfo() const { return _runInfo; }
-
-    /// Add a character child on top depth
-    //
-    /// @param ch
-    ///     The child character to add
-    void addChild(character* ch);
-
-    /// Add a character child at given depth
-    //
-    /// @param ch
-    ///     The child character to add
-    ///
-    /// @param depth
-    ///     The depth to add the child to
-    ///
-    void addChildAt(character* ch, int depth);
+    const RunResources& runResources() const { return _runResources; }
 
 private:
 
-    const RunInfo& _runInfo; 
+    /// Set the root movie, replacing the current one if any.
+    //
+    /// This is needed for the cases in which the top-level movie
+    /// is replaced by another movie by effect of a loadMovie call
+    /// or similar.
+    ///
+    /// TODO: inspect what happens about VM version
+    ///   (should the *new* movie drive VM operations?
+    ///    -- hope not ! )
+    ///
+    /// Make sure to call this method before using the movie_root,
+    /// as most operations are delegated to the associated/wrapped
+    /// Movie.
+    ///
+    /// Note that the display viewport will be updated to match
+    /// the size of given movie.
+    ///
+    /// A call to this method is equivalent to a call to setLevel(0, movie).
+    ///
+    /// @param movie
+    /// The Movie to wrap.
+    /// Must have a depth of 0.
+    ///
+    void setRootMovie(Movie* movie);
+
+    const RunResources& _runResources; 
 
     /// The URL of the original root movie.
     //
@@ -861,54 +938,8 @@ private:
     /// Registered FsCommand handler, if any
     AbstractFsCallback* _fsCommandHandler;
 
-    /// A load movie request
-    class LoadMovieRequest {
-    public:
-        /// @param postdata
-        ///   If not null POST method will be used for HTTP.
-        ///
-        LoadMovieRequest(const URL& u, const std::string& t,
-                const std::string* postdata)
-                :
-                _target(t),
-                _url(u),
-                _usePost(false)
-        {
-            if ( postdata )
-            {
-                _postData = *postdata;
-                _usePost = true;
-            }
-        }
-
-        const std::string& getTarget() const { return _target; }
-        const URL& getURL() const { return _url; }
-        const std::string& getPostData() const { return _postData; }
-        bool usePost() const { return _usePost; }
-
-    private:
-        std::string _target;
-        URL _url;
-        bool _usePost;
-        std::string _postData;
-    };
-
-    /// Load movie requests
-    typedef std::list<LoadMovieRequest> LoadMovieRequests;
-    LoadMovieRequests _loadMovieRequests;
-
-    /// Process all load movie requests
-    void processLoadMovieRequests();
-
-    /// Process a single load movie request
-    void processLoadMovieRequest(const LoadMovieRequest& r);
-
     /// Listeners container
-    typedef std::list< boost::intrusive_ptr<character> > CharacterList;
-
-    /// key and mouse listeners container
-    typedef CharacterList KeyListeners;
-    typedef CharacterList MouseListeners;
+    typedef std::list<DisplayObject*> Listeners;
 
     /// Take care of dragging, if needed
     void doMouseDrag();
@@ -919,129 +950,37 @@ private:
     /// Delete all elements on the timers list
     void clearIntervalTimers();
 
-    /// An element of the advanceable characters
-    typedef boost::intrusive_ptr<character> AdvanceableCharacter;
-
-    /// A list of AdvanceableCharacters
-    //
-    /// This is a list (not a vector) as we want to allow
-    /// ::advance of each element to insert new characters before
-    /// the start w/out invalidating iterators scanning the
-    /// list forward for proper movie advancement
-    typedef std::list<AdvanceableCharacter> LiveChars;
-
-    /// The list of advanceable character, in placement order
-    LiveChars _liveChars;
-
+    /// Execute expired timers
+    void executeAdvanceCallbacks();
+    
     /// Execute expired timers
     void executeTimers();
 
-    /// Notify the global Key ActionScript object about a key status change
-    Key_as * notify_global_key(key::code k, bool down);
-
     /// Remove unloaded key and mouselisteners.
     void cleanupUnloadedListeners()
     {
-        cleanupUnloadedListeners(m_key_listeners);
-        cleanupUnloadedListeners(m_mouse_listeners);
+        cleanupUnloadedListeners(_keyListeners);
+        cleanupUnloadedListeners(_mouseListeners);
     }
 
-    /// Erase unloaded characters from the given listeners list
-    static void cleanupUnloadedListeners(CharacterList& ll);
+    /// Erase unloaded DisplayObjects from the given listeners list
+    static void cleanupUnloadedListeners(Listeners& ll);
 
-    /// Cleanup references to unloaded characters and run the garbage collector.
+    /// Cleanup references to unloaded DisplayObjects and run the GC.
     void cleanupAndCollect();
 
-    /// Push a character listener to the front of given container, if not
+    /// Push a DisplayObject listener to the front of given container, if not
     /// already present
-    static void add_listener(CharacterList& ll, character* elem);
+    static void add_listener(Listeners& ll, DisplayObject* elem);
 
     /// Remove a listener from the list
-    static void remove_listener(CharacterList& ll, character* elem);
-
-    /// Return the current Stage object
-    //
-    /// Can return NULL if it's been deleted or not
-    /// yet initialized.
-    boost::intrusive_ptr<Stage_as> getStageObject();
-
-    /// Return the singleton Selection object
-    //
-    /// Can return 0 if it's been deleted.
-    as_object* getSelectionObject() const;
-
-    typedef std::list<ExecutableCode*> ActionQueue;
-
-    ActionQueue _actionQueue[apSIZE];
-
-    /// Process all actions in the queue
-    void processActionQueue();
-
-    // TODO: use Range2d<int> ?
-    int m_viewport_x0, m_viewport_y0;
-
-    /// Width and height of viewport, in pixels
-    int m_viewport_width, m_viewport_height;
-
-    rgba m_background_color;
-    bool m_background_color_set;
-
-    float m_timer;
-    int m_mouse_x, m_mouse_y, m_mouse_buttons;
-
-    MouseButtonState  m_mouse_button_state;
-
-    typedef std::map<int, Timer*> TimerMap;
-
-    TimerMap _intervalTimers;
-    unsigned int _lastTimerId;
-
-    /// Characters for listening key events
-    KeyListeners m_key_listeners;
-
-    boost::intrusive_ptr<Key_as> _keyobject;
-
-    boost::intrusive_ptr<as_object> _mouseobject;
-
-    /// Objects listening for mouse events (down,up,move)
-    MouseListeners m_mouse_listeners;
-
-    /// The character currently holding focus, or 0 if no focus.
-    boost::intrusive_ptr<character> _currentFocus;
-
-    float m_time_remainder;
-
-    /// @todo fold this into m_mouse_button_state?
-    drag_state m_drag_state;
-
-    typedef boost::intrusive_ptr<MovieClip> LevelMovie;
-    typedef std::map<int, LevelMovie> Levels;
-
-    /// The movie instance wrapped by this movie_root
-    //
-    /// We keep a pointer to the base MovieClip class
-    /// to avoid having to replicate all of the base class
-    /// interface to the movie_instance class definition
-    Levels _movies;
-
-    typedef character* DisplayObject;
-    typedef std::map<int, DisplayObject> Childs;
-
-    /// The stage childs
-    Childs _childs;
-
-    /// The root movie. This is initially the same as getLevel(0) but might
-    /// change during the run. It will be used to setup and retrive initial
-    /// stage size
-    boost::intrusive_ptr<movie_instance> _rootMovie;
+    static void remove_listener(Listeners& ll, DisplayObject* elem);
 
     /// This function should return TRUE iff any action triggered
     /// by the event requires redraw, see \ref events_handling for
     /// more info.
     bool fire_mouse_event();
 
-    bool generate_mouse_button_events();
-
     /// \brief
     /// Return the topmost entity covering the given point
     /// and enabled to receive mouse events.
@@ -1057,43 +996,23 @@ private:
     /// @param y
     ///     Y ordinate of the pointer, in world coordiante space (twips).
     ///
-    character* getTopmostMouseEntity(boost::int32_t x, boost::int32_t y);
+    InteractiveObject* getTopmostMouseEntity(boost::int32_t x,
+            boost::int32_t y) const;
 
-    /// Delete characters removed from the stage
+    /// Delete DisplayObjects removed from the stage
     /// from the display lists
     void cleanupDisplayList();
 
-    /// Advance a live character
+    /// Advance a live DisplayObject
     //
     /// @param ch
-    ///     The character to advance, will NOT be advanced if unloaded
+    ///     The DisplayObject to advance, will NOT be advanced if unloaded
     ///
-    static void advanceLiveChar(boost::intrusive_ptr<character> ch);
+    static void advanceLiveChar(DisplayObject* ch);
 
     /// Advance all non-unloaded live chars
     void advanceLiveChars();
 
-    /// Put the given movie at the given level 
-    //
-    /// @param movie
-    /// The movie_instance to store at the given level.
-    /// Will be stored in an intrusive_ptr.
-    /// Its depth will be set to <num>+character::staticDepthOffset and
-    /// its name to _level<num>
-    void setLevel(unsigned int num, boost::intrusive_ptr<movie_instance> movie);
-
-    /// Return the global Key object 
-    //
-    /// @@ might be worth making public
-    ///
-    boost::intrusive_ptr<Key_as> getKeyObject();
-
-    /// Return the global Mouse object 
-    //
-    /// TODO: expose the Mouse as_object directly for faster calls ?
-    ///
-    boost::intrusive_ptr<as_object> getMouseObject();
-
     /// Boundaries of the Stage are always world boundaries
     /// and are only invalidated by changes in the background
     /// color.
@@ -1115,13 +1034,6 @@ private:
     ///
     bool isInvalidated() { return _invalidated; }
 
-    /// See setInvalidated
-    bool _invalidated;
-
-    /// This is set to true if execution of scripts
-    /// aborted due to action limit set or whatever else
-    bool _disableScripts;
-
     /// Return the priority level of first action queue containing actions.
     //
     /// Scanned in proprity order (lower first)
@@ -1133,16 +1045,103 @@ private:
     /// return.
     int processActionQueue(int lvl);
 
-    int _processingActionLevel;
-
     bool processingActions() const
     {
         return (_processingActionLevel < apSIZE);
     }
 
-    const character* findDropTarget(boost::int32_t x, boost::int32_t y,
-            character* dragging) const;
+    const DisplayObject* findDropTarget(boost::int32_t x, boost::int32_t y,
+            DisplayObject* dragging) const;
 
+    void handleActionLimitHit(const std::string& ref);
+    /// A list of AdvanceableCharacters
+    //
+    /// This is a list (not a vector) as we want to allow
+    /// ::advance of each element to insert new DisplayObjects before
+    /// the start w/out invalidating iterators scanning the
+    /// list forward for proper movie advancement
+    typedef std::list<DisplayObject*> LiveChars;
+
+    /// The list of advanceable DisplayObject, in placement order
+    LiveChars _liveChars;
+
+
+    /// A number of queues of code to execute
+    //
+    /// This is a deque because it needs no insertion in the middle but
+    /// frequent push_back and pop_front. We also have to traverse it, so
+    /// a std::queue is not usable.
+    typedef std::deque<ExecutableCode*> ActionQueue;
+    ActionQueue _actionQueue[apSIZE];
+
+    /// Process all actions in the queue
+    void processActionQueue();
+
+    int m_viewport_x0, m_viewport_y0;
+
+    /// Width and height of viewport, in pixels
+    int m_viewport_width, m_viewport_height;
+
+    rgba m_background_color;
+    bool m_background_color_set;
+
+    float m_timer;
+    int m_mouse_x, m_mouse_y, m_mouse_buttons;
+
+    MouseButtonState  _mouseButtonState;
+
+    /// Objects requesting a callback on every movie_root::advance()
+    typedef std::set<ActiveRelay*> ObjectCallbacks;
+    ObjectCallbacks _objectCallbacks;
+
+    LoadCallbacks _loadCallbacks;
+
+    typedef std::map<int, Timer*> TimerMap;
+
+    TimerMap _intervalTimers;
+    unsigned int _lastTimerId;
+
+    /// bit-array for recording the unreleased keys
+    Keys _unreleasedKeys;   
+
+    key::code _lastKeyEvent;
+
+    /// Characters for listening key events
+    Listeners _keyListeners;
+
+    /// Objects listening for mouse events (down,up,move)
+    Listeners _mouseListeners;
+
+    /// The DisplayObject currently holding focus, or 0 if no focus.
+    DisplayObject* _currentFocus;
+
+    /// @todo fold this into m_mouse_button_state?
+    drag_state m_drag_state;
+
+    typedef MovieClip* LevelMovie;
+    typedef std::map<int, LevelMovie> Levels;
+
+    /// The movie instance wrapped by this movie_root
+    //
+    /// We keep a pointer to the base MovieClip class
+    /// to avoid having to replicate all of the base class
+    /// interface to the Movie class definition
+    Levels _movies;
+
+    /// The root movie. This is initially the same as getLevel(0) but might
+    /// change during the run. It will be used to setup and retrive initial
+    /// stage size
+    Movie* _rootMovie;
+
+    /// See setInvalidated
+    bool _invalidated;
+
+    /// This is set to true if execution of scripts
+    /// aborted due to action limit set or whatever else
+    bool _disableScripts;
+
+    int _processingActionLevel;
+    
     /// filedescriptor to write to for host application requests
     //
     /// -1 if none
@@ -1150,11 +1149,13 @@ private:
 
     /// The display quality of the entire movie.
     //
-    /// This is here, not just in the render_handler, so that AS compatibility
+    /// This is here, not just in the Renderer, so that AS compatibility
     /// does not rely on the presence of a renderer.
     Quality _quality;
 
     std::bitset<4u> _alignMode;
+
+	bool _showMenu;
     
     ScaleMode _scaleMode;
     
@@ -1168,15 +1169,31 @@ private:
     // ScriptLimits tag.    
     boost::uint16_t _timeoutLimit;
 
-    void handleActionLimitHit(const std::string& ref);
-
     // delay between movie advancement, in milliseconds
     unsigned int _movieAdvancementDelay;
 
     // time of last movie advancement, in milliseconds
     unsigned int _lastMovieAdvancement;
+
+    size_t _unnamedInstance;
+
+    MovieLoader _movieLoader;
+
 };
 
+/// Return true if the given string can be interpreted as a _level name
+//
+/// @param name
+///   The target string.
+///   Will be considered case-insensitive if VM version is < 7.
+///
+/// @param levelno
+///   Output parameter, will be set to the level number, if true is
+///   returned
+bool isLevelTarget(int version, const std::string& name, unsigned int& levelno);
+
+DSOEXPORT short stringToStageAlign(const std::string& s);
+
 } // namespace gnash
 
 #endif // GNASH_MOVIE_ROOT_H
