@@ -1,6 +1,6 @@
 // BitmapMovieDefinition.cpp:  Bitmap movie definition, for Gnash.
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,13 +25,16 @@
 #include "log.h"
 #include "Bitmap.h"
 #include "Renderer.h"
+#include "Global_as.h"
+#include "namedStrings.h"
 
 namespace gnash {
 
 Movie*
-BitmapMovieDefinition::createMovie(DisplayObject* parent)
+BitmapMovieDefinition::createMovie(Global_as& gl, DisplayObject* parent)
 {
-    return new BitmapMovie(this, parent);
+    as_object* o = getObjectWithPrototype(gl, NSV::CLASS_MOVIE_CLIP);
+    return new BitmapMovie(o, this, parent);
 }
 
 BitmapMovieDefinition::BitmapMovieDefinition(std::auto_ptr<GnashImage> image,
@@ -48,18 +51,11 @@ BitmapMovieDefinition::BitmapMovieDefinition(std::auto_ptr<GnashImage> image,
 }
 
 DisplayObject*
-BitmapMovieDefinition::createDisplayObject(DisplayObject* parent, int id) const
+BitmapMovieDefinition::createDisplayObject(Global_as& /*gl*/,
+        DisplayObject* /*parent*/) const
 {
-    /// What should we do if construction of the bitmap fails?
-    return new Bitmap(this, parent, id);
+    std::abort();
+    return 0;
 }
 
-#ifdef GNASH_USE_GC
-void
-BitmapMovieDefinition::markReachableResources() const
-{
-	if (_bitmap.get()) _bitmap->setReachable();
-}
-#endif // GNASH_USE_GC
-
 } // namespace gnash
