@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,23 +20,26 @@
 
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "movie_definition.h" // for inheritance
-#include "rect.h" // for composition
-#include "BitmapMovieInstance.h" // for create_movie_instance
+#include "SWFRect.h" // for composition
 #include "BitmapInfo.h" // for destructor visibility by intrusive_ptr
 #include "DynamicShape.h" // for destructor visibility by intrusive_ptr
 #include "GnashImage.h"
+#include "GnashNumeric.h"
 
 #include <string>
 #include <memory> // for auto_ptr
 
 // Forward declarations
+namespace gnash {
+    class Renderer;
+}
 
 namespace gnash
 {
 
 /// A "movie" definition for a bitmap file
 //
-/// The create_movie_instance function will return a BitmapMovieInstance
+/// The createMovie function will return a BitmapMovie
 ///
 class BitmapMovieDefinition : public movie_definition
 {
@@ -54,39 +57,33 @@ public:
 	///  - image->size() bytes (for get_bytes_loaded()/get_bytes_total())
 	///  - provided url
 	///
-	BitmapMovieDefinition(std::auto_ptr<GnashImage> image, const std::string& url);
+	BitmapMovieDefinition(std::auto_ptr<GnashImage> image, Renderer* renderer,
+            const std::string& url);
 
-	// Discard id, always return the only shape character we have 
-	virtual character_def* get_character_def(int /*id*/)
-	{
-		return getShapeDef();
-	}
+    virtual DisplayObject* createDisplayObject(Global_as&, DisplayObject*)
+        const;
 
 	virtual int	get_version() const {
 		return _version;
 	}
 
 	virtual float get_width_pixels() const {
-		return std::ceil(TWIPS_TO_PIXELS(_framesize.width()));
+		return std::ceil(twipsToPixels(_framesize.width()));
 	}
 
 	virtual float get_height_pixels() const {
-		return std::ceil(TWIPS_TO_PIXELS(_framesize.height()));
+		return std::ceil(twipsToPixels(_framesize.height()));
 	}
 
-	virtual size_t	get_frame_count() const {
+	virtual size_t get_frame_count() const {
 		return _framecount;
 	}
 
-	virtual float	get_frame_rate() const {
+	virtual float get_frame_rate() const {
 		return _framerate;
 	}
 
-	virtual const rect& get_frame_size() const {
-		return _framesize;
-	}
-
-	virtual const rect& get_bound() const {
+	virtual const SWFRect& get_frame_size() const {
 		return _framesize;
 	}
 
@@ -107,11 +104,8 @@ public:
 		return _bytesTotal;
 	}
 	
-	/// Create a playable movie_instance from this def.
-	virtual movie_instance* create_movie_instance(character* parent=0)
-	{
-		return new BitmapMovieInstance(this, parent);
-	}
+	/// Create a playable Movie from this def.
+	virtual Movie* createMovie(Global_as& gl, DisplayObject* parent = 0);
 
 	virtual const std::string& get_url() const {
 		return _url;
@@ -126,26 +120,16 @@ public:
 		return 1;
 	}
 
-    BitmapInfo* getBitmap() const {
+    const BitmapInfo* bitmap() const {
         return _bitmap.get();
     }
 
 protected:
 
-#ifdef GNASH_USE_GC
-	/// Mark reachable resources of a BitmapMovieDefinition
-	//
-	/// Reachable resources are:
-	///	- dynamic shape (_shapedef)
-	///	- bitmap (_bitmap)
-	///
-	void markReachableResources() const;
-#endif // GNASH_USE_GC
-
 private:
 
 	int _version;
-	rect _framesize;
+	SWFRect _framesize;
 	size_t _framecount;
 	float _framerate;
 	std::string _url;
@@ -153,15 +137,6 @@ private:
 	size_t _bytesTotal;
 
     boost::intrusive_ptr<BitmapInfo> _bitmap;
-
-	boost::intrusive_ptr<DynamicShape> _shapedef;
-
-	/// Get the shape character definition for this bitmap movie
-	//
-	/// It will create the definition the first time it's called
-	///
-	shape_character_def* getShapeDef();
-
 };
 
 } // namespace gnash
