@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -101,13 +102,13 @@ public:
 	typedef std::vector<SWF::ControlTag*> PlayList;
 
 	virtual int	get_version() const = 0;
-	virtual float	get_width_pixels() const = 0;
-	virtual float	get_height_pixels() const = 0;
-	virtual size_t	get_frame_count() const = 0;
-	virtual float	get_frame_rate() const = 0;
+	virtual float get_width_pixels() const = 0;
+	virtual float get_height_pixels() const = 0;
+	virtual size_t get_frame_count() const = 0;
+	virtual float get_frame_rate() const = 0;
 
 	/// Return size of frame, in TWIPS 
-	virtual const rect& get_frame_size() const = 0;
+	virtual const SWFRect& get_frame_size() const = 0;
 
 	virtual size_t get_bytes_loaded() const = 0;
 
@@ -134,7 +135,7 @@ public:
 	/// SWFMovieDefinition is one such example, future examples
 	/// should include jpeg_movie_def and similar..
 	///
-	virtual Movie* createMovie(DisplayObject* /*parent*/=0)
+	virtual Movie* createMovie(Global_as& /*gl*/, DisplayObject* /*parent*/=0)
 	{
 		return NULL;
 	}
