@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,38 +16,49 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#ifndef GNASH_EXECUTE_TAG_H
-#define GNASH_EXECUTE_TAG_H
+#ifndef GNASH_CONTROL_TAG_H
+#define GNASH_CONTROL_TAG_H
 
 #include <cassert>
 
-namespace gnash {
-
 // Forward declarations
-class MovieClip;
-class DisplayList;
+namespace gnash {
+    class MovieClip;
+    class DisplayList;
+}
+
+namespace gnash {
+namespace SWF {
 
 /// Control tags are swf tags that control the operation of the movie. 
 //
-/// Essentially, these are the events associated with a frame.
+/// These are the events associated with a frame.
 ///
 class ControlTag
 {
 public:
+    
+    /// Type of ControlTag
+    enum Type
+    {
+        TAG_ACTION = 1 << 0,
+        TAG_DLIST  = 1 << 1
+    };
 
 	virtual ~ControlTag()
 	{
 	}
 
 	/// Execute this tag, whatever it is.
-	virtual void execute(MovieClip* /*m*/, DisplayList& /*dlist*/) const
+	//
+    /// The default does nothing.
+    virtual void execute(MovieClip* /*m*/, DisplayList& /*dlist*/) const
 	{
 	}
 
 	/// Execute this tag but only if it's a "state" tag.
 	//
 	/// State tags include all tags except action tags.
-	///
 	virtual void execute_state(MovieClip* /*m*/,  DisplayList& /*dlist*/) const
 	{
 	}
@@ -54,19 +66,21 @@ public:
 	/// Execute this tag but only if it is an action tag
 	void execute_action(MovieClip* m, DisplayList& dlist) const
 	{
-		if ( is_action_tag() ) execute(m, dlist);
+		if (is_action_tag()) execute(m, dlist);
 	}
 
 	/// Return true if this is an action tag.
-	virtual bool	is_action_tag() const { return false; }
+    //
+    /// The default returns false.
+	virtual bool is_action_tag() const { return false; }
 
 };
 
 
+} // namespace SWF
 } // namespace gnash
 
-
-#endif // GNASH_EXECUTE_TAG_H
+#endif 
 
 
 // Local Variables:
