@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,19 +16,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-
 #ifndef GNASH_SWF_VIDEOFRAMETAG_H
 #define GNASH_SWF_VIDEOFRAMETAG_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "DefinitionTag.h"
 #include "movie_definition.h"
 #include "SWF.h"
-#include "rect.h" // for composition
+#include "SWFRect.h" // for composition
 #include "ControlTag.h"
 #include "VideoDecoder.h"
 
