@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,13 +19,10 @@
 #ifndef GNASH_EXECUTABLECODE_H
 #define GNASH_EXECUTABLECODE_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include "smart_ptr.h" // GNASH_USE_GC
 #include "as_function.h"
 #include "ActionExec.h"
+#include "Global_as.h"
 #include "fn_call.h"
 
 namespace gnash
@@ -50,11 +47,11 @@ public:
 };
 
 /// Global code (out of any function)
-class GlobalCode: public ExecutableCode {
+class GlobalCode : public ExecutableCode {
 
 public:
 
-    GlobalCode(const action_buffer& nBuffer, boost::intrusive_ptr<DisplayObject> nTarget)
+    GlobalCode(const action_buffer& nBuffer, DisplayObject* nTarget)
         :
         buffer(nBuffer),
         target(nTarget)
@@ -67,15 +64,10 @@ public:
 
     virtual void execute()
     {
-        if ( ! target->unloaded() )
-        {
+        if (!target->unloaded()) {
             ActionExec exec(buffer, target->get_environment());
             exec();
         }
-        else
-        {
-            //log_debug("Sprite %s unloaded, won't execute global code in it", target->getTargetPath().c_str());
-        }
     }
 
 #ifdef GNASH_USE_GC
@@ -84,9 +76,8 @@ public:
     /// Reachable resources are:
     ///  - the action target (target)
     ///
-    virtual void markReachableResources() const
-    {
-        if ( target ) target->setReachable();
+    virtual void markReachableResources() const {
+        if (target) target->setReachable();
     }
 #endif // GNASU_USE_GC
 
@@ -94,22 +85,22 @@ private:
 
     const action_buffer& buffer;
 
-    boost::intrusive_ptr<DisplayObject> target;
+    DisplayObject* target;
 };
 
 /// Event code 
-class EventCode: public ExecutableCode {
+class EventCode : public ExecutableCode {
 
 public:
 
     typedef std::vector<const action_buffer*> BufferList;
 
-    EventCode(boost::intrusive_ptr<DisplayObject> nTarget)
+    EventCode(DisplayObject* nTarget)
         :
         _target(nTarget)
     {}
 
-    EventCode(boost::intrusive_ptr<DisplayObject> nTarget, const BufferList& buffers)
+    EventCode(DisplayObject* nTarget, const BufferList& buffers)
         :
         _target(nTarget),
         _buffers(buffers)
@@ -132,8 +123,7 @@ public:
     {
         // don't push actions for destroyed DisplayObjects, 
         // our opcode guard is bogus at the moment.
-        if( ! _target->isDestroyed() )
-        {
+        if (!_target->isDestroyed()) {
             _buffers.push_back(&buffer);
         }
     }
@@ -145,7 +135,7 @@ public:
         {
             // onClipEvents code are guarded by isDestroyed(),
             // still might be also guarded by unloaded()
-            if( _target->isDestroyed() )  break;
+            if (_target->isDestroyed())  break;
 
             ActionExec exec(*(*it), _target->get_environment(), false);
             exec();
@@ -166,18 +156,18 @@ public:
 
 private:
 
-    boost::intrusive_ptr<DisplayObject> _target;
+    DisplayObject* _target;
 
     BufferList _buffers;
 
 };
 
-/// Generic event  (constructed by id, invoked using on_event
+/// Generic event  (constructed by id, invoked using notifyEvent
 class QueuedEvent: public ExecutableCode {
 
 public:
 
-    QueuedEvent(boost::intrusive_ptr<DisplayObject> nTarget, const event_id& id)
+    QueuedEvent(DisplayObject* nTarget, const event_id& id)
         :
         _target(nTarget),
         _eventId(id)
@@ -194,7 +184,7 @@ public:
         // don't execute any events for destroyed DisplayObject.
         if( !_target->isDestroyed() )
         {
-            _target->on_event(_eventId);
+            _target->notifyEvent(_eventId);
         }
     }
 
@@ -212,7 +202,7 @@ public:
 
 private:
 
-    boost::intrusive_ptr<DisplayObject> _target;
+    DisplayObject* _target;
 
     const event_id _eventId;
 
@@ -223,7 +213,7 @@ class FunctionCode: public ExecutableCode {
 
 public:
 
-    FunctionCode(boost::intrusive_ptr<as_function> nFunc, boost::intrusive_ptr<DisplayObject> nTarget)
+    FunctionCode(as_function* nFunc, DisplayObject* nTarget)
         :
         func(nFunc),
         target(nTarget)
@@ -236,8 +226,8 @@ public:
 
     virtual void execute()
     {
-        as_environment env(getVM(*func)); env.set_target(target.get());
-        func->call(fn_call(target.get(), env));
+        as_environment env(getVM(*func)); env.set_target(target);
+        func->call(fn_call(getObject(target), env));
     }
 
 #ifdef GNASH_USE_GC
@@ -249,16 +239,74 @@ public:
     ///
     virtual void markReachableResources() const
     {
-        if ( func ) func->setReachable();
-        if ( target ) target->setReachable();
+        if (func) func->setReachable();
+        if (target) target->setReachable();
     }
 #endif // GNASU_USE_GC
 
 private:
 
-    boost::intrusive_ptr<as_function> func;
+    as_function* func;
+
+    DisplayObject* target;
+};
+
+/// This class is used to queue a function call action
+//
+/// Exact use is to queue onLoadInit, which should be invoked
+/// after actions of in first frame of a loaded movie are executed.
+/// Since those actions are queued the only way to execute something
+/// after them is to queue the function call as well.
+///
+/// The class might be made more general and accessible outside
+/// of the MovieClipLoader class. For now it only works for
+/// calling a function with a two argument.
+///
+class DelayedFunctionCall : public ExecutableCode
+{
+
+public:
+
+    DelayedFunctionCall(as_object* target, string_table::key name,
+            const as_value& arg1, const as_value& arg2)
+        :
+        _target(target),
+        _name(name),
+        _arg1(arg1),
+        _arg2(arg2)
+    {}
+
+
+    ExecutableCode* clone() const
+    {
+        return new DelayedFunctionCall(*this);
+    }
+
+    virtual void execute()
+    {
+        callMethod(_target, _name, _arg1, _arg2);
+    }
+
+#ifdef GNASH_USE_GC
+    /// Mark reachable resources (for the GC)
+    //
+    /// Reachable resources are:
+    ///  - the action target (_target)
+    ///
+    virtual void markReachableResources() const
+    {
+      _target->setReachable();
+      _arg1.setReachable();
+      _arg2.setReachable();
+    }
+#endif // GNASH_USE_GC
+
+private:
+
+    as_object* _target;
+    string_table::key _name;
+    as_value _arg1, _arg2;
 
-    boost::intrusive_ptr<DisplayObject> target;
 };
 
 
