@@ -1,6 +1,6 @@
 // Machine.h A VM to run AS3 code, and AS2 code in the future.
 //
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,18 +23,23 @@
 #include <sstream>
 #include "SafeStack.h"
 #include "as_value.h"
-#include "asClass.h"
 #include "SWF.h"
 #include "as_environment.h"
 #include "VM.h"
 #include "fn_call.h"
 
 namespace gnash {
+    namespace abc {
+        class AbcBlock;
+        class MultiName;
+        class Class;
+        class abc_function;
+        class Method;
+        class Namespace;
+    }
     class Global_as;
     class DisplayObject;
     class as_object;
-    class AbcBlock;
-    class asName;
     class Property;
     class CodeStream;
 }
@@ -42,6 +47,10 @@ namespace gnash {
 
 namespace gnash {
 
+namespace abc {
+
+/// The virtual machine for executing ABC (ActionScript Bytecode).
+//
 /// This machine is intended to work without relying on the C++ call stack,
 /// by resetting its Stream and Stack members (actually, by limiting the stack)
 /// to make function calls, rather than calling them directly in C++.
@@ -93,7 +102,7 @@ public:
 	/// the stack.
 	///
 	/// @param name
-	/// A partially filled asBoundName, this should be the id
+	/// A partially filled MultiName, this should be the id
 	/// from the stream.
 	///
 	/// @param initial
@@ -102,7 +111,7 @@ public:
 	/// @return
 	/// The number of stack elements used by the name.
 	/// At present, always 0, 1, or 2. These are not dropped.
-	int completeName(asName& name, int initial = 0);
+	int completeName(MultiName& name, int initial = 0);
 
 	/// Given a value v, find the class object of the superclass of v.
 	///
@@ -114,7 +123,7 @@ public:
 	///
 	/// @return
 	/// Null if the superclass was not found, or the superclass.
-	asClass* findSuper(as_value& obj, bool find_primitive);
+	Class* findSuper(as_value& obj, bool find_primitive);
 
 	/// Get a member from an object.
 	///
@@ -132,7 +141,7 @@ public:
 	/// This returns the value, but on the stack.
 	/// (Since the return value is not known until after control has left
 	/// the caller of this, it's impossible to return a meaningful value.
-	void getMember(asClass* pDefinition, asName& name, as_value& source);
+	void getMember(Class* pDefinition, MultiName& name, as_value& source);
 
 	/// Set a member in an object.
 	///
@@ -150,9 +159,9 @@ public:
 	///
 	/// @return
 	/// Nothing.
-	void setMember(asClass*, asName&, as_value& target, as_value& val);
+	void setMember(Class*, MultiName&, as_value& target, as_value& val);
 
-	asBinding* findProperty(asName&) { return NULL; }
+	Property* findProperty(MultiName&) { return NULL; }
 
 	void execute();
 
@@ -231,7 +240,7 @@ public:
 
 	void initMachine(AbcBlock* pool_block);
 
-	as_value executeFunction(asMethod* function, const fn_call& fn);
+	as_value executeFunction(Method* function, const fn_call& fn);
 
 	void instantiateClass(std::string className, as_object* global);
     /// Return the Global object for this Machine.
@@ -255,7 +264,7 @@ private:
 		unsigned int mScopeTotalSize;
 		bool mReturn;
 		CodeStream *mStream;
-		asNamespace *mDefaultXMLNamespace;
+		Namespace *mDefaultXMLNamespace;
 		as_object *mCurrentScope;
 		as_value *mGlobalReturn;
 		as_object *mThis;
@@ -282,7 +291,7 @@ private:
 	void saveState();
 	void restoreState();
 
-	as_value find_prop_strict(asName multiname);
+	as_value find_prop_strict(MultiName multiname);
 
 	void print_stack();
 
@@ -323,15 +332,14 @@ private:
 
 	void push_scope_stack(as_value object);
 
-	boost::intrusive_ptr<as_object> pop_scope_stack() {
+	as_object* pop_scope_stack() {
 		log_abc("Popping value %s off the scope stack.  There will be "
                 "%u items left.", as_value(_scopeStack.top(0)),
                 _scopeStack.size()-1);
 		return _scopeStack.pop();
 	}
 
-	boost::intrusive_ptr<as_object> get_scope_stack(boost::uint8_t depth)
-        const {
+	as_object* get_scope_stack(boost::uint8_t depth) const {
 		log_abc("Getting value from scope stack %u from the bottom.",
                 depth | 0x0);
 		return _scopeStack.value(depth);
@@ -349,13 +357,13 @@ private:
     /// before.
     /// Most importantly, the complete stack is used for lookups, including
     /// the section that is not changeable.
-	SafeStack<boost::intrusive_ptr<as_object> > _scopeStack;
+	SafeStack<as_object*> _scopeStack;
 
     CodeStream *mStream;
 
 	string_table& mST;
 
-	asNamespace* mDefaultXMLNamespace;
+	Namespace* mDefaultXMLNamespace;
 	as_object* mCurrentScope;
 	as_object* mGlobalScope;
 	as_object* mDefaultThis;
@@ -368,12 +376,12 @@ private:
 	as_value mIgnoreReturn; // Throw away returns go here.
 
 	bool mExitWithReturn;
-	AbcBlock* mPoolObject; // Where all of the pools are stored.
+    AbcBlock* mPoolObject; // Where all of the pools are stored.
 
 	abc_function* mCurrentFunction;
 
 	VM& _vm;
 };
-
+} // namespace abc
 } // namespace gnash
 #endif 
