@@ -1,6 +1,7 @@
 // VM.h: the Virtual Machine class, for Gnash
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,45 +24,45 @@
 #include "gnashconfig.h"
 #endif
 
-#include "smart_ptr.h" // GNASH_USE_GC
-#include "movie_root.h" // for composition
-#include "GC.h" // for ineritance of VmGcRoot
-#include "string_table.h" // for the string table
+#include "smart_ptr.h" 
+#include "GC.h"
+#include "string_table.h"
+#include "SafeStack.h"
+#include "CallStack.h"
 
-#include <memory> // for auto_ptr
+#include <memory> 
 #include <locale>
-#include <boost/cstdint.hpp> // for cstdints 
+#include <boost/cstdint.hpp> 
 #include <boost/random.hpp>
 #include <boost/noncopyable.hpp>
 
 // Forward declarations
 namespace gnash {
-	class movie_definition;
-	class builtin_function;
+    namespace abc {
+        class Machine;
+    }
+	class Global_as;
+	class VM;
+	class fn_call;
+	class movie_root;
+	class NativeFunction;
     class SharedObjectLibrary;
+	class as_value;
 	class as_object;
-	class Machine;
 	class VirtualClock;
 }
 
 namespace gnash {
 
-class ClassHierarchy;
-
 /// A GC root used to mark all reachable collectable pointers
 class VmGcRoot : public GcRoot 
 {
-	VM& _vm;
-
 public:
-
-	VmGcRoot(VM& vm)
-		:
-		_vm(vm)
-	{
-	}
-
+	VmGcRoot(VM& vm) : _vm(vm) {}
 	virtual void markReachableResources() const;
+
+private:
+    VM& _vm;
 };
 
 /// The virtual machine
@@ -84,23 +85,20 @@ public:
 /// For the moment, it will be a singleton, providing one-time
 /// initialization.
 ///
-class DSOEXPORT VM : boost::noncopyable {
+class DSOEXPORT VM : boost::noncopyable
+{
 
 public:
 
 	typedef as_value (*as_c_function_ptr)(const fn_call& fn);
 
-	SafeStack<as_value>& getStack()
-	{
-		return _stack;
-	}
-
-	CallStack& getCallStack()
-	{
-		return _callStack;
-	}
-
-	/// \brief
+    enum AVMVersion
+    {
+        AVM1,
+        AVM2
+    };
+	
+    /// \brief
 	/// Initialize the virtual machine singleton with the given
 	/// movie definition and return a reference to it.
 	//
@@ -119,6 +117,25 @@ public:
 	///
 	static VM& init(int version, movie_root& root, VirtualClock& clock);
 
+	SafeStack<as_value>& getStack() {
+		return _stack;
+	}
+
+	CallStack& getCallStack() {
+		return _callStack;
+	}
+
+    /// Get the VM clock
+    //
+    /// NOTE: this clock should drive all internal operations
+    /// but maybe accessing it trough VM isn't the best idea.
+    /// TODO: consider making this accessible trough RunResources
+    /// instead.
+    ///
+    VirtualClock& getClock() {
+        return _clock;
+    }
+
 	/// Return true if the singleton VM has been initialized
 	static bool isInitialized();
 
@@ -151,14 +168,29 @@ public:
 	/// Set SWF version of the currently executing code
 	void setSWFVersion(int v);
 
+    /// Get the version of the currently executing VM
+    //
+    /// Note: this value changes according to the current execution context;
+    /// do not use it to determine what VM version a particular SWF uses.
+    AVMVersion getAVMVersion() const {
+        return _avmVersion;
+    }
+
+    /// Set the version of the currently executing VM.
+    void setAVMVersion(AVMVersion v) {
+        _avmVersion = v;
+    }
+
 	/// Get the number of milliseconds since VM was started
 	unsigned long int getTime() const;
 
 	/// Get a reference to the string table used by the VM.
-	string_table& getStringTable() const { return mStringTable; }
+	string_table& getStringTable() const { return _stringTable; }
 
+#ifdef ENABLE_AVM2
 	/// Get a pointer to the machine, if it exists.
-	Machine* getMachine() const { return mMachine; }
+    abc::Machine* getMachine() const { return _machine; }
+#endif
 
 	/// Get version of the player, in a compatible representation
 	//
@@ -217,11 +249,8 @@ public:
     }
 
 	/// Get a pointer to this VM's _global Object
-	as_object* getGlobal() const;
+	Global_as* getGlobal() const;
 
-	/// Get a pointer to this VM's global ClassHierarchy object.
-	ClassHierarchy* getClassHierarchy() const { return mClassHierarchy.get(); }
-	
 	/// Mark all reachable resources (for GC)
 	//
 	/// - root movie / stage (_rootMovie)
@@ -234,8 +263,8 @@ public:
 
 	void registerNative(as_c_function_ptr fun, unsigned int x, unsigned int y);
 
-	/// Return a newly created builtin_function or null
-	builtin_function* getNative(unsigned int x, unsigned int y);
+	/// Return a native function or null
+	NativeFunction* getNative(unsigned int x, unsigned int y) const;
 
 #ifdef GNASH_USE_GC
 	void addStatic(GcResource* res)
@@ -270,8 +299,8 @@ private:
 	/// Stage associated with this VM
 	movie_root& _rootMovie;
 
-	/// The _global ActionScript object
-	boost::intrusive_ptr<as_object> _global;
+	/// The _global ActionScript object for AVM1
+	Global_as* _global;
 
 	/// Target SWF version
 	int _swfversion;
@@ -280,10 +309,11 @@ private:
 	//
 	/// Will be called by the init() function
 	/// 
-	void setGlobal(as_object*);
+	void setGlobal(Global_as*);
 
 #ifdef GNASH_USE_GC
-	/// A vector of static GcResources (tipically used for built-in class constructors)
+	/// A vector of static GcResources (typically used for built-in
+    /// class constructors)
 	//
 	/// The resources in this list will always be marked as reachable
 	///
@@ -296,13 +326,12 @@ private:
 	AsNativeTable _asNativeTable;
 
 	/// Mutable since it should not affect how the VM runs.
-	mutable string_table mStringTable;
-
-	/// Not mutable since changing this changes behavior of the VM.
-	std::auto_ptr<ClassHierarchy> mClassHierarchy;
+	mutable string_table _stringTable;
 
+#ifdef ENABLE_AVM2
 	/// A running execution thread.
-	Machine *mMachine;
+    abc::Machine *_machine;
+#endif
 
 	VirtualClock& _clock;
 
@@ -313,8 +342,63 @@ private:
 	/// Library of SharedObjects. Owned by the VM.
     std::auto_ptr<SharedObjectLibrary> _shLib;
 
+    /// The currently executing machine
+    //
+    /// This is a hack like switching the SWF version, but without this
+    /// there is no way for AS functions to know which version of the
+    /// virtual machine called them. This is necessary e.g. for initializing
+    /// the correct object prototypes etc.
+    AVMVersion _avmVersion;
+
 };
 
+/////////////////////////////////////////////////////////////////////////////
+///
+/// VM ops on as_value.
+///
+/// These are currently used in:
+/// 1. VM (AVM1)
+/// 2. Machine (AVM2)
+/// 3. Gnash's C++ implementation of ActionScript.
+///
+/// Usage 3 replicates VM behaviour, but does not use the VM's stack or other
+/// resources. This can lead to different behaviour, for instance when there
+/// are limits on the stack size. Such cases are probably rare.
+///
+/////////////////////////////////////////////////////////////////////////////
+
+/// Carry out ActionNewAdd
+//
+/// @param op1      The as_value to add to.
+/// @param op2      The as_value to add.
+/// @param vm       The VM executing the operation.
+//
+/// TODO:           Consider whether it would be better to pass something
+///                 other than the VM. But it is a VM operation, so it
+///                 is logically sound.
+void newAdd(as_value& op1, const as_value& op2, VM& vm);
+
+/// Carry out ActionSubtract
+//
+/// @param op1      The as_value to subtract from.
+/// @param op2      The as_value to subtract.
+/// @param vm       The VM executing the operation.
+void subtract(as_value& op1, const as_value& op2, VM& vm);
+
+/// Carry out ActionSubtract
+//
+/// @param op1      The first comparand.
+/// @param op2      The second comparand.
+/// @param vm       The VM executing the operation.
+as_value newLessThan(const as_value& op1, const as_value& op2, VM& vm);
+
+/// Return true if the VM is executing AS3 (ABC bytecode).
+inline bool
+isAS3(VM& vm)
+{
+    return vm.getAVMVersion() == VM::AVM2;
+}
+
 } // namespace gnash
 
 #endif // GNASH_VM_H
