@@ -1,6 +1,6 @@
 // AudioDecoderNellymoser.h: Nellymoser decoding
 // 
-//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -44,11 +44,10 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-#ifndef __AUDIODECODERNELLYMOSER_H__
-#define __AUDIODECODERNELLYMOSER_H__
+#ifndef GNASH_AUDIODECODERNELLYMOSER_H
+#define GNASH_AUDIODECODERNELLYMOSER_H
 
-#include "log.h"
-#include "AudioDecoder.h"
+#include "AudioDecoder.h" // for inheritance
 
 #define NELLY_BLOCK_LEN 64
 #define NELLY_HEADER_BITS 116
@@ -59,44 +58,83 @@
 #define NELLY_BASE_OFF 4228
 #define NELLY_BASE_SHIFT 19
 
+// Forward declarations
+namespace gnash {
+    namespace media {
+        class AudioInfo;
+        class SoundInfo;
+    }
+}
 
 
 typedef struct nelly_handle_struct {
-	float state[64];
+    float state[64];
 } nelly_handle;
 
 namespace gnash {
 namespace media {
 
 /// Audio decoding using internal Nellymoser decoder.
+//
+/// TODO: as ffmpeg now has Nellymoser support (maintained
+/// by people who know what they're doing, I hope), do
+/// we need this?
 class AudioDecoderNellymoser : public AudioDecoder {
 
 public:
-	AudioDecoderNellymoser();
-	~AudioDecoderNellymoser();
 
-	bool setup(AudioInfo* info);
-	bool setup(SoundInfo* info);
+    /// This is needed by gstreamer still. TODO: drop.
+    AudioDecoderNellymoser();
 
-	boost::uint8_t* decode(boost::uint8_t* input, boost::uint32_t inputSize, boost::uint32_t& outputSize, boost::uint32_t& decodedBytes, bool parse);
-	
-	/// @return a new[]-allocated pointer to decoded data in floats.
-	float* decode(boost::uint8_t* in_buf, boost::uint32_t inputSize, boost::uint32_t* outputSize);
+    /// @param info
+    ///     AudioInfo class with all the info needed to decode
+    ///     the sound correctly. Throws a MediaException on fatal
+    ///     error.
+    ///
+    /// @throws MediaException on failure
+    ///
+    AudioDecoderNellymoser(const AudioInfo& info);
 
+    /// @param info
+    ///     SoundInfo class with all the info needed to decode
+    ///     the sound correctly. Throws a MediaException on fatal
+    ///      error.
+    /// 
+    /// @deprecated use the AudioInfo based constructor
+    ///
+    /// @throws MediaException on failure
+    ///
+    AudioDecoderNellymoser(const SoundInfo& info);
+
+    ~AudioDecoderNellymoser();
+
+    // See dox in AudioDecoder.h
+    boost::uint8_t* decode(const boost::uint8_t* input,
+        boost::uint32_t inputSize, boost::uint32_t& outputSize,
+        boost::uint32_t& decodedBytes, bool parse);
+    
 private:
 
-	// The handle used by the decoder
-	nelly_handle* _nh;
+    /// @return a new[]-allocated pointer to decoded data in floats.
+    float* decode(const boost::uint8_t* in_buf, boost::uint32_t inputSize,
+            boost::uint32_t* outputSize);
 
-	// samplerate
-	boost::uint16_t _sampleRate;
 
-	// stereo
-	bool _stereo;
+    void setup(const AudioInfo& info);
+    void setup(const SoundInfo& info);
+
+    // The handle used by the decoder
+    nelly_handle* _nh;
+
+    // samplerate
+    boost::uint16_t _sampleRate;
+
+    // stereo
+    bool _stereo;
 };
-	
+    
 } // gnash.media namespace 
 } // gnash namespace
 
 #endif // __AUDIODECODERNELLYMOSER_H__
-	
+    
