@@ -1,6 +1,6 @@
 // MediaParserFfmpeg.cpp: FFMPG media parsers, for Gnash
 //
-//   Copyright (C) 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ namespace {
 	// Used to calculate a decimal value from a ffmpeg fraction
 	inline double as_double(AVRational time)
 	{
-		return time.num / (double) time.den;
+		return time.num / static_cast<double>(time.den);
 	}
 
 } // anonymous namespace
@@ -122,14 +122,18 @@ MediaParserFfmpeg::seek(boost::uint32_t& pos)
 			return 0; // ??
 		}
 
-		newtime = timebase * (double)_formatCtx->streams[_videoStreamIndex]->cur_dts;
+		newtime = timebase * static_cast<double>(_formatCtx->streams[_videoStreamIndex]->cur_dts);
 	}
 
 	//av_free_packet( &Packet );
 	av_seek_frame(_formatCtx, _videoStreamIndex, newpos, 0);
 
 	newtime = static_cast<boost::int32_t>(newtime / 1000.0);
-	return newtime;
+	log_debug("Seek requested to time %d triggered seek to key frame at "
+            "time %d", pos, newtime);
+	pos = newtime;
+
+    return true;
 }
 
 bool
@@ -200,7 +204,7 @@ MediaParserFfmpeg::parseAudioFrame(AVPacket& packet)
         dts=0;
     }
 	boost::uint64_t timestamp = static_cast<boost::uint64_t>(dts * as_double(_audioStream->time_base) * 1000.0); 
-    log_debug("On getting audio frame with timestamp %d, duration is %d", timestamp, _audioStream->duration);
+    //log_debug("On getting audio frame with timestamp %d, duration is %d", timestamp, _audioStream->duration);
 
 	std::auto_ptr<EncodedAudioFrame> frame ( new EncodedAudioFrame );
 
@@ -383,7 +387,7 @@ MediaParserFfmpeg::initializeParser()
 	if ( _formatCtx->album[0] )     log_debug(_("  Album:'%s'"), _formatCtx->album);
 
 	// Find first audio and video stream
-	for (unsigned int i = 0; i < (unsigned)_formatCtx->nb_streams; i++)
+	for (unsigned int i = 0; i < static_cast<unsigned int>(_formatCtx->nb_streams); i++)
 	{
 		AVStream* stream = _formatCtx->streams[i];
 		if ( ! stream ) 
