@@ -1,6 +1,6 @@
 // VideoDecoderFfmpeg.h: Video decoding using the FFMPEG library.
 // 
-//     Copyright (C) 2007, 2008 Free Software Foundation, Inc.
+//     Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,14 +16,9 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA    02110-1301    USA
 
-
 #ifndef GNASH_VIDEODECODERFFMPEG_H
 #define GNASH_VIDEODECODERFFMPEG_H
 
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
-
 #include <memory>
 #include "dsodefs.h" //For DSOEXPORT
 #include "log.h"
@@ -34,56 +29,64 @@
 
 namespace gnash {
 namespace media {
+namespace ffmpeg {
 
 
-/// Forward declarations
+// Forward declarations
 class CodecContextWrapper;
 #ifdef HAVE_SWSCALE_H
 class SwsContextWrapper;
 #endif
 
 
+/// FFMPEG based VideoDecoder
 class VideoDecoderFfmpeg : public VideoDecoder {
     
 public:
 
     DSOEXPORT VideoDecoderFfmpeg(videoCodecType format, int width, int height);
 
-    DSOEXPORT VideoDecoderFfmpeg(VideoInfo& info);
+    DSOEXPORT VideoDecoderFfmpeg(const VideoInfo& info);
 
     DSOEXPORT ~VideoDecoderFfmpeg();
     
     void push(const EncodedVideoFrame& buffer);
 
-    std::auto_ptr<image::ImageBase> pop();
+    std::auto_ptr<GnashImage> pop();
     
     bool peek();
+
+    int width() const;
+
+    int height() const;
     
+private:
     
+    /// Convert FLASH codec id to FFMPEG codec id
+    //
+    /// @return CODEC_ID_NONE for unsupported flash codecs
+    ///
+    static CodecID flashToFfmpegCodec(videoCodecType format);
+
     /// \brief converts an video frame from (almost) any type to RGB24.
     ///
     /// @param srcCtx The source context that was used to decode srcFrame.
     /// @param srcFrame the source frame to be converted.
     /// @return an AVPicture containing the converted image. Please be advised
-    ///                 that the RGB data pointer is stored in AVPicture::data[0]. The
-    ///                 caller owns that pointer, which must be freed with delete [].
-    ///                 It is advised to wrap the pointer in a boost::scoped_array.
-    ///                 If conversion fails, AVPicture::data[0] will be NULL.
-    std::auto_ptr<image::ImageBase> frameToImage(AVCodecContext* srcCtx, const AVFrame& srcFrame);
+    ///         that the RGB data pointer is stored in AVPicture::data[0]. The
+    ///         caller owns that pointer, which must be freed with delete [].
+    ///         It is advised to wrap the pointer in a boost::scoped_array.
+    ///         If conversion fails, AVPicture::data[0] will be NULL.
+    std::auto_ptr<GnashImage> frameToImage(AVCodecContext* srcCtx,
+            const AVFrame& srcFrame);
 
-    /// Convert FLASH codec id to FFMPEG codec id
-    //
-    /// @return CODEC_ID_NONE for unsupported flash codecs
-    ///
-    DSOEXPORT static enum CodecID flashToFfmpegCodec(videoCodecType format);
+    void init(enum CodecID format, int width, int height,
+            boost::uint8_t* extradata=0, int extradataSize=0);
 
-private:
+    std::auto_ptr<GnashImage> decode(const boost::uint8_t* input,
+            boost::uint32_t input_size);
 
-    void init(enum CodecID format, int width, int height, boost::uint8_t* extradata=0, int extradataSize=0);
-
-    std::auto_ptr<image::ImageBase> decode(const boost::uint8_t* input, boost::uint32_t input_size);
-
-    std::auto_ptr<image::ImageBase> decode(const EncodedVideoFrame* vf)
+    std::auto_ptr<GnashImage> decode(const EncodedVideoFrame* vf)
     {
     	return decode(vf->data(), vf->dataSize());
     }
@@ -104,6 +107,7 @@ private:
     std::vector<const EncodedVideoFrame*> _video_frames;
 };
     
+} // gnash.media.ffmpeg namespace 
 } // gnash.media namespace 
 } // gnash namespace
 
