@@ -1,6 +1,6 @@
 // MediaParserGst.h: gstreamer media parsers, for Gnash
 // 
-//   Copyright (C) 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,13 +16,8 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-
-#ifndef __MEDIAPARSER_GST_H__
-#define __MEDIAPARSER_GST_H__
-
-#ifdef HAVE_CONFIG_H
-#include "gnashconfig.h"
-#endif
+#ifndef GNASH_MEDIAPARSER_GST_H
+#define GNASH_MEDIAPARSER_GST_H
 
 #include "MediaParser.h" // for inheritance
 
@@ -45,7 +40,8 @@ namespace media {
 namespace gst {
 
 /// Class to hold extra info found in any stream by the parser.
-struct ExtraInfoGst : public AudioInfo::ExtraInfo, VideoInfo::ExtraInfo, boost::noncopyable
+struct ExtraInfoGst : public AudioInfo::ExtraInfo, VideoInfo::ExtraInfo,
+	boost::noncopyable
 {
     ExtraInfoGst(GstCaps* gstcaps)
     :
