@@ -1,9 +1,10 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,14 +16,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
 // Written by Koos Vriezen <koos ! vriezen ? xs4all ! nl>
 
 #ifdef KDE_USE_FINAL
 #undef Always
 #endif
 
-#include <config.h>
+#include "gnashconfig.h"
 #include <cassert>
 #include <qcstring.h>
 #include <qpopupmenu.h>
@@ -132,7 +132,8 @@ KDE_NO_CDTOR_EXPORT KlashPart::KlashPart (QWidget * wparent, const char *wname,
                 m_autostart = value.toInt (&ok);
                 if (!ok)
                     m_autostart = (value.lower () == "false");
-	    }
+            }
+            m_args.push_back(name + QChar('=') + value);
         }
     }
     KParts::Part::setWidget (new KlashView (wparent));
@@ -160,8 +161,7 @@ KDE_NO_EXPORT void KlashPart::play ()
     QString procname;
     char *gnash_env = getenv("KLASH_PLAYER");
     if (!gnash_env) {
-      procname = GNASHBINDIR;
-      procname += "/klash";
+      procname = GNASHBINDIR "/kde-gnash";
     } else {
       procname = gnash_env;
     }
@@ -172,17 +172,29 @@ KDE_NO_EXPORT void KlashPart::play ()
     m_process = new KProcess;
     m_process->setUseShell (true);
     m_process->setEnvironment (QString::fromLatin1 ("SESSION_MANAGER"), QString::fromLatin1 (""));
-    QString dim;
+
+    QString cmd = procname + QString(" -x ") +
+                             QString::number(static_cast<KlashView*>(widget())->embedId());
+
     if (m_width > 0 && m_height > 0)
-        dim = QString ("-j ") + QString::number (m_width) +
-            QString (" -k ") + QString::number (m_height);
-    // TODO: check KLASH_PLAYER env ?
-    QString cmd = procname + QString (" -x ") +
-        QString::number (static_cast <KlashView *> (widget ())->embedId()) +
-        QChar (' ') + dim +
-        QChar (' ') + KProcess::quote (m_src_url);
+        cmd += QString(" -j ") + QString::number(m_width) +
+               QString(" -k ") + QString::number(m_height);
+
+    QString url = m_url.url();
+    if (!url.isEmpty())
+        cmd += QString(" -u ") + KProcess::quote(url);
+    url = m_docbase.url();
+    if (!url.isEmpty())
+        cmd += QString(" -U ") + KProcess::quote(url);
+
+    for (QStringList::const_iterator it=m_args.begin(), end=m_args.end();it != end; ++it)
+        cmd += QString(" -P ") + KProcess::quote(*it);
+
+    cmd += QString (" ") + KProcess::quote(m_src_url);
+
     kdDebug () << cmd << endl;
     *m_process << cmd;
+
     connect (m_process, SIGNAL (processExited (KProcess *)),
             this, SLOT (processStopped (KProcess *)));
     m_process->start (KProcess::NotifyOnExit, KProcess::NoCommunication);
@@ -211,7 +223,7 @@ KDE_NO_EXPORT void KlashPart::stop () {
             m_process->wait(2);
         }
         delete m_process;
-        m_process = 0l;
+        m_process = 0L;
     }
 }
 
