@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "Movie.h" 
 #include "ManualClock.h" // for composition
 #include "RunResources.h" // For initialization.
+#include "movie_root.h"
 
 #include <memory> // for auto_ptr
 #include <string> 
@@ -154,8 +155,10 @@ public:
 			int depth);
 
 	/// Get the topmost sprite instance of this movie
+    //
+    /// We const_cast this because we don't care.
 	gnash::MovieClip* getRootMovie() {
-		return _movie;
+		return const_cast<Movie*>(&_movie_root->getRootMovie());
 	}
 
 	/// Notify mouse pointer movement to the given coordinate
@@ -317,9 +320,7 @@ private:
 
 	gnash::movie_root* _movie_root;
 
-	gnash::movie_definition* _movie_def;
-
-	gnash::Movie* _movie;
+	boost::intrusive_ptr<gnash::movie_definition> _movie_def;
 
     boost::shared_ptr<sound::sound_handler> _sound_handler;
 
