@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,9 +40,21 @@
 
 #include "check.h"
 
+#define check_strictly_equals(a, b) check(a.strictly_equals(b))
+
 using namespace std;
 using namespace gnash;
 
+bool
+getVal(PropertyList& p, string_table::key k, as_value& val, as_object& obj)
+{
+    if (Property* prop = p.getProperty(k)) {
+        val = prop->getValue(obj);
+        return true;
+    }
+    return false;
+}
+
 int
 main(int /*argc*/, char** /*argv*/)
 {
@@ -68,14 +81,14 @@ main(int /*argc*/, char** /*argv*/)
 
     movie_root root(*md5, clock, runResources);
 
-    root.setRootMovie( md5->createMovie() );
+    root.init(md5.get(), MovieClip::MovieVariables());
 
     VM& vm = root.getVM();
 
 	log_debug("VM version %d", vm.getSWFVersion());
 
 	as_object obj;
-	PropertyList props(vm);
+	PropertyList props(obj);
 
 	as_value val("value");
 	as_value val2("value2");
@@ -87,32 +100,32 @@ main(int /*argc*/, char** /*argv*/)
 	if (vm.getSWFVersion() > 6) // SWF 7 or higher is case sensitive.
 	{
 		check_equals(props.size(), 0);
-		check ( props.setValue(st.find("Var0"), val, obj) );
+		check ( props.setValue(st.find("Var0"), val) );
 		check_equals(props.size(), 1);
 
-		check ( props.getValue(st.find("Var0"), ret, obj) );
-		check_equals ( ret, val );
+		check (getVal(props, st.find("Var0"), ret, obj) );
+		check_strictly_equals ( ret, val );
 
 		// search should be case-sensitive
-		check ( ! props.getValue(st.find("var0"), ret, obj) );
+		check (!getVal(props, st.find("var0"), ret, obj) );
 
 		// new value overrides existing value
-		check ( props.setValue(st.find("Var0"), val2, obj) );
+		check ( props.setValue(st.find("Var0"), val2) );
 		check_equals(props.size(), 1);
-		check ( props.getValue(st.find("Var0"), ret, obj) );
-		check_equals ( ret, val2 );
+		check (getVal(props, st.find("Var0"), ret, obj) );
+		check_strictly_equals ( ret, val2 );
 
 		// case-sensitive setting value doesn't overrides existing value
-		check ( props.setValue(st.find("var0"), val3, obj) );
+		check ( props.setValue(st.find("var0"), val3) );
 		check_equals(props.size(), 2);
-		check ( ! props.getValue(st.find("vAr0"), ret, obj) );
+		check (!getVal(props, st.find("vAr0"), ret, obj) );
 
 		// Now add some new labels
-		check ( props.setValue(st.find("var1"), val, obj) );
+		check ( props.setValue(st.find("var1"), val) );
 		check_equals(props.size(), 3);
-		check ( props.setValue(st.find("var2"), val, obj) );
+		check ( props.setValue(st.find("var2"), val) );
 		check_equals(props.size(), 4);
-		check ( props.setValue(st.find("var3"), val, obj) );
+		check ( props.setValue(st.find("var3"), val) );
 		check_equals(props.size(), 5);
 
 		// Test deletion of properties
@@ -133,50 +146,41 @@ main(int /*argc*/, char** /*argv*/)
 		check_equals(delpair.second, false); // property was NOT deleted
 		check_equals(props.size(), 4);
 
-        PropertyList::SortedPropertyList vals;
-		props.enumerateKeyValue(obj, vals);
-		check_equals( vals.size(), 4 );
-		check_equals( vals[0].first, "var0");
-		check_equals( vals[0].second, "value3");
-		check_equals( vals[1].first, "Var0");
-		check_equals( vals[1].second, "value2");
-		check_equals( vals[2].first, "var1");
-		check_equals( vals[2].second, "value");
-
 	}
 	else
 	{
+
 		// Below SWF or is not case sensitive.
 
 		check_equals(props.size(), 0);
-		check ( props.setValue(st.find("Var0"), val, obj) );
+		check ( props.setValue(st.find("Var0"), val) );
 		check_equals(props.size(), 1);
 
-		check ( props.getValue(st.find("Var0"), ret, obj) );
-		check_equals ( ret, val );
+		check (getVal(props, st.find("Var0"), ret, obj) );
+		check_strictly_equals ( ret, val );
 
 		// search should be case-insensitive
-		check ( props.getValue(st.find("var0"), ret, obj) );
-		check_equals ( ret, val );
+		check (getVal(props, st.find("var0"), ret, obj) );
+		check_strictly_equals ( ret, val );
 
 		// new value overrides existing value
-		check ( props.setValue(st.find("Var0"), val2, obj) );
+		check ( props.setValue(st.find("Var0"), val2) );
 		check_equals(props.size(), 1);
-		check ( props.getValue(st.find("Var0"), ret, obj) );
-		check_equals ( ret, val2 );
+		check (getVal(props, st.find("Var0"), ret, obj) );
+		check_strictly_equals ( ret, val2 );
 
 		// case-insensitive setting value should override existing value
-		check ( props.setValue(st.find("var0"), val3, obj) );
+		check ( props.setValue(st.find("var0"), val3) );
 		check_equals(props.size(), 1);
-		check ( props.getValue(st.find("vAr0"), ret, obj) );
-		check_equals ( ret, val3 );
+		check (getVal(props, st.find("vAr0"), ret, obj) );
+		check_strictly_equals ( ret, val3 );
 
 		// Now add some new labels
-		check ( props.setValue(st.find("var1"), val, obj) );
+		check ( props.setValue(st.find("var1"), val) );
 		check_equals(props.size(), 2);
-		check ( props.setValue(st.find("var2"), val, obj) );
+		check ( props.setValue(st.find("var2"), val) );
 		check_equals(props.size(), 3);
-		check ( props.setValue(st.find("var3"), val, obj) );
+		check ( props.setValue(st.find("var3"), val) );
 		check_equals(props.size(), 4);
 
 		// Test deletion of properties
@@ -197,16 +201,6 @@ main(int /*argc*/, char** /*argv*/)
 		check_equals(delpair.second, false); // property was NOT deleted
 		check_equals(props.size(), 3);
 
-        PropertyList::SortedPropertyList vals;
-		props.enumerateKeyValue(obj, vals);
-		check_equals( vals.size(), 3 );
-		check_equals( vals[0].first, "var2");
-		check_equals( vals[0].second, "value");
-		check_equals( vals[1].first, "var1");
-		check_equals( vals[1].second, "value");
-		check_equals( vals[2].first, "Var0");
-		check_equals( vals[2].second, "value3");
-
 	}
 }
 
