@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- *   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2009, 2010 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
  * Tests are triggered by events, in particular:
  * - Test for _target and _name referring to button's parent.
  * - Test for bounds of buttons being the union of all active state
- *   characters' bounds.
+ *   DisplayObjects' bounds.
  *
  * Note that you need to play with your mouse on the button for the tests
  * to be run, and that there's currently no END OF TEST condition.
@@ -134,12 +134,13 @@ add_button(SWFMovie mo)
 			"var myName = ''+this;"
 			"if ( _root.buttonChild[myDepth] == undefined ) _root.buttonChild[myDepth] = {nam:myName,exe:1,uld:0};"
 			"else _root.buttonChild[myDepth]['exe']++;"
-			 "_root.note('Actions in frame0 of '+this+' at depth '+myDepth+' executed.');" 
+			 //"_root.note('Actions in frame0 of '+this+' at depth '+myDepth+' executed.');" 
 			"this.onUnload = function() {"
 			"	var myDepth = -(getDepth()+32769-16383);"
-			"	_root.note(''+this+' at depth '+myDepth+' unloaded.');"
+			//"	_root.note(''+this+' at depth '+myDepth+' unloaded.');"
 			"	_root.buttonChild[myDepth]['uld']++;"
 			"};"
+            "for (i in _level0.square1.button) { trace (i); };"
 			//"_root.note('buttonChilds:'+dumpObj(_root.buttonChild));"
 		));
 		SWFMovieClip_nextFrame(ermc);
@@ -154,47 +155,47 @@ add_button(SWFMovie mo)
 	sh4 = make_fill_square(0, 0, 40, 40, 255, 255, 0, 255, 255, 0);
 	sh4a = make_fill_square(30, 30, 5, 5, 128, 128, 0, 128, 128, 0);
 
-	/* Higher depth character is intentionally added before lower depth one */
+	/* Higher depth DisplayObject is intentionally added before lower depth one */
 	br = SWFButton_addCharacter(bu, (SWFCharacter)sh1a, SWFBUTTON_HIT);
 	SWFButtonRecord_setDepth(br, 2);
 	br = SWFButton_addCharacter(bu, (SWFCharacter)sh1, SWFBUTTON_HIT);
 	SWFButtonRecord_setDepth(br, 1);
 
-	/* Higher depth character is intentionally added before lower depth one */
+	/* Higher depth DisplayObject is intentionally added before lower depth one */
 	br = SWFButton_addCharacter(bu, (SWFCharacter)sh2a, SWFBUTTON_UP );
 	SWFButtonRecord_setDepth(br, 2);
 	br = SWFButton_addCharacter(bu, (SWFCharacter)sh2, SWFBUTTON_UP );
 	SWFButtonRecord_setDepth(br, 1);
 
-	/* Higher depth character is intentionally added before lower depth one */
+	/* Higher depth DisplayObject is intentionally added before lower depth one */
 	br = SWFButton_addCharacter(bu, (SWFCharacter)sh3a, SWFBUTTON_DOWN );
 	SWFButtonRecord_setDepth(br, 2);
 	br = SWFButton_addCharacter(bu, (SWFCharacter)sh3, SWFBUTTON_DOWN );
 	SWFButtonRecord_setDepth(br, 1);
 
-	/* Higher depth character is intentionally added before lower depth one */
+	/* Higher depth DisplayObject is intentionally added before lower depth one */
 	br = SWFButton_addCharacter(bu, (SWFCharacter)sh4a, SWFBUTTON_OVER );
 	SWFButtonRecord_setDepth(br, 2);
 	br = SWFButton_addCharacter(bu, (SWFCharacter)sh4, SWFBUTTON_OVER );
 	SWFButtonRecord_setDepth(br, 1);
 
-	/* Add events reported character in all states at depth 10 */
+	/* Add events reported DisplayObject in all states at depth 10 */
 	br = SWFButton_addCharacter(bu, (SWFCharacter)ermc, SWFBUTTON_HIT|SWFBUTTON_DOWN|SWFBUTTON_OVER|SWFBUTTON_UP);
 	SWFButtonRecord_setDepth(br, 10);
 
-	/* Add events reported character just HIT state at depth 11 */
+	/* Add events reported DisplayObject just HIT state at depth 11 */
 	br = SWFButton_addCharacter(bu, (SWFCharacter)ermc, SWFBUTTON_HIT);
 	SWFButtonRecord_setDepth(br, 11);
 
-	/* Add events reported character just UP state at depth 12 */
+	/* Add events reported DisplayObject just UP state at depth 12 */
 	br = SWFButton_addCharacter(bu, (SWFCharacter)ermc, SWFBUTTON_UP);
 	SWFButtonRecord_setDepth(br, 12);
 
-	/* Add events reported character just OVER state at depth 13 */
+	/* Add events reported DisplayObject just OVER state at depth 13 */
 	br = SWFButton_addCharacter(bu, (SWFCharacter)ermc, SWFBUTTON_OVER);
 	SWFButtonRecord_setDepth(br, 13);
 
-	/* Add events reported character just DOWN state at depth 14 */
+	/* Add events reported DisplayObject just DOWN state at depth 14 */
 	br = SWFButton_addCharacter(bu, (SWFCharacter)ermc, SWFBUTTON_DOWN);
 	SWFButtonRecord_setDepth(br, 14);
 
@@ -541,7 +542,7 @@ main(int argc, char **argv)
 	/*****************************************************
 	 *
 	 * On second frame, check construction of the button
-	 * character states and give instructions to proceed
+	 * DisplayObject states and give instructions to proceed
 	 *
 	 *
 	 *****************************************************/
@@ -731,9 +732,17 @@ main(int argc, char **argv)
 	{
 
 		add_actions(mo,
+            "check_equals(square1.button.enabled, true);"
+			"square1.button.enabled = 6;"
+            "check_equals(square1.button.enabled, 6);"
+			"square1.button.enabled = 'string';"
+            "check_equals(square1.button.enabled, 'string');"
+            "square1.button._visible = false;"
+            "check_equals(square1.button.enabled, 'string');"
+            "square1.button._visible = true;"
 			"square1.button.enabled = false;"
 			"stop();"
-			"_root.totals(156);"
+			"_root.totals(160);"
 			"_root.note('-- Button disabled, try playing with it, nothing should happen --');"
 		);
 		SWFMovie_nextFrame(mo); /* showFrame */
