@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -60,7 +61,7 @@ extern char *optarg;
 #include <cerrno>
 
 #ifdef ENABLE_NLS
-# include <locale>
+# include <clocale>
 #endif
 
 #include "log.h"
@@ -77,10 +78,10 @@ using namespace gnash;
 namespace boost
 {
 
-	void throw_exception(std::exception const & e)
-	{
-		std::abort();
-	}
+void throw_exception(std::exception const & e)
+{
+    std::abort();
+}
 }
 #endif
 
@@ -114,8 +115,8 @@ main(int argc, char *argv[])
 {
     int                   c;
     bool                  listfiles  = false;
-    bool		  sysv = false;
-    bool		  convert = false;
+    bool          sysv = false;
+    bool          convert = false;
     int                   size  = 0;
     string                filespec, realname, tmpname;
     vector<const char *>  dirlist;
@@ -129,45 +130,45 @@ main(int argc, char *argv[])
     // scan for the two main standard GNU options
     for (c = 0; c < argc; c++) {
       if (strcmp("--help", argv[c]) == 0) {
-        usage();
-        exit(0);
+      usage();
+      exit(EXIT_SUCCESS);
       }
       if (strcmp("--version", argv[c]) == 0) {
-        printf (_("Gnash dumpshm version: %s, Gnash version: %s\n"),
-		   DUMPSHM_VERSION, VERSION);
-        exit(0);
+      printf (_("Gnash dumpshm version: %s, Gnash version: %s\n"),
+          DUMPSHM_VERSION, VERSION);
+      exit(EXIT_SUCCESS);
       }
     }
- 
+    
     while ((c = getopt (argc, argv, "hircv")) != -1) {
         switch (c) {
           case 'h':
-            usage ();
-            break;
-            
+          usage ();
+          break;
+          
           case 'r':
-            sysv = true;
-            convert = false;
-            break;
-            
+          sysv = true;
+          convert = false;
+          break;
+          
           case 'c':
-            sysv = true;
-            convert = true;
-            break;
-            
+          sysv = true;
+          convert = true;
+          break;
+          
           case 'i':
-            sysv = true;
-            listfiles = true;
-            break;
-            
+          sysv = true;
+          listfiles = true;
+          break;
+          
           case 'v':
-	      // turn on verbosity for the libraries
-	      dbglogfile.setVerbosity();
-            break;
-
+          // turn on verbosity for the libraries
+          dbglogfile.setVerbosity();
+          break;
+          
           default:
-            usage ();
-            break;
+          usage ();
+          break;
         }
     }
     
@@ -176,31 +177,31 @@ main(int argc, char *argv[])
     // print the  usage message.
     if (argc < 2) {
         usage();
-        exit(0);
+        exit(EXIT_SUCCESS);
     }
     
 #if defined(USE_SYSV_SHM) && defined(HAVE_IPC_INFO)
     // Just list the shared memory segments
     if (listfiles && sysv) {
-	list_lcs();
-        exit(0);
+    list_lcs();
+        exit(EXIT_SUCCESS);
     }
 #endif
     
     if (optind <= argc - 1) {
-	if (*argv[optind] == '-') {
-	    filespec = '-';
-	}
+    if (*argv[optind] == '-') {
+        filespec = '-';
+    }
     }
     
     if (sysv) {
-	if (filespec == "-") {
-	    dump_shm(convert, true);
-	} else {
-	    dump_shm(convert, false);
-	}
-	
-	    exit(0);
+    if (filespec == "-") {
+        dump_shm(convert, true);
+    } else {
+        dump_shm(convert, false);
+    }
+    
+        exit(EXIT_SUCCESS);
     }    
     
     if (size == 0) {
@@ -210,12 +211,11 @@ main(int argc, char *argv[])
     // get the file name from the command line
     if (optind < argc) {
         filespec = argv[optind];
-	if (!convert) {
-	    cout << "Will use \"" << filespec << "\" for memory segment file"
-		 << endl;
-	}
+    if (!convert) {
+        log_debug(_("Will use \"%s\" for memory segment file"), filespec);
     }
-
+    }
+    
 }
 
 // Dump the older style SYS V shared memory segments
@@ -228,21 +228,21 @@ dump_shm(bool convert, bool out)
     key_t key = rcfile.getLCShmKey();
 
     if (key == 0) {
-	cerr << "No LcShmKey set in ~/.gnashrc, trying to find it ourselves" << endl;
+    log_debug(_("No LcShmKey set in ~/.gnashrc, trying to find it ourselves"));
 #if defined(USE_SYSV_SHM) && defined(HAVE_IPC_INFO)
-	key = list_lcs();
+    key = list_lcs();
 #endif
     }
     
-    int size = 64528;			// 1007 bytes less than unsigned
+    int size = 64528;            // 1007 bytes less than unsigned
 
     if (key == 0) {
-	cerr << "No shared memory segments found!" << endl;
-	return;
+        log_debug(_("No shared memory segments found!"));
+        return;
     }
     if (dbglogfile.getVerbosity()) {
-	cout << "Existing SHM Key is: 0x" << hex << key
-	     << ", Size is: " << dec << size << endl;
+        log_debug(_("Existing SHM Key is: %s, Size is: %s"),
+                boost::io::group(hex, showbase, key), size);
     }
     
     amf::LcShm lc;
@@ -252,43 +252,43 @@ dump_shm(bool convert, bool out)
     // If the -c convert options was specified, dump the memory segment to disk.
     // This makes it easy to store them as well as to examine them in great detail.
     if (convert) {
-	int fd = open("segment.raw",O_WRONLY|O_CREAT, S_IRWXU);
-	if (fd == -1) {
-	    perror("open");
-	}
-	cout << "Writing memory segment to disk: \"segment.raw\"" << endl;
-	shmaddr = lc.getAddr();
-	write(fd, shmaddr, size);
-	if (out) {
+        int fd = open("segment.raw",O_WRONLY|O_CREAT, S_IRWXU);
+        if (fd == -1) {
+            perror("open");
+        }
+        log_debug(_("Writing memory segment to disk: \"segment.raw\""));
+        shmaddr = lc.getAddr();
+        write(fd, shmaddr, size);
+        if (out) {
 #if 0
-	    log_debug(_("The data is: 0x%s"), hexify((uint8_t *)shmaddr, size, false));
+            log_debug(_("The data is: 0x%s"), hexify((uint8_t *)shmaddr, size, false));
 #endif
-	}
-	
-	close(fd);
+        }
+        
+        close(fd);
     }
     
-    exit (0);
+    exit (EXIT_SUCCESS);
 }
-
+    
 #if defined(USE_SYSV_SHM) && defined(HAVE_IPC_INFO)
 key_t
 list_lcs()
 {
     int maxid, shmid, id;
     struct shmid_ds shmseg;
-
+    
 // #ifdef USE_POSIX_SHM
 //     if (library_dir != NULL) {
-// 	for (i=0; entry>0; i++) {
-// 	    entry = readdir(library_dir);
-// 	    if (entry != NULL) {
+//     for (i=0; entry>0; i++) {
+//         entry = readdir(library_dir);
+//         if (entry != NULL) {
 //                     cout << "Found segment: " << entry->d_name << endl;
 //                 }
 //             }
 //         } else {
-// 	cout << _("Sorry, we can only list the files on systems with"
-// 		  " disk based shared memory") << endl;
+//     cout << _("Sorry, we can only list the files on systems with"
+//           " disk based shared memory") << endl;
 //     }
 // #eendif
     
@@ -301,39 +301,39 @@ list_lcs()
     struct shmid_ds shm_info;
     maxid = shmctl(0, SHM_INFO, &shm_info);
     if (maxid < 0) {
-	cerr << "kernel not configured for shared memory";
-	return 0;
+    log_debug(_("kernel not configured for shared memory"));
+    return 0;
     }
     
 //    struct shminfo shminfo;
     if ((shmctl(0, IPC_INFO, &shm_info)) < 0) {
-	return 0;
+        return 0;
     }
     for (id = 0; id <= maxid; id++) {
-	shmid = shmctl(id, SHM_STAT, &shmseg);
-	if (shmid < 0) {
-	    continue;
-	}
+        shmid = shmctl(id, SHM_STAT, &shmseg);
+        if (shmid < 0) {
+            continue;
+        }
 #ifdef IPC_PERM_KEY
-	if (shmseg.shm_segsz == 64528) {
-	    cout << "Found it! \"set LCShmKey 0x"
-		 << hex << shmseg.shm_perm.IPC_PERM_KEY
-		 << "\" in your ~/.gnashrc" << endl;
-	    cout << "Last changed on: " << ctime(&shmseg.shm_ctime);
-	    cout << "Last attached on: " << ctime(&shmseg.shm_atime);
-	    cout << "Last detached on: " << ctime(&shmseg.shm_dtime);
-	    return shmseg.shm_perm.IPC_PERM_KEY;
-	}
-#endif	// end of IPC_PERM_KEY
+        if (shmseg.shm_segsz == 64528) {
+            log_debug(_("Found it! \"set LCShmKey %s\" in your ~/.gnashrc",
+                        boost::io::group(hex, showbase,
+                            shmseg.shm_perm.IPC_PERM_KEY)));
+            log_debug(_("Last changed on: %s", ctime(&shmseg.shm_ctime)));
+            log_debug(_("Last attached on: %s", ctime(&shmseg.shm_atime)));
+            log_debug(_("Last detached on: %s", ctime(&shmseg.shm_dtime)));
+            return shmseg.shm_perm.IPC_PERM_KEY;
+        }
+#endif    // end of IPC_PERM_KEY
     }
 // #else
 // # error "No supported shared memory type for this platform"
-//#endif	// end of USE_POSIX_SHM
-
+//#endif    // end of USE_POSIX_SHM
+    
     // Didn't find any segments of the right size
     return static_cast<key_t>(0);
 }
-#endif	// end of USE_SYSV_SHM & HAVE_IPC_INFO
+#endif    // end of USE_SYSV_SHM & HAVE_IPC_INFO
 
 /// \brief  Display the command line arguments
 static void
@@ -341,13 +341,13 @@ usage (void)
 {
     cerr << _("This program dumps the internal data of a shared memory segment")
          << endl;
-    cerr << _("Usage: dumpmem [hdsanlif] filename") << endl;
+    cerr << _("Usage: dumpshm [hdsanlif] filename") << endl;
     cerr << _("-h\tHelp") << endl;
     cerr << _("-i\tList segments") << endl;
     cerr << _("-r\tDump SYSV segments") << endl;
     cerr << _("-c\tDump SYSV segments to disk") << endl;
     cerr << _("-v\tVerbose output") << endl;
-    exit (-1);
+    exit (EXIT_FAILURE);
 }
 
 /// \brief Dumps the internal data of the found ShmControl
