@@ -1,6 +1,7 @@
 // processor.cpp:  Flash movie processor (gprocessor command), for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -35,13 +36,12 @@
 #include <ctime>
 
 #ifdef ENABLE_NLS
-#include <locale>
+# include <clocale>
 #endif
 
 #include "MovieFactory.h"
 #include "swf/TagLoadersTable.h"
 #include "swf/DefaultTagLoaders.h"
-#include "gettext.h"
 #include "ClockTime.h"
 #include "gnash.h"
 #include "movie_definition.h"
@@ -116,16 +116,9 @@ gnash::Debugger& debugger = gnash::Debugger::getDefaultInstance();
 #endif
 }
 
-struct movie_data
-{
-    gnash::movie_definition* m_movie;
-    std::string	m_filename;
-};
-
-static boost::intrusive_ptr<gnash::movie_definition> play_movie(
+static bool play_movie(
         const std::string& filename, const RunResources& runResources);
 
-static bool s_do_output = false;
 static bool s_stop_on_errors = true;
 
 // How many time do we allow to hit the end ?
@@ -242,12 +235,12 @@ main(int argc, char *argv[])
     for (c = 0; c < argc; c++) {
       if (strcmp("--help", argv[c]) == 0) {
         usage(argv[0]);
-        exit(0);
+        exit(EXIT_SUCCESS);
       }
       if (strcmp("--version", argv[c]) == 0) {
         printf (_("Gnash gprocessor version: %s, Gnash version: %s\n"),
 		   GPROC_VERSION, VERSION);
-        exit(0);
+        exit(EXIT_SUCCESS);
       }
     }
  
@@ -276,15 +269,12 @@ main(int argc, char *argv[])
         dbglogfile.setVerbosity();
     }
 
-    while ((c = getopt (argc, argv, ":hwvapr:gf:d:n")) != -1) {
+    while ((c = getopt (argc, argv, ":hvapr:gf:d:n")) != -1) {
 	switch (c) {
 	  case 'h':
 	      usage (argv[0]);
               dbglogfile.removeLog();
-	      exit(0);
-	  case 'w':
-	      s_do_output = true;
-	      break;
+	      exit(EXIT_SUCCESS);
 	  case 'v':
 	      dbglogfile.setVerbosity();
 	      log_debug (_("Verbose output turned on"));
@@ -327,11 +317,11 @@ main(int argc, char *argv[])
 	      break;
 	  case ':':
               fprintf(stderr, "Missing argument for switch ``%c''\n", optopt); 
-	      exit(1);
+	      exit(EXIT_FAILURE);
 	  case '?':
 	  default:
               fprintf(stderr, "Unknown switch ``%c''\n", optopt); 
-	      exit(1);
+	      exit(EXIT_FAILURE);
 	}
     }
     
@@ -347,7 +337,7 @@ main(int argc, char *argv[])
 	    std::cerr << "no input files" << std::endl;
 	    usage(argv[0]);
         dbglogfile.removeLog();
-	    exit(1);
+	    exit(EXIT_FAILURE);
     }
 
     if (infiles.size() > 1)
@@ -356,7 +346,7 @@ main(int argc, char *argv[])
         std::cerr << "Multiple input files not supported." << std::endl;
         usage(argv[0]);
         dbglogfile.removeLog();
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
 
@@ -377,8 +367,6 @@ main(int argc, char *argv[])
 
     boost::shared_ptr<StreamProvider> sp(new StreamProvider);
 
-    std::vector<movie_data>	data;
-        
 
     boost::shared_ptr<SWF::TagLoadersTable> loaders(new SWF::TagLoadersTable());
     addDefaultLoaders(*loaders);
@@ -393,23 +381,17 @@ main(int argc, char *argv[])
         runResources.setStreamProvider(sp);
         runResources.setTagLoaders(loaders);
 
-	    boost::intrusive_ptr<gnash::movie_definition> m =
-            play_movie(*i, runResources);
-	    if (!m) {
+	    bool success = play_movie(*i, runResources);
+	    if (!success) {
 	        if (s_stop_on_errors) {
 		    // Fail.
                 std::cerr << "error playing through movie " << *i << std::endl;
-		        std::exit(1);
+		        std::exit(EXIT_FAILURE);
 	        }
         }
 	
-        movie_data	md;
-        md.m_movie = m.get();
-        md.m_filename = *i;
-        data.push_back(md);
     }
     
-    
     return 0;
 }
 
@@ -417,16 +399,13 @@ main(int argc, char *argv[])
 // I.e. run through and render all the frames, even though we are not
 // actually doing any output (our output handlers are disabled).
 //
-// What this does is warm up all the cached data in the movie, so that
-// if we save that data for later, we won't have to tesselate shapes
-// or build font textures again.
-//
-// Return the movie definition.
-boost::intrusive_ptr<gnash::movie_definition>
+bool
 play_movie(const std::string& filename, const RunResources& runResources)
 {
     boost::intrusive_ptr<gnash::movie_definition> md;
 
+    quitrequested = false;
+
     URL url(filename);
 
     try
@@ -462,7 +441,7 @@ play_movie(const std::string& filename, const RunResources& runResources)
     }
     if (md == NULL) {
         std::cerr << "error: can't play movie: "<< filename << std::endl;
-	    std::exit(1);
+	    return false;
     }
 
     float fps = md->get_frame_rate();
@@ -485,14 +464,8 @@ play_movie(const std::string& filename, const RunResources& runResources)
 
     md->completeLoad();
 
-    std::auto_ptr<Movie> mi ( md->createMovie() );
-
-    m.setRootMovie( mi.release() );
-    if ( quitrequested )  // setRootMovie would execute actions in first frame
-    {
-        quitrequested = false;
-        return md;
-    }
+    MovieClip::MovieVariables v;
+    m.init(md.get(), v);
 
     log_debug("iteration, timer: %lu, localDelay: %ld\n",
             cl.elapsed(), localDelay);
@@ -506,7 +479,7 @@ play_movie(const std::string& filename, const RunResources& runResources)
     size_t end_hitcount=0;
     size_t nadvances=0;
     // Run through the movie.
-    for (;;) {
+    while (!quitrequested) {
         // @@ do we also have to run through all sprite frames
         // as well?
         //
@@ -525,7 +498,7 @@ play_movie(const std::string& filename, const RunResources& runResources)
         if ( quitrequested ) 
         {
             quitrequested = false;
-            return md;
+            break;
         }
 
         m.display(); // FIXME: for which reason are we calling display here ??
@@ -602,31 +575,50 @@ play_movie(const std::string& filename, const RunResources& runResources)
                 cl.elapsed(), localDelay);
         gnashSleep(localDelay);
     }
+
+    log_debug("-- Playback completed");
+
+    log_debug("-- Dropping ref of movie_definition");
+
+    // drop reference to movie_definition, to force
+    // destruction when core gnash doesn't need it anymore
+    md = 0;
+
+    log_debug("-- Clearning movie_root");
+
+    // clear movie_root (shouldn't have bad consequences on itself)
+    m.clear();
+
+    log_debug("-- Clearning gnash");
  
     // Clear resources.
+    // Forces run of GC, which in turn may invoke
+    // destuctors of (say) MovieClip which would try
+    // to access the movie_root to unregister self
+    //
+    // This means that movie_root must be available
+    // while gnash::clear() runs
+    // 
     gnash::clear();
-    
-    return md;
+
+    return true;
 }
 
 static void
 usage (const char *name)
 {
     printf(
-	_("gprocessor -- an SWF preprocessor for Gnash.\n"
+	_("gprocessor -- an SWF processor for Gnash.\n"
 	"\n"
 	"usage: %s [options] <file>\n"
 	"\n"
-	"Preprocesses the given SWF movie files.  Optionally write preprocessed shape\n"
-	"and font data to cache files, so the associated SWF files can be loaded\n"
-	"faster.\n"
+	"Process the given SWF movie files.\n"
 	"\n"
         "%s%s%s%s"), name, _(
 	"options:\n"
 	"\n"
 	"  --help(-h)  Print this info.\n"	
 	"  --version   Print the version numbers.\n"	
-	"  -w          Write a .gsc file with preprocessed info, for each input file.\n"	
 	"  -v          Be verbose; i.e. print log messages to stdout\n"
           ),
 #if VERBOSE_PARSE
