@@ -1,5 +1,6 @@
 // 
-//   Copyright (C) 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+//   Foundation, Inc
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,7 +26,7 @@
 #include <cstring>
 
 #ifdef ENABLE_NLS
-# include <locale>
+# include <clocale>
 #endif
 
 extern "C"{
@@ -98,12 +99,12 @@ main(int argc, char *argv[])
     for (c = 0; c < argc; c++) {
       if (strcmp("--help", argv[c]) == 0) {
         usage(cout);
-        exit(0);
+        exit(EXIT_SUCCESS);
       }
       if (strcmp("--version", argv[c]) == 0) {
         printf (_("Gnash soldumper version: %s, Gnash version: %s\n"),
 		   SOLDUMPER_VERSION, VERSION);
-        exit(0);
+        exit(EXIT_SUCCESS);
       }
     }
  
@@ -112,7 +113,7 @@ main(int argc, char *argv[])
         switch (c) {
           case 'h':
             usage(cout);
-	    exit(0);
+	    exit(EXIT_SUCCESS);
             break;
             
 	  case 'v':
@@ -141,7 +142,7 @@ main(int argc, char *argv[])
     // print the  usage message.
     if (argc < 2) {
         usage(cerr);
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     // get the file name from the command line
@@ -182,7 +183,7 @@ main(int argc, char *argv[])
                 }
             }
         }
-        exit(0);
+        exit(EXIT_SUCCESS);
     }
     
     string newspec;
